import { takeEvery, takeLatest, put, call, take, fork, cancel, cancelled, delay } from "redux-saga/effects"
import { get, patch } from '../Common/Utils/AxiosClient'
import Utils, { firebaseToken, firebaseTokenTimer } from '../Common/Utils/Utils'
import { auth, firebase } from '../Common/Firebase/Firebase'
import * as Auth from '../Common/Firebase/Auth'
import Enum from '../Common/Constants/Enum'

import {
  LOAD_USER, LOAD_USER_SUCCESS,
  SAVE_PASSWORD, SAVE_PASSWORD_FAILED, SAVE_PASSWORD_SUCCESS,
  SAVE_PROFILE_PICTURE, SAVE_PROFILE_PICTURE_SUCCESS, SAVE_PROFILE_PICTURE_FAILED,
  SAVE_USER, SAVE_USER_FAILED, SAVE_USER_SUCCESS,
  RECEIVE_NEW_TOKEN,
  LOAD_USER_FAILED,
  CHECK_TOKEN, USER_LOGOUT,
  GET_ACCOUNTS, GET_ACCOUNTS_SUCCESS, GET_ACCOUNTS_FAILURE
} from './AccountActionCreator'

const getNewToken = (user) => {
  return user.getIdToken(true);
}

function* renewToken(user) {
  try {
    while (true) {
      const idToken = yield call(getNewToken, user);
      Utils.setCookie(firebaseToken, idToken, 60 * 60);
      yield put({ type: RECEIVE_NEW_TOKEN });
      const date = new Date().getTime();
      Utils.setCookie(firebaseTokenTimer, date, 60 * 60);
      yield delay(55 * 60 * 1000);
    }
  } finally {
    if (yield cancelled()) {
    }
  }
}

function* handleRenewToken(user) {
  const renewTask = yield fork(renewToken, user);
  yield take(USER_LOGOUT);
  yield cancel(renewTask);
}

function* watchCheckToken({ payload: { user } }) {
  try {
    if (user) {
      const token = Utils.getCookie(firebaseToken);
      if (!token) {
        yield call(handleRenewToken, user);
      } else {
        const tokenTimer = Utils.getCookie(firebaseTokenTimer);
        if (tokenTimer) {
          const current = new Date().getTime();
          if (current - tokenTimer > 55 * 60 * 1000) {
            yield call(handleRenewToken, user);
          } else {
            const timeToRenew = 55 * 60 * 1000 - (current - tokenTimer);
            yield delay(timeToRenew);
            yield call(handleRenewToken, user);
          }
        } else {
          yield call(handleRenewToken, user);
        }
      }
    } else {
      Utils.setCookie(firebaseToken, "", 0);
    }
  } catch (e) {

  }
}

function* watchLoadUser({ payload: { id } }) {
  try {
    if (!Utils.getCookie(firebaseToken)) {
      yield take(RECEIVE_NEW_TOKEN);
    }
    const url = `/accounts/firebase/${id}`;
    const res = yield call(get, url);
    yield put({ type: LOAD_USER_SUCCESS, payload: res.data.data });
  } catch (e) {
    yield put({ type: LOAD_USER_FAILED, payload: e });
  }
}

function* watchSaveProfilePicture({ payload: { user } }) {
  try {
    const url = `/accounts/${user._id}`;
    yield call(patch, url, user);
    // Get user
    const userUrl = `/accounts/firebase/${user.firebase_uid}`;
    const userRes = yield call(get, userUrl);

    yield put({
      type: SAVE_PROFILE_PICTURE_SUCCESS,
      payload: userRes.data.data,
    });
  } catch (e) {
    yield put({
      type: SAVE_PROFILE_PICTURE_FAILED,
      payload: e,
    });
  }
}

function* watchSaveUser({ payload: { user } }) {
  try {
    const url = `/accounts/${user._id}`;
    yield call(patch, url, user);
    yield put({ type: SAVE_USER_SUCCESS, payload: user });
  } catch (e) {
    yield put({
      type: SAVE_USER_FAILED,
      payload: e,
    });
  }
}

const checkCurrentPassword = (passwordInfo) => {
  return auth.currentUser.reauthenticateWithCredential(
    firebase.auth.EmailAuthProvider.credential(
      auth.currentUser.email,
      passwordInfo.currentPassword,
    )
  );
}

const saveNewPassword = (newPassword) => {
  return auth.currentUser.updatePassword(newPassword);
}

function* watchSavePassword({ payload: { passwordInfo } }) {
  try {
    yield call(checkCurrentPassword, passwordInfo);
    yield call(saveNewPassword, passwordInfo.newPassword);
    yield put({
      type: SAVE_PASSWORD_SUCCESS,
    });
  } catch (e) {
    yield put({
      type: SAVE_PASSWORD_FAILED,
      payload: e,
    });
  }
}

function* watchUserLogout() {
  Utils.setCookie(firebaseToken, "", 0);
  Utils.setCookie(firebaseTokenTimer, "", 0);
  yield Auth.logout();
}

function* watchGetAccounts({payload: {activePage, filter}}) {
  try {
    const newFilter = Utils.cleanObject(filter)
    const skip = (activePage - 1) * Enum.AccountPerPage
    const url = `/accounts`
    const params = {
      filter: newFilter,
      limit: Enum.AccountPerPage,
      skip,
      sort: {first_name: 1}
    }
    const res = yield call(get, url, params)
    const totalPage = Utils.getTotalPage(res.data.metadata)

    yield put({
      type: GET_ACCOUNTS_SUCCESS,
      payload: {
        activePage,
        filter: filter,
        totalPage,
        data: res.data.data,
      }
    })
  } catch (error) {
    yield put({
      type: GET_ACCOUNTS_FAILURE,
      payload: {
        ...error
      },
    })
  }
}

function* AccountSaga() {
  yield takeEvery(CHECK_TOKEN, watchCheckToken)
  yield takeEvery(LOAD_USER, watchLoadUser)
  yield takeEvery(SAVE_PROFILE_PICTURE, watchSaveProfilePicture)
  yield takeEvery(SAVE_USER, watchSaveUser)
  yield takeEvery(SAVE_PASSWORD, watchSavePassword)
  yield takeEvery(USER_LOGOUT, watchUserLogout)
  yield takeLatest(GET_ACCOUNTS, watchGetAccounts)
}

export default AccountSaga;
