import React from 'react';
import { Button, Form, Header, Message } from "semantic-ui-react";
import injectSheet from "react-jss";
import * as Auth from "../../Firebase/Auth";
import colors from "../../Constants/color";
import AppContext from "../../Constants/AppContext";
import { firebase } from "../../Firebase/Firebase";

const style = {
  helpText: {
    marginTop: '0.2rem',
    textAlign: 'right',
    color: colors.primaryRed1,
  },
  forgotPasswordToggle: {
    textAlign: 'center',
    cursor: 'pointer',
    fontSize: '1rem',
    marginTop: '1.2rem',
    color: colors.primaryGray3,
  },
}

class FormLogin extends React.Component {
  static contextType = AppContext;

  constructor(props) {
    super(props);

    this.state = {
      errorMessage: '',
      loginInfo: {
        email: '',
        password: '',
      },
      loading: false,
      recaptcha: false,
      turnOnRecaptcha: false,
    };
    this.handleChange = this.handleChange.bind(this);
    this.isFieldError = this.isFieldError.bind(this);
    this.isFormError = this.isFormError.bind(this);
    this.handleSubmit = this.handleSubmit.bind(this);
    this.handleRecaptcha = this.handleRecaptcha.bind(this);
  }

  handleChange({ target: { name, value } }) {
    this.setState({
      loginInfo: {
        ...this.state.loginInfo,
        [name]: value,
      }
    })
  }

  handleRecaptcha() {
    const $this = this;
    window.recaptchaVerifier = new firebase.auth.RecaptchaVerifier(this.recaptcha, {
      'size': 'normal',
      'callback': function (response) {
        $this.setState({
          recaptcha: true,
        })
      },
      'expired-callback': function () {
        $this.setState({
          recaptcha: false,
        });
      }
    });
    window.recaptchaVerifier.render().then(function (widgetId) {
      window.recaptchaWidgetId = widgetId;
    });
  }

  isFieldError(name) {
    const { intl: { messages } } = this.context;
    if (!this.state.loginInfo[name]) {
      return `${messages.pleaseEnter} ${messages[name]}`;
    }
    return '';
  }

  isFormError() {
    return Object.keys(this.state.loginInfo).some(item => {
      return this.isFieldError(item);
    });
  }

  handleSubmit() {
    if (!this.state.isHitSave) {
      this.setState({
        isHitSave: true,
      });
    }

    let passRecaptcha = true;
    if(this.state.turnOnRecaptcha && !this.state.recaptcha) {
      passRecaptcha = false;
    }
    const { intl: { messages } } = this.context;
    if (!this.isFormError()) {
      if(!passRecaptcha) {
        return;
      }
      this.setState({
        loading: true,
      })
      const { loginInfo } = this.state;
      Auth.userSession("signIn", loginInfo.email, loginInfo.password)
        .then((res) => {
          this.setState({
            loading: false,
          })
          const { user } = res;
          if (!user.emailVerified) {
            Auth.logout();
            this.setState({
              errorMessage: messages.verifyEmailMessage,
            });
          }
        }).catch(e => {
        if(e.code === 'auth/too-many-requests') {
          if(this.state.turnOnRecaptcha) {
            window.recaptchaVerifier.clear();
            this.handleRecaptcha();
            this.setState({
              errorMessage: e.message,
              loading: false,
              recaptcha: false,
            });
          } else {
            this.handleRecaptcha();
            this.setState({
              turnOnRecaptcha: true,
              errorMessage: e.message,
              loading: false,
            });
          }
        } else {
          this.setState({
            errorMessage: e.message,
            loading: false,
          });
        }
      });
    }
  }

  render() {
    const { classes, changeForm } = this.props;
    const { intl: { messages } } = this.context;
    const { loginInfo, errorMessage } = this.state;
    return (
      <Form
        onSubmit={this.handleSubmit}
        error={errorMessage}
      >
        <Header as='h1' textAlign='center'>{messages.signIn}</Header>
        <Message
          error
          header={messages.loginFailed}
          content={errorMessage}
        />
        <Form.Field
          required
          error={this.state.isHitSave && this.isFieldError('email')}
        >
          <label>{messages.email}</label>
          <input
            placeholder={messages.email}
            name='email'
            value={loginInfo.email}
            onChange={this.handleChange}
          />
          {
            this.state.isHitSave && this.isFieldError('email') &&
            <div className={classes.helpText}>{this.isFieldError('email')}</div>
          }
        </Form.Field>
        <Form.Field
          required
          error={this.state.isHitSave && this.isFieldError('password')}
        >
          <label>{messages.password}</label>
          <input
            type='password'
            placeholder={messages.password}
            name='password'
            value={loginInfo.password}
            onChange={this.handleChange}
          />
          {
            this.state.isHitSave && this.isFieldError('password') &&
            <div className={classes.helpText}>{this.isFieldError('password')}</div>
          }
        </Form.Field>
        <Form.Field>
          <div ref={(ref) => this.recaptcha = ref}></div>
        </Form.Field>
        <Button
          type='submit'
          color='blue'
          fluid
          disabled={!loginInfo.email && !loginInfo.password}
          loading={this.state.loading}
        >
          {messages.signIn}
        </Button>
        <div
          className={classes.forgotPasswordToggle}
          onClick={() => changeForm('forgotPassword')}
        >
          {messages.forgotYourPassword}
        </div>
      </Form>
    )
  }
}

export default injectSheet(style)(FormLogin);
