import isempty from 'lodash.isempty';
import moment from "moment";
import Enum from "../Constants/Enum";
import _ from 'lodash'

export const firebaseToken = 'firebaseToken';
export const firebaseTokenTimer = 'firebaseTokenTimer';

const setCookie = (cname, cvalue, expSeconds) => {
  let d = new Date();
  d.setTime(d.getTime() + (expSeconds * 1000));
  const expires = "expires=" + d.toUTCString();
  document.cookie = cname + "=" + cvalue + ";" + expires + ";path=/";
};

const getCookie = (cname) => {
  const name = cname + "=";
  const decodedCookie = decodeURIComponent(document.cookie);
  const ca = decodedCookie.split(';');
  for (let i = 0; i < ca.length; i++) {
    let c = ca[i];
    while (c.charAt(0) === ' ') {
      c = c.substring(1);
    }
    if (c.indexOf(name) === 0) {
      return c.substring(name.length, c.length);
    }
  }
  return "";
};

const getLocalStorage = (name) => {
  const item = localStorage.getItem(name);
  try {
    return JSON.parse(item);
  } catch (e) {
    return null;
  }
};

const setLocalStorage = (name, value) => {
  localStorage.setItem(name, JSON.stringify(value));
};

const removeLocalStorage = (name) => {
  localStorage.removeItem(name);
};

const buildFormData = (formData) => {
  const fd = new FormData();
  Object.keys(formData).forEach(key => {
    if (formData[key] instanceof Array) {
      for (let i = 0; i < formData[key].length; i++) {
        fd.append(key, formData[key][i]);
      }
    } else {
      fd.append(key, formData[key]);
    }
  });
  return fd;
};

const validateEmail = (email) => {
  /* disable eslint */
  // const re = /^\w+([-]?\w+)*@\w+([-]?\w+)*(\.\w{2,5})+$/;
  const re = /\S+@\S+\.\S+/;
  return re.test(email);
  
};

const combineAddress = (address) => {
  if (isempty(address)) {
    return '';
  }
  let line = [address.line1, address.line2].join(' ').trim();
  const result = [
    (line) ? `${line},` : '',
    address.city,
    '\n',
    address.state,
    address.postal_code,
    address.country,
  ];
  return result.join(' ').trim();
};

const combineShortAddress = (address) => {
  if (isempty(address)) {
    return '';
  }
  const result = [
    address.city,
    address.state,
    address.postal_code,
    address.country,
  ];
  return result.join(' ');
};

const combinePersonName = (contact, messages, nickNameIncluded) => {
  if(isempty(contact)) {
    return '';
  }
  const result = [
    contact.prefix_name ? ((messages && messages[contact.prefix_name]) ? messages[contact.prefix_name] : _.upperFirst(contact.prefix_name)) : '',
    contact.first_name,
    contact.middle_name,
    contact.last_name,
  ];
  if(nickNameIncluded && contact.nickname) {
    result.splice(2, 0, `(${contact.nickname})`)
  }
  return result.join(' ');
}

const combineName = (contact, messages, nickNameIncluded) => {
  if (isempty(contact)) {
    return '';
  }
  if(contact.type === Enum.ContactTypeEnum.person) {
    return combinePersonName(contact, messages, nickNameIncluded);
  }
  return contact.organisation_name;
};

const getTotalPage = ({ limit, total }) => {
  if (total === 0) {
    return 1;
  }
  return Math.ceil(total / limit);
};

const getFormatDate = (date) => {
  const day = date.getDate();
  const month = date.getMonth() + 1;
  const year = date.getFullYear();
  return `${year}/${month}/${day}`;
};

const getUTCStartDate = (date) => {
  return new Date(Date.UTC(date.getFullYear(), date.getMonth(), date.getDate(), 0,0,0))
}

const cleanObject = (obj) => {
  const cleanObj = { ...obj };
  for (let propName in cleanObj) {
    if (cleanObj[propName] === '' || cleanObj[propName] === undefined || cleanObj[propName] === null) {
      delete cleanObj[propName];
    }
  }
  return cleanObj;
};

const getLocationFromEvent = (events, eventId) => {
  if (!events || events.length === 0) {
    return {};
  }
  const event = events.find(item => item.event === eventId);
  if (event && event.location) {
    return event.location;
  }
  return {};
};

const getContactEmail = (emails) => {
  let email = '';
  if (emails && emails.length > 0) {
    if (emails.find(item => item.primary)) {
      email = emails.find(item => item.primary).email;
    } else {
      email = emails[0].email;
    }
  }
  return email;
};

const getLastContacted = (logInfo) => {
  let date = null;
  if (logInfo && logInfo.length > 0) {
    date = moment(new Date(logInfo[logInfo.length - 1].sent_at)).format('DD MMM YYYY')
  }
  return date;
};

const getSortConfig = (name, direction) => {
  return {
    [name]: direction === Enum.SortDirection.asc ? 1 : -1,
  }
};

const parseSortConfig = (sort) => {
  const sortColumn = Object.keys(sort)[0];
  const sortDirection = Object.values(sort)[0] === 1 ? Enum.SortDirection.asc : Enum.SortDirection.des;
  return {
    sortColumn,
    sortDirection,
  }
};

const downloadFile = (blod, filename) => {
  const href = window.URL.createObjectURL(new Blob([blod]));
  const link = document.createElement("a");
  link.href = href;
  link.setAttribute("download", filename);
  document.body.appendChild(link);
  link.click();
};

const _buildGetDonation = () => ({
  $or: [
    {'payments.payment_type': 'once-off-donation'},
    {'payments.payment_type': 'recurring-donation'},
  ]
})

const _buildIsFilter = (segmentField, segmentValue) => {
  switch (segmentField) {
    case 'day_of_birth': {
      return {[segmentField]: {$eq: getUTCStartDate(segmentValue)}}
    }
    case 'payments.membership_started_at': {
      const date = moment(segmentValue)
      return {[segmentField]: {$gte: date.startOf('day').toDate().getTime(), $lte: date.endOf('day').toDate().getTime()}}
    }
    case 'payments.membership_expires_at': {
      const date = moment(segmentValue)
      return {[segmentField]: {$gte: date.startOf('day').toDate().getTime(), $lte: date.endOf('day').toDate().getTime()}}
    }
    case 'state':
      return {
        $or: [
          {'postal_address.state': {$eq: segmentValue}},
          {'work_address.state': {$eq: segmentValue}},
          {'home_address.state': {$eq: segmentValue}}
        ]
      }
    case 'postal_code':
      return {
        $or: [
          _buildPostalCodeFilter('$eq', '$postal_address.postal_code', segmentValue),
          _buildPostalCodeFilter('$eq', '$work_address.postal_code', segmentValue),
          _buildPostalCodeFilter('$eq', '$home_address.postal_code', segmentValue)
        ]
      }
    case 'payee.address.postal_code':
      return _buildPostalCodeFilter('$eq', '$payee.address.postal_code', segmentValue)  
    case 'payments.donation_started_at': {
      const date = moment(segmentValue)
      return {
        'payments.processed_date': {$gte: date.startOf('day').toDate().getTime(), $lte: date.endOf('day').toDate().getTime()},
        ..._buildGetDonation()
      }
    }
    case 'interests':
      return {[segmentField]: `${segmentValue.toLowerCase()}`}
    default:
      return {[segmentField]: {$eq: segmentValue}}
  }
}

const _buildIsNotFilter = (segmentField, segmentValue) => {
  switch (segmentField) {
    case 'day_of_birth':
      return {[segmentField]: {$ne: getUTCStartDate(segmentValue)}}
    case 'payments.membership_started_at' : {
      const date = moment(segmentValue)
      return {$or: [{[segmentField]: {$gt: date.endOf('day').toDate().getTime()}}, {[segmentField]: {$lt: date.startOf('day').toDate().getTime()}}]}
    }
    case 'payments.membership_expires_at' : {
      const date = moment(segmentValue)
      return {$or: [{[segmentField]: {$gt: date.endOf('day').toDate().getTime()}}, {[segmentField]: {$lt: date.startOf('day').toDate().getTime()}}]}
    }
    case 'state':
      return {
        $and: [
          {'postal_address.state': {$ne: segmentValue}},
          {'work_address.state': {$ne: segmentValue}},
          {'home_address.state': {$ne: segmentValue}}
        ]
      }
    case 'postal_code':
      return {
        $and: [
          _buildPostalCodeFilter('$ne', '$postal_address.postal_code', segmentValue),
          _buildPostalCodeFilter('$ne', '$work_address.postal_code', segmentValue),
          _buildPostalCodeFilter('$ne', '$home_address.postal_code', segmentValue)
        ]
      }
    case 'payee.address.postal_code':
      return _buildPostalCodeFilter('$ne', '$payee.address.postal_code', segmentValue)  
    case 'payments.donation_started_at':{
      const date = moment(segmentValue)
      return {
        $or: [{'payments.processed_date': {$gt: date.endOf('day').toDate().getTime()}}, {'payments.processed_date': {$lt: date.startOf('day').toDate().getTime()}}],
        ..._buildGetDonation()
      }    
    }
    case 'interests':
      return {$and: [
          {[segmentField]: {$exists: true, $ne: [] }}, 
          {[segmentField]:{$ne: `${segmentValue.toLowerCase()}`}}
        ]
      }
    default:
      return {[segmentField]: {$ne: segmentValue}}
  }
}

const _buildGreaterThanFilter = (segmentField, segmentValue) => {
  switch (segmentField) {
    case 'postal_code':
      return {
        $or: [
          _buildPostalCodeFilter('$gt', '$postal_address.postal_code', segmentValue),
          _buildPostalCodeFilter('$gt', '$work_address.postal_code', segmentValue),
          _buildPostalCodeFilter('$gt', '$home_address.postal_code', segmentValue)
        ]
      }
    case 'payee.address.postal_code':
      return _buildPostalCodeFilter('$gt', '$payee.address.postal_code', segmentValue)
    default:
      return {[segmentField]: {$gt: segmentValue}}
  }
}

const _buildLowerThanFilter = (segmentField, segmentValue) => {
  switch (segmentField) {
    case 'postal_code':
      return {
        $or: [
          _buildPostalCodeFilter('$lt', '$postal_address.postal_code', segmentValue),
          _buildPostalCodeFilter('$lt', '$work_address.postal_code', segmentValue),
          _buildPostalCodeFilter('$lt', '$home_address.postal_code', segmentValue)
        ]
      }
    case 'payee.address.postal_code':
      return _buildPostalCodeFilter('$lt', '$payee.address.postal_code', segmentValue)  
    default:
      return {[segmentField]: {$lt: segmentValue}}
  }
}

const _buildBeforeFilter = (segmentField, segmentValue) => {
  switch (segmentField) {
    case 'day_of_birth':
      return {[segmentField]: {$lt: getUTCStartDate(segmentValue)}}
    case 'payments.donation_started_at':
      return {
        'payments.processed_date': {$lt: (new Date(segmentValue)).getTime()},
        ..._buildGetDonation()
      } 
    default:
      return {[segmentField]: {$lt: (new Date(segmentValue)).getTime()}}
  }
}

const _buildAfterFilter = (segmentField, segmentValue) => {
  switch (segmentField) {
    case 'day_of_birth':
      return {[segmentField]: {$gt: getUTCStartDate(segmentValue)}}
    case 'payments.donation_started_at':
      return {
        'payments.processed_date': {$gt: (new Date(segmentValue)).getTime()},
        ..._buildGetDonation()
      } 
    default:
      return {[segmentField]: {$gt: (new Date(segmentValue)).getTime()}}
  }
}

const _manipulateMultipleMembershipEndDate = (segments) => {
  // populate conditions
  const membershipEndDateFilters = segments.filter(v => Object.keys(v).includes('payments.membership_expires_at'))
  const abc = membershipEndDateFilters.reduce((pre, cur) => {
    return {...pre,...cur['payments.membership_expires_at']}
  },{})
  const { $gt, $lt } = abc
  const combineFilter = {
    "payments": {
      "$elemMatch":
        {   
          $and: [
            {"membership_expires_at":{"$gt":$gt}}, 
            {"membership_expires_at":{"$lt":$lt}}
          ]
        }
    }
  }
  const remainFilters = segments.filter(v => !Object.keys(v).includes('payments.membership_expires_at'))
  return [...remainFilters, combineFilter]
}

const _manipulateMultipleMembershipStartDate = (segments) => {
  // populate conditions
  const membershipEndDateFilters = segments.filter(v => Object.keys(v).includes('payments.membership_started_at'))
  const abc = membershipEndDateFilters.reduce((pre, cur) => {
    return {...pre,...cur['payments.membership_started_at']}
  },{})
  const { $gt, $lt } = abc
  const combineFilter = {
    "payments": {
      "$elemMatch":
        {   
          $and: [
            {"membership_started_at":{"$gt":$gt}}, 
            {"membership_started_at":{"$lt":$lt}}
          ]
        }
    }
  }
  const remainFilters = segments.filter(v => !Object.keys(v).includes('payments.membership_started_at'))
  return [...remainFilters, combineFilter]
}

const buildAdvancedQueryString = ({ operator, segmentFilter }) => {
  if (isempty(segmentFilter) || !operator || !segmentFilter) {
    return {};
  }
  const mapOperator = {
    and: '$and',
    or: '$or',
  };
  const segmentGroups = [];
  Object.values(segmentFilter).forEach(groupSegment => {
    const { operator: childOperator, segmentFilter: childSegmentFilter } = groupSegment;
    if (!isempty(childSegmentFilter)) {
      const mapChildOperator = mapOperator[childOperator];
      let segments = [];
      Object.values(childSegmentFilter).forEach(segment => {
        const { field: segmentField, operator: segmentOperator, value: segmentValue } = segment;
        switch (segmentOperator) {
          case 'is':
            segments.push(_buildIsFilter(segmentField, segmentValue))
            break;
          case 'is_not':
            segments.push(_buildIsNotFilter(segmentField, segmentValue))          
            break;
          case 'contains':
            segments.push({ [segmentField]: { $regex: segmentValue, $options: "i" } });
            break;
          case 'not_contains':
            segments.push({ [segmentField]: { $regex: `^((?!${segmentValue}).)*$`, $options: 'i' } });
            break;
          case 'greater_than':
            segments.push(_buildGreaterThanFilter(segmentField, segmentValue));
            break;
          case 'lower_than':
              segments.push(_buildLowerThanFilter(segmentField, segmentValue));
            break;
          case 'before':
            segments.push(_buildBeforeFilter(segmentField, segmentValue));
            break;
          case 'after':
            segments.push(_buildAfterFilter(segmentField, segmentValue));
            break;
          case 'this_fiscal_year':  
            const thisQuarter = getFiscalYear(moment().toDate())
            segments.push({$and: [{[segmentField]: {$gte: thisQuarter.from.getTime()}}, {[segmentField]: {$lte: thisQuarter.till.getTime()}}]})
            break; 
          case 'last_fiscal_year':
            const lastQuarter = getLastFiscalYear(moment().toDate())
            segments.push({$and: [{[segmentField]: {$gte: lastQuarter.from.getTime()}}, {[segmentField]: {$lte: lastQuarter.till.getTime()}}]})
            break;  
          default:
        }
      });

      // rebuild filter if multiple membership_expires_at or membership_started_at exist by using $elemMatch
      if(segments.filter(v => Object.keys(v).includes('payments.membership_expires_at')).length > 1) {
        segments = _manipulateMultipleMembershipEndDate(segments)
      }
      if(segments.filter(v => Object.keys(v).includes('payments.membership_started_at')).length > 1) {
        segments = _manipulateMultipleMembershipStartDate(segments)
      }

      segmentGroups.push({
        [mapChildOperator]: segments,
      })
    }
  });
  return {
    [mapOperator[operator]]: segmentGroups,
  };
};

const getFiscalYear = (date) => {
  const month = moment(date).month()
  if (month >= 0 && month < 6)
    return {
      from: moment().add(-1, 'y').month(6).startOf('month').toDate(),
      till: moment().month(5).endOf('month').toDate()
    } 
    else return {
      from: moment().month(6).startOf('month').toDate(),
      till: moment().add(+1, 'y').month(5).endOf('month').toDate()
    }
}

const getLastFiscalYear = (date) => {
  const month = moment(date).month()
  if (month >= 0 && month < 6)
    return {
      from: moment().add(-2, 'y').month(6).startOf('month').toDate(),
      till: moment().add(-1, 'y').month(5).endOf('month').toDate()
    } 
    else return {
      from: moment().add(-1, 'y').month(6).startOf('month').toDate(),
      till: moment().add.month(5).endOf('month').toDate()
    }
}

const getScheduleInfo = (schedule) => {
  if (!schedule || !schedule.time_slots || schedule.time_slots.length === 0) {
    return '';
  }

  const result = [];
  schedule.time_slots.forEach(item => {
    const time = moment(new Date(item.start_time)).format('h:mma') + " - " + moment(new Date(item.end_time)).format('h:mma');
    result.push(time);
  });
  return result.join(" and ");
};

const getMembershipPeriodText = (period) => {
  if(period === 'daily') return 'day'
  if(period === 'weekly') return 'week'
  if(period === 'fortnightly') return '2 weeks'
  if(period === 'monthly') return 'month'
  if(period === 'every-3-months') return '3 months'
  if(period === 'every-6-months') return '6 months'
  if(period === 'yearly') return 'year'
}

const getContactInfo = (contact = {}) => {
  if(!contact) return ''

  if(contact.mobile_number && contact.mobile_number !== 'NA') return contact.mobile_number
  if(contact.home_number && contact.home_number !== 'NA') return contact.home_number

  if(Array.isArray(contact.emails) && contact.emails.length > 0) {
    const email = contact.emails.find(item => item.primary) || contact.emails[0]
    if(email) return email.email
  }

  return ""
}

const _buildPostalCodeFilter = (operator, field, value) => {
  return { $expr: { [operator]: [ 
    {$convert: { input: field, to: "int", onError: "Error", onNull: 0 }}, parseInt(value) || 0 
  ] } }
}

export default {
  setCookie,
  getCookie,
  buildFormData,
  validateEmail,
  combineAddress,
  combineShortAddress,
  combinePersonName,
  combineName,
  getTotalPage,
  getFormatDate,
  cleanObject,
  getLocationFromEvent,
  getLocalStorage,
  setLocalStorage,
  removeLocalStorage,
  getContactEmail,
  getLastContacted,
  getSortConfig,
  parseSortConfig,
  downloadFile,
  buildAdvancedQueryString,
  getScheduleInfo,
  getMembershipPeriodText,
  getFiscalYear,
  getContactInfo,
  getUTCStartDate
}

