import React from 'react'
import PropTypes from "prop-types"
import { Button, Modal, Form, List, Radio } from "semantic-ui-react"
import injectSheet from "react-jss"
import { ModalStyleSheet } from "../../../Common/GlobalStyle"
import AppContext from "../../../Common/Constants/AppContext"
import { toast } from "react-toastify"
import { injectStripe, CardNumberElement, CardExpiryElement } from 'react-stripe-elements'

const stylesheet = {
  ...ModalStyleSheet,
  modalContent: {
    padding: '2em 2em, 3em 2em !important',
  },
  sourceList: {
    marginTop: '2em !important',
    '& .item': {
      padding: '1.5em 2em !important',
      textAlign: 'left'
    }
  },
  actionPopup: {
    padding: '25px 35px 25px 30px'
  },
  cardElement: {
    border: '1px solid rgba(34,36,38,.15)',
    borderRadius: '3px',
    padding: '10px'
  },
  newSourceCheckBox: {
    marginLeft: '2em',
    marginBottom: '1em'
  }
}

class PaymentSourceModal extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      error: '',
      selectedSource: ''
    }
  }

  handleSubmit = async () => {
    if(!this.state.selectedSource) {
      return
    }
    if(this.state.selectedSource === 'new_source') {
      const { first_name, last_name } = this.props.contact
      const {token, error} = await this.props.stripe.createToken({name: first_name + ' ' + last_name});
      if (error) {
        toast.error(error.message)
        return
      }
      if (!token) {
        return
      }
      this.props.onUpdateSource(this.props.order._id, token.id)
    } else {
      this.props.onUpdateSource(this.props.order._id, this.state.selectedSource)
    }
    this.onClose()
  }

  handleChangeDefaultSource = async (e, { value }) => {
    this.setState({selectedSource: value})
  }

  onClose = () => {
    this.setState({selectedSource: ''})
    this.props.onClose()
  }

  render() {
    const { classes, isOpen, contact, order } = this.props;
    const { intl: { messages } } = this.context;
    let sources = []
    if(contact.stripe_customer && contact.stripe_customer.sources) {
      sources = contact.stripe_customer.sources.data
    }

    const intialSelectedSource = (order.stripe_subscription && order.stripe_subscription.default_source) || (contact.stripe_customer && contact.stripe_customer.default_source) || ''
    
    return (
      <Modal
        open={isOpen}
        onClose={this.onClose}
        size="tiny"
      >
        <Modal.Header className={classes.header}>
          {messages['contact.details.update_card']}
        </Modal.Header>
        <Modal.Content className={classes.modalContent}>
          <List celled className={classes.sourceList}>
            {
              sources.map((source, idx) => (
                <List.Item key={idx}>
                  <List.Content floated='left' verticalAlign='middle'>
                  <Radio
                      name='defaultSource'
                      value={source.id}
                      checked={this.state.selectedSource ? (this.state.selectedSource === source.id) : intialSelectedSource === source.id}
                      onChange={this.handleChangeDefaultSource}
                  />
                  </List.Content>
                  <List.Content>
                    <List.Description><b>{source.name}</b>    **** **** **** {source.last4}</List.Description>
                  </List.Content>
                </List.Item>
              ))
            }
          </List>
          <Radio
            className={classes.newSourceCheckBox}
            label='Add a new card'
            name='defaultSource'
            value='new_source'
            checked={this.state.selectedSource === 'new_source'}
            onChange={this.handleChangeDefaultSource}
          />
          {
            this.state.selectedSource === 'new_source' &&  
            <Form>
              <Form.Group>
                <Form.Field width={12}>
                  <label>Credit or debit card</label>
                  <div className={classes.cardElement}><CardNumberElement /></div>
                </Form.Field>
                <Form.Field width={4}>
                  <label>Expiry</label>
                  <div className={classes.cardElement}><CardExpiryElement /></div>
                </Form.Field>
              </Form.Group>
              <p>{this.state.error}</p>
            </Form>
          }
        </Modal.Content>
        <Modal.Actions>
          <Button onClick={this.onClose}>
            {messages['common.cancel']}
          </Button>
          <Button
            onClick={this.handleSubmit}
            color='green'
            disabled={!this.state.selectedSource}>
            {messages['common.update']}
          </Button>
        </Modal.Actions>
      </Modal>
    );
  }
}

PaymentSourceModal.propTypes = {
  isOpen: PropTypes.bool.isRequired,
  onClose: PropTypes.func.isRequired,
  onUpdateSource: PropTypes.func.isRequired,
  order: PropTypes.object.isRequired,
  contact: PropTypes.object.isRequired
};

PaymentSourceModal.contextType = AppContext;

export default injectSheet(stylesheet)(injectStripe(PaymentSourceModal));
