import React from 'react'
import PropTypes from "prop-types"
import { Button, Modal, List, Icon, Popup } from "semantic-ui-react"
import injectSheet from "react-jss"
import { ModalStyleSheet } from "../../../../Common/GlobalStyle"
import AppContext from "../../../../Common/Constants/AppContext"
import { config } from '../../../../configuration'
import AddCardModal from './AddCardModal'
import { StripeProvider, Elements} from 'react-stripe-elements'
import ContactActionCreator from '../../../Actions/ContactActionCreator'

const stylesheet = {
  ...ModalStyleSheet,
  modalContent: {
    padding: '2em 0 3em 0 !important',
    textAlign: 'center'
  },
  sourceList: {
    marginTop: '2em !important',
    '& .item': {
      padding: '1.5em 2em !important',
      textAlign: 'left'
    }
  },
  actionPopup: {
    padding: '25px 35px 25px 30px'
  },
  hide: {
    visibility: 'hidden !important'
  }
}

class PaymentSourceModal extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      isShowAddCardModal: false
    };
  }

  onClose = () => {
    this.props.onClose()
  }

  onAddSource = (source) => {
    const { contact } = this.props
    const { dispatch } = this.context
    dispatch(ContactActionCreator.addPaymentSource(contact._id, source))
    this.setState({isShowAddCardModal: false})
  }

  onRemoveSource = (source) => {
    const { contact } = this.props
    const { dispatch } = this.context
    dispatch(ContactActionCreator.removePaymentSource(contact._id, source))
    this.setState({isShowAddCardModal: false})
  }

  onSetDefault = source => {
    const { contact } = this.props
    const { dispatch } = this.context
    dispatch(ContactActionCreator.setDefaultPaymentSource(contact._id, source))
  }

  render() {
    const { classes, isOpen, contact } = this.props;
    const { intl: { messages } } = this.context;
    let sources = []
    if(contact.stripe_customer && contact.stripe_customer.sources) {
      sources = contact.stripe_customer.sources.data.map(source => {
        source['isDefault'] = source.id === contact.stripe_customer.default_source
        return source
      })
    }
    return (
      <Modal
        open={isOpen}
        onClose={this.onClose}
        size="tiny"
      >
        <Modal.Header className={classes.header}>
          {messages['contact.details.payment_source']}
        </Modal.Header>
        <Modal.Content className={classes.modalContent}>
          <Button
            icon='plus circle'
            labelPosition='right'
            color='green'
            className='add-card'
            content='Add card'
            onClick={() => this.setState({isShowAddCardModal: true})}
          />
          <List celled className={classes.sourceList}>
            {
              sources.map((source, idx) => (
                <List.Item key={idx}>
                  <List.Content floated='right' verticalAlign='middle'>
                    <Popup className={classes.actionPopup} style={{padding: '20px 30px 20px 20px'}} position='bottom right' hoverable on='click' trigger={<Icon link name='ellipsis horizontal' />}>
                      <List link>
                        <List.Item style={{marginBottom: 5}} as='a' onClick={() => this.onSetDefault(source.id)}>{messages['common.make_default']}</List.Item>
                        {/* <List.Item style={{marginBottom: 5}} as='a' onClick={() => {console.log('CLICK')}}>{messages.edit}</List.Item> */}
                        <List.Item as='a' onClick={() => this.onRemoveSource(source.id)}>{messages.delete}</List.Item>
                      </List>
                    </Popup>
                  </List.Content>
                  {
                    <List.Icon name='checkmark' color='blue' verticalAlign='middle' className={!source.isDefault && classes.hide} />
                  }
                  <List.Content>
                    <List.Header>{source.name}</List.Header>
                    <List.Description>**** **** **** {source.last4}</List.Description>
                  </List.Content>
                </List.Item>
              ))
            }
          </List>
          <StripeProvider apiKey={config.REACT_APP_STRIPE_KEY}>
            <Elements>
              <AddCardModal 
                isOpen={this.state.isShowAddCardModal} 
                onClose={() => this.setState({isShowAddCardModal: false})}
                onAddSource={this.onAddSource}
                contact={contact} />
            </Elements>
          </StripeProvider>
        </Modal.Content>
        <Modal.Actions>
          <Button onClick={this.onClose}>
            {messages['common.dismiss']}
          </Button>
        </Modal.Actions>
      </Modal>
    );
  }
}

PaymentSourceModal.propTypes = {
  isOpen: PropTypes.bool.isRequired,
  onClose: PropTypes.func.isRequired,
  contact: PropTypes.object.isRequired
};

PaymentSourceModal.contextType = AppContext;

export default injectSheet(stylesheet)(PaymentSourceModal);
