import React from 'react';
import PropTypes from "prop-types";
import AppContext from "../../../../Common/Constants/AppContext";
import { Button, Modal, Form } from "semantic-ui-react";
import isequal from "lodash.isequal";
import injectSheet from "react-jss";
import { ModalStyleSheet } from "../../../../Common/GlobalStyle";
import DatePicker from "react-datepicker/es";
import Enum from "../../../../Common/Constants/Enum";
import { config } from '../../../../configuration';

const stylesheet = {
  helpText: {
    extend: ModalStyleSheet.helpText,
  },
  actionGroup: {
    extend: ModalStyleSheet.actionGroup,
  },
  header: {
    extend: ModalStyleSheet.header,
  },
  modalContent: {
    '& .react-datepicker-wrapper': {
      width: '100%',
      '& .react-datepicker__input-container': {
        width: '100%',
        '& .expiry-date': {
          width: '100%',
        }
      }
    }
  },
}

class SpecialisationInfoModal extends React.Component {
  constructor(props) {
    super(props);
    const {specialisationInfo} = props;
    this.state = {
      specialisationInfo: {
        ...specialisationInfo,
        expiry_date: specialisationInfo.expiry_date ? new Date(specialisationInfo.expiry_date): new Date(),
      },
    };
    this.isFieldError = this.isFieldError.bind(this);
    this.handleChange = this.handleChange.bind(this);
    this.handleSubmit = this.handleSubmit.bind(this);
    this.handleChangeDate = this.handleChangeDate.bind(this);
  }

  handleChange({ target: { name, value } }) {
    this.setState({
      specialisationInfo: {
        ...this.state.specialisationInfo,
        [name]: value,
      }
    })
  }

  handleChangeDate(value) {
    const specialisationInfo = { ...this.state.specialisationInfo };
    const date = value || new Date();
    specialisationInfo.expiry_date = date;
    this.setState({
      specialisationInfo,
    });
  }

  handleSubmit() {
    if (!this.state.isHitSave) {
      this.setState({
        isHitSave: true,
      });
    }
    if (this.isFieldError('name')) {
      return;
    }
    const { onOk } = this.props;
    const specialisationInfo = { ...this.state.specialisationInfo };
    onOk({
      ...this.state.specialisationInfo,
      expiry_date: specialisationInfo.expiry_date.getTime(),
    });
  }

  isFieldError(name) {
    const { intl: { messages } } = this.context;
    if (!this.state.specialisationInfo[name]) {
      let fieldText = messages[name];
      if (name === 'name') {
        fieldText = messages.specialisationName;
      }
      return `${messages.pleaseEnter} ${fieldText}`;
    }
    return '';
  }

  render() {
    const { classes, isOpen, onClose } = this.props;
    const { intl: { messages } } = this.context;
    const { specialisationInfo } = this.state;
    const emptyFilter = {
      key: 'select',
      text: messages.selectState,
      value: '',
    };
    const stateList = Object.keys(Enum.StateAustralia).map(item => ({
      key: item,
      value: item,
      text: Enum.StateAustralia[item],
    }));
    stateList.unshift(emptyFilter);
    return (
      <Modal
        open={isOpen}
        onClose={onClose}
        size="tiny"
      >
        <Modal.Header className={classes.header}>
          {
            specialisationInfo.id ? messages.updateSpecialisation : messages.addSpecialisation
          }
        </Modal.Header>
        <Modal.Content className={classes.modalContent}>
          <Form
            onSubmit={this.handleSubmit}
          >
            <Form.Field required error={this.state.isHitSave && this.isFieldError('name')}>
              <label>{messages.specialisationName}</label>
              <input
                placeholder={messages.specialisationName}
                name='name'
                value={specialisationInfo.name}
                onChange={this.handleChange}
              />
            </Form.Field>
            <Form.Group widths='equal'>
              <Form.Field required error={this.state.isHitSave && this.isFieldError('state')}>
                <label>{messages.status}</label>
                <Form.Select
                  options={stateList}
                  placeholder={messages.selectState}
                  className='state-dropdown'
                  name='state'
                  value={specialisationInfo.state}
                  onChange={(e, target) => this.handleChange({ target: target })}
                />
                {
                  this.state.isHitSave && this.isFieldError('state') &&
                  <div className={classes.helpText}>{this.isFieldError('state')}</div>
                }
              </Form.Field>
              <Form.Field required error={this.state.isHitSave && this.isFieldError('expiry_date')}>
                <label>{messages.expiryDate}</label>
                <DatePicker
                  selected={specialisationInfo.expiry_date}
                  className='expiry-date'
                  onChange={this.handleChangeDate}
                  dateFormat='dd/MM/yyyy'
                  placeholderText={config.DATE_FORMAT}
                  todayButton={'Today'}
                />
              </Form.Field>
            </Form.Group>
            <Form.Field className={classes.actionGroup}>
              <Button onClick={onClose}>
                {messages.cancel}
              </Button>
              <Button
                type='submit'
                color='green'
                disabled={isequal(this.props.specialisationInfo, specialisationInfo)}
              >
                {
                  specialisationInfo.id ? messages.update : messages.add
                }
              </Button>
            </Form.Field>
          </Form>
        </Modal.Content>
      </Modal>
    );
  }
}

SpecialisationInfoModal.propTypes = {
  isOpen: PropTypes.bool.isRequired,
  onClose: PropTypes.func.isRequired,
  onOk: PropTypes.func.isRequired,
  specialisationInfo: PropTypes.shape({}).isRequired,
};

SpecialisationInfoModal.contextType = AppContext;

export default injectSheet(stylesheet)(SpecialisationInfoModal);