import React from "react";
import PropTypes from "prop-types";
import { Button, Modal, Form, Label, Message } from "semantic-ui-react";
import {
  injectStripe,
  CardNumberElement,
  CardExpiryElement,
} from "react-stripe-elements";
import isequal from "lodash.isequal";
import injectSheet from "react-jss";
import { ModalStyleSheet } from "../../../Common/GlobalStyle";
import AppContext from "../../../Common/Constants/AppContext";
import Enum from "../../../Common/Constants/Enum";
import DatePicker from "react-datepicker/es";
import { toast } from "react-toastify";
import { config } from "../../../configuration";
import PaymentSourceSelector from "../Common/PaymentSourceSelector";
import isEmpty from "lodash.isempty";

const stylesheet = {
  ...ModalStyleSheet,
  modalContent: {
    "& .react-datepicker-wrapper": {
      width: "100%",
      "& .react-datepicker__input-container": {
        width: "100%",
        "& .date-received": {
          width: "100%",
        },
      },
    },
  },
  cardElement: {
    border: "1px solid rgba(34,36,38,.15)",
    borderRadius: "3px",
    padding: "10px",
  },
  note: {
    fontSize: 13,
    color: "#7c7c7c",
    "& span": {
      fontWeight: 600
    }
  }
};

class AddDonationModal extends React.Component {
  static getDerivedStateFromProps(props, state) {
    if (props.activeTab !== state.activeTab) {
      if (props.activeTab !== "summary") {
        return {
          donationInfo: {
            ...state.donationInfo,
            ...{ donation_type: props.activeTab },
          },
        };
      }
    }
    return null;
  }
  constructor(props) {
    super(props);
    this.state = {
      donationInfo: {
        period: "monthly",
      },
      source:
        (this.props.contactDetail.stripe_customer &&
          this.props.contactDetail.stripe_customer.default_source) ||
        "new_source",
    };
    this.handleChange = this.handleChange.bind(this);
    this.handleSubmit = this.handleSubmit.bind(this);
    this.handleChangeDate = this.handleChangeDate.bind(this);
  }

  handleChange({ target: { name, value } }) {
    if (name === "donation_type") {
      this.setState({
        donationInfo: { ...this.statedonationInfo, tender: "" },
      });
    }
    this.setState({
      donationInfo: {
        ...this.state.donationInfo,
        [name]: value,
        ...(name === "donation_type" && { tender: "" }),
      },
    });
  }

  handleChangeDate(value) {
    const donationInfo = { ...this.state.donationInfo };
    const date = value || new Date();
    if (donationInfo.donation_type === "once-off-donation")
      donationInfo.received_date = date;
    if (donationInfo.donation_type === "recurring-donation")
      donationInfo.start_date = date;
    this.setState({
      donationInfo,
    });
  }

  isFieldError = (name) => {
    if (!this.state.isHitSave) {
      return false;
    }
    if (!this.state.donationInfo[name]) {
      return true;
    }
    return false;
  };

  onSelectSource = (source) => {
    this.setState({ source });
  };

  async handleSubmit() {
    if (!this.state.isHitSave) {
      this.setState({
        isHitSave: true,
      });
    }
    const donationInfo = this.state.donationInfo;
    if (
      !donationInfo.tender ||
      !donationInfo.donation_type ||
      !donationInfo.amount
    )
      return;
    if (
      donationInfo.donation_type === "once-off-donation" &&
      !donationInfo.received_date
    )
      return;
    if (
      donationInfo.donation_type === "recurring-donation" &&
      (!donationInfo.start_date || !donationInfo.period)
    )
      return;
    const { onCreate } = this.props;
    
    if (donationInfo.tender === "credit_card_stripe") {
      try {
        let source = this.state.source;
        if (this.state.source === "new_source") {
          const {
            contactDetail: { first_name, last_name },
          } = this.props;
          const { token, error } = await this.props.stripe.createToken({
            name: first_name + " " + last_name,
          });
          if (error) {
            toast.error(error.message);
          }
          if (!token) return;
          source = token.id;
        }

        onCreate({ ...donationInfo, ...{ stripe_token_id: source } });
        this._resetForm();
      } catch (error) {
        toast.error(error.message);
      }
    } else {
      onCreate({ ...donationInfo });
      this._resetForm();
    }
  }

  _showPeriodUnit = (period) => {
    if (period === "monthly") return "mo";
    if (period === "weekly") return "wk";
    return "";
  };

  _resetForm = () => {
    this.setState({
      donationInfo: {
        period: "monthly",
      },
      isHitSave: false,
      source:
        (this.props.contactDetail.stripe_customer &&
          this.props.contactDetail.stripe_customer.default_source) ||
        "new_source",
    });
  };

  onClose = () => {
    this._resetForm();
    this.props.onClose();
  };

  render() {
    const { classes, isOpen, activeTab, contactDetail } = this.props;
    const {
      intl: { messages },
    } = this.context;
    const { donationInfo } = this.state;
    const emptyFilter = {
      key: "select",
      text: "Please Select",
      value: "",
    };

    const donationTypes = Object.values(Enum.DonationTypeEnum).map((item) => ({
      key: item,
      value: item,
      text: messages[item],
    }));
    donationTypes.unshift(emptyFilter);

    let tenderList = Object.keys(Enum.Tender).map((item) => ({
      key: item,
      value: item,
      text: messages[item],
    }));
    if (donationInfo.donation_type === "recurring-donation") {
      tenderList = tenderList.filter((t) => t.value === "credit_card_stripe");
    }
    tenderList.unshift(emptyFilter);

    const periods = Object.keys(Enum.DonationPeriods).map((item) => ({
      key: item,
      value: item,
      text: Enum.DonationPeriods[item],
    }));
    periods.unshift(emptyFilter);

    const isHasAddress =
      !isEmpty(contactDetail?.home_address) ||
      !isEmpty(contactDetail?.postal_address) ||
      !isEmpty(contactDetail?.work_address);

    const isHasEmail = !isEmpty(contactDetail?.emails);

    const contactName = [contactDetail?.first_name, contactDetail?.last_name]
      .filter((item) => item)
      .join(" ");

    return (
      <Modal open={isOpen} onClose={this.onClose} size="tiny">
        <Modal.Header className={classes.header}>
          {messages.newDonation}
        </Modal.Header>
        <Modal.Content className={classes.modalContent}>
          {!isHasAddress && (
            <Message error>
              <Message.Header>Unable to Proceed</Message.Header>
              <p>The contact must have a current address</p>
            </Message>
          )}
          {!isHasEmail && (
            <p className={classes.note}>
              <span>Note.</span> {contactName} dose not have an email address listed and will
              not receive receipts or follow-up communications by email.
            </p>
          )}
          <Form onSubmit={this.handleSubmit}>
            <Form.Field
              required
              error={this.state.isHitSave && !donationInfo.donation_type}
            >
              <label>{messages.type}</label>
              <Form.Select
                options={donationTypes}
                name="donation_type"
                value={donationInfo.donation_type}
                onChange={(e, target) => {
                  this.handleChange({ target: target });
                }}
                disabled={activeTab !== "summary"}
              />
            </Form.Field>
            {donationInfo.donation_type === "recurring-donation" && (
              <Form.Field
                required
                error={this.state.isHitSave && !donationInfo.period}
              >
                <label>{messages.frequency}</label>
                <Form.Select
                  options={periods}
                  name="period"
                  value={donationInfo.period}
                  onChange={(e, target) => {
                    this.handleChange({ target: target });
                  }}
                />
              </Form.Field>
            )}
            {donationInfo.donation_type && (
              <Form.Group widths="equal">
                <Form.Field
                  required
                  error={this.state.isHitSave && !donationInfo.tender}
                >
                  <label>{messages.amount}</label>

                  <Form.Input
                    labelPosition={
                      donationInfo.donation_type === "recurring-donation"
                        ? "right"
                        : "left"
                    }
                  >
                    <Label basic>$</Label>
                    <input
                      name="amount"
                      type="number"
                      value={donationInfo.amount || ""}
                      onChange={this.handleChange}
                    />
                    {donationInfo.donation_type === "recurring-donation" && (
                      <Label>
                        /{this._showPeriodUnit(donationInfo.period)}
                      </Label>
                    )}
                  </Form.Input>
                </Form.Field>
                {donationInfo.donation_type === "once-off-donation" && (
                  <Form.Field
                    required
                    error={
                      this.state.isHitSave && this.isFieldError("received_date")
                    }
                  >
                    <label>{messages.dateReceived}</label>
                    <DatePicker
                      selected={donationInfo.received_date}
                      className="date-received"
                      onChange={this.handleChangeDate}
                      dateFormat="dd/MM/yyyy"
                      placeholderText={config.DATE_FORMAT}
                      todayButton={"Today"}
                    />
                  </Form.Field>
                )}
                {donationInfo.donation_type === "recurring-donation" && (
                  <Form.Field
                    required
                    error={
                      this.state.isHitSave && this.isFieldError("start_date")
                    }
                  >
                    <label>{messages.startDate}</label>
                    <DatePicker
                      selected={donationInfo.start_date}
                      className="date-received"
                      onChange={this.handleChangeDate}
                      dateFormat="dd/MM/yyyy"
                      placeholderText={config.DATE_FORMAT}
                      todayButton={"Today"}
                    />
                  </Form.Field>
                )}
              </Form.Group>
            )}
            {donationInfo.donation_type && (
              <Form.Group widths="equal">
                <Form.Field
                  required
                  error={this.state.isHitSave && !donationInfo.tender}
                >
                  <label>{messages.tender}</label>
                  <Form.Select
                    options={tenderList}
                    placeholder={messages.tender}
                    className="state-dropdown"
                    name="tender"
                    value={donationInfo.tender}
                    onChange={(e, target) =>
                      this.handleChange({ target: target })
                    }
                  />
                </Form.Field>
                <Form.Field>
                  <label>{messages.campaignId}</label>
                  <Form.Input
                    placeholder="Campaign ID (optional)"
                    name="campaign_id"
                    value={donationInfo.campaign_id || ""}
                    onChange={this.handleChange}
                  />
                </Form.Field>
              </Form.Group>
            )}
            {donationInfo.tender === "credit_card_stripe" && (
              <PaymentSourceSelector
                contact={contactDetail}
                onSelectSource={this.onSelectSource}
                currentSource={this.state.source}
              />
            )}
            {donationInfo.tender === "credit_card_stripe" &&
              this.state.source === "new_source" && (
                <Form.Group>
                  <Form.Field width={12}>
                    <label>Credit or debit card</label>
                    <div className={classes.cardElement}>
                      <CardNumberElement />
                    </div>
                  </Form.Field>
                  <Form.Field width={4}>
                    <label>Expiry</label>
                    <div className={classes.cardElement}>
                      <CardExpiryElement />
                    </div>
                  </Form.Field>
                </Form.Group>
              )}
            <Form.Field className={classes.actionGroup}>
              <Button onClick={this.onClose}>{messages.cancel}</Button>
              <Button
                type="submit"
                color={!isHasAddress ? "grey" : "green"}
                disabled={
                  !isHasAddress ||
                  isequal(this.props.donationInfo, donationInfo)
                }
              >
                {messages.save}
              </Button>
            </Form.Field>
          </Form>
        </Modal.Content>
      </Modal>
    );
  }
}

AddDonationModal.propTypes = {
  isOpen: PropTypes.bool.isRequired,
  onClose: PropTypes.func.isRequired,
};

AddDonationModal.contextType = AppContext;

export default injectSheet(stylesheet)(injectStripe(AddDonationModal));
