import React from 'react';
import { Button, Grid, Icon, Tab } from "semantic-ui-react";
import injectSheet from "react-jss";
import { Elements, StripeProvider } from 'react-stripe-elements';
import isequal from 'lodash.isequal';

import ProfileCard from "../ContactDetailProfile/ProfileCard";
import stylesheet from "./stylesheet";
import { config } from "../../../configuration";
import ContactDonationsSummary from "./ContactDonationsSummary/ContactDonationsSummary";
import ContactDonationsOnceOff from "./ContactDonationsOnceOff/ContactDonationsOnceOff";
import ContactDonationsRecurring from "./ContactDonationsRecurring/ContactDonationsRecurring";
import AddDonationModal from "./AddDonationModal";

import AppContext from "../../../Common/Constants/AppContext";
import ContactOrderCreator from "../../Actions/ContactOrderActionCreator";
import ContactCreator from '../../Actions/ContactActionCreator'

class ContactDonations extends React.Component {
  constructor() {
    super();
    this.state = {
      addDonationToggle: false
    }
  }

  getPanels = ({donationSummaryDetail, contactOnceOffDonationOrders, contactRecurringDonationOrders, selectedContact, contactDonationPayments}) => {
    return [
      {
        menuItem: "Summary",
        render: () => (
          <ContactDonationsSummary 
            donationSummaryDetail = {donationSummaryDetail} 
            selectedContact={selectedContact} />
        )
      },
      {
        menuItem: "Once-off Donations",
        render: () => (
          <ContactDonationsOnceOff 
            contactOnceOffDonationOrders = {contactOnceOffDonationOrders} 
            contactDonationPayments={contactDonationPayments}
            selectedContact={selectedContact} />
        )
      },
      {
        menuItem: "Recurring Donations",
        render: () => (
          <ContactDonationsRecurring 
            contactRecurringDonationOrders = {contactRecurringDonationOrders} 
            contactDonationPayments={contactDonationPayments}
            selectedContact={selectedContact} />
        )
      },
    ];
  }

  getSnapshotBeforeUpdate(prevProps, prevState) {
    const { selectedContactDetail: { selectedContact } } = this.props;
    const { selectedContactDetail: { selectedContact: prevSelectedContact } } = prevProps;
    if (!isequal(selectedContact, prevSelectedContact)) {
      return true;
    }
    return null;
  }

  componentDidUpdate(prevProps, prevState, snapshot) {
    if (snapshot) {
      const { dispatch } = this.context;
      const {selectedContactDetail: {selectedContact}} = this.props;

      const filterDonationPayment = {
        linked_profile: selectedContact._id,
        payment_type: {$in: ["once-off-donation", "recurring-donation"]},
        nb_donation_id: { $exists: true }
      }
      dispatch(ContactOrderCreator.getContactDonationPayments(1, filterDonationPayment));
    }
  }

  onCreateDonation = (donationOrder) => {
    const { amount, donation_type, start_date, received_date, campaign_id, stripe_token_id, tender, period } = donationOrder;
    const { selectedContactDetail: {selectedContact} } = this.props;
    const { dispatch } = this.context;
    const order = {
      contact: selectedContact._id,
      type: "donation",
      donation: {
        donation_type,
        amount,
        period,
        received_date: received_date && received_date.getTime(),
        start_date: start_date && start_date.getTime()
      },
      campaign_id,
      stripe_token_id,
      tender
    }
    dispatch(ContactOrderCreator.createContactDonationOrder(order));
    this.setState({addDonationToggle: false});
    
  }

  onTabChange = (e, data) => {
    let activeTabs = [
      'summary',
      'once-off-donation',
      'recurring-donation'
    ]
    const { dispatch } = this.context;
    dispatch(ContactCreator.setDonationTabIndex(activeTabs[data.activeIndex]))
  }

  render() {
    const { 
      donationSummaryDetail, 
      contactOnceOffDonationOrders, 
      contactRecurringDonationOrders,
      contactDonationPayments,
      classes, 
      selectedContactDetail, 
      history,
      donationTabIndex } = this.props;
    const { selectedContact } = selectedContactDetail;

    const donationTabs = [
      'summary',
      'once-off-donation',
      'recurring-donation'
    ]
    const activeIndex = donationTabs.indexOf(donationTabIndex)
    return (
      <div className={classes.contactDetailOverview}>
        <Grid>
          <Grid.Row>
            <Grid.Column computer={5}>
              <ProfileCard selectedContactDetail={selectedContactDetail} history={history} />
            </Grid.Column>
            <Grid.Column computer={11}>
              {
                selectedContact._id &&
                <div className='contact-donation-tab'>
                  <Tab menu={{secondary: true, pointing: true}} 
                    panes={this.getPanels({donationSummaryDetail, contactOnceOffDonationOrders, contactRecurringDonationOrders, selectedContact, contactDonationPayments})}
                    onTabChange={this.onTabChange} defaultActiveIndex={activeIndex} />
                  <StripeProvider apiKey={config.REACT_APP_STRIPE_KEY}>
                    <Elements>
                      <AddDonationModal
                        onClose={() => this.setState({addDonationToggle: false})}
                        isOpen={this.state.addDonationToggle}
                        onCreate={this.onCreateDonation}
                        contactDetail={selectedContact}
                        activeTab={donationTabIndex}
                      />
                    </Elements>
                  </StripeProvider> 
                  <Button
                    icon
                    labelPosition='right'
                    color='green'
                    className='add-donation'
                    onClick={() => this.setState({ addDonationToggle: true })}
                  >
                    Donation
                    <Icon name='add' circular inverted />
                  </Button>
                </div>
              }
            </Grid.Column>
          </Grid.Row>
        </Grid>
      </div>
    );
  }
}

ContactDonations.contextType = AppContext;

export default injectSheet(stylesheet)(ContactDonations);
