import React from 'react';
import PropTypes from 'prop-types';
import { Button, Form, Icon, Modal, Select, Dropdown } from 'semantic-ui-react';
import injectSheet from "react-jss";
import AppContext from "../../../../../../Common/Constants/AppContext";
import stylesheet from './criteriaModalStylesheet';
import SegmentGroup from "../FilterEditor/SegmentGroup/SegmentGroup";
import uuid from "uuid/v4";

const operatorOptions = {
  and: 'and',
  or: 'or',
};

class CriteriaInfoModal extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      criteriaInfo: {
        ...props.criteriaInfo,
      },
      sharedToAccounts: (props.criteriaInfo.sharedTo && props.criteriaInfo.sharedTo.length > 0) ? 
        props.criteriaInfo.sharedTo : 
        (props.criteriaInfo.is_public ? ['any'] : [])
    };
    this.applyFilter = this.applyFilter.bind(this);
  }

  applyFilter() {
    if (!this.state.isHitSave) {
      this.setState({
        isHitSave: true,
      });
    }
    const criteriaInfo = { ...this.state.criteriaInfo };
    if (!this.checkValidFilterData()) {
      return;
    }
    const { onApplyFilter } = this.props;
    if(this.state.sharedToAccounts.includes('any')) {
      criteriaInfo.is_public = true
      criteriaInfo.sharedTo = []
    } else {
      criteriaInfo.sharedTo = this.state.sharedToAccounts
      criteriaInfo.is_public = false
    }
    onApplyFilter(criteriaInfo);
  }

  handleOperatorChange = (e, { name, value }) => {
    const { criteriaInfo } = this.state;
    const { criteria } = criteriaInfo;
    criteria.operator = value;
    this.setState({
      criteriaInfo: {
        ...criteriaInfo,
        criteria,
      },
    });
  }

  handleAddSegmentGroup = () => {
    const criteriaInfo = { ...this.state.criteriaInfo };
    const { criteria } = criteriaInfo;
    const id = uuid();
    const childId = uuid();
    criteria.segmentFilter[id] = {
      id,
      operator: operatorOptions.and,
      segmentFilter: {
        [childId]: {
          id: childId,
          field: '',
          operator: '',
          value: ''
        }
      },
    };

    this.setState({
      criteriaInfo,
    });
  }

  handleRemoveSegmentGroup = (id) => {
    const criteriaInfo = { ...this.state.criteriaInfo };
    const { criteria } = criteriaInfo;
    delete criteria.segmentFilter[id];
    this.setState({
      criteriaInfo,
    });
  }

  handleFilterChange = (item) => {
    const criteriaInfo = { ...this.state.criteriaInfo };
    const { criteria } = criteriaInfo;
    criteria.segmentFilter[item.id] = item;
    this.setState({
      criteriaInfo,
    });
  }

  checkValidFilterData() {
    const criteriaInfo = { ...this.state.criteriaInfo };
    const { criteria } = criteriaInfo;
    const { operator, segmentFilter } = criteria;
    if (!operator) {
      return false;
    }
    let isValid = true;
    Object.values(segmentFilter).forEach(groupSegment => {
      const { operator: childOperator, segmentFilter: childSegmentFilter } = groupSegment;
      if (!childOperator) {
        isValid = false;
      }
      Object.values(childSegmentFilter).forEach(segment => {
        Object.values(segment).forEach(field => {
          if (typeof field !== 'boolean' && !field) {
            isValid = false;
          }
        })
      })
    })
    return isValid;
  }

  handleChangeAccountShared = (e, {value}) => {
    this.setState({sharedToAccounts: value})
  }

  render() {
    const { classes, isOpen, onClose, accountList } = this.props;
    const { intl: { messages } } = this.context;
    const { criteriaInfo, isHitSave } = this.state;
    const matches = Object.keys(operatorOptions).map(key => ({
      key,
      text: messages[`common.advanced_search.${key}`],
      value: operatorOptions[key],
    }));

    const sharedToAccountsOptions = accountList.data.map(item => ({
      key: item._id,
      text: item.first_name + ' ' + item.last_name,
      value: item._id
    }))

    sharedToAccountsOptions.unshift({
      key: 'any',
      text: 'Everyone',
      value: 'any'
    })

    const { operator, segmentFilter } = criteriaInfo.criteria;

    return (
      <Modal
        open={isOpen}
        onClose={onClose}
        closeOnDimmerClick={false}
        size={'small'}
        className={classes.criteriaModal}
      >
        <Modal.Header className={classes.header}>
          {messages.editFilter}
        </Modal.Header>
        <Modal.Content>
          <Form>
            <div className={classes.filterEditor}>
              <div className='match-operator-container'>
                <span>Matches</span>
                <Select
                  className='match-operator'
                  options={matches}
                  value={operator}
                  onChange={this.handleOperatorChange}
                />
                <span>of the following</span>
              </div>
              {
                Object.values(segmentFilter).map((item, index) => (
                  <SegmentGroup
                    key={item.id}
                    index={index}
                    parentOperator={operator}
                    segmentGroup={item}
                    handleChange={this.handleFilterChange}
                    handleRemoveSegmentGroup={this.handleRemoveSegmentGroup}
                    isHitSave={isHitSave}
                  />
                ))
              }
              <div className='action-section'>
                <Button
                  onClick={this.handleAddSegmentGroup}
                  className='btn-add-segment-group'
                >
                  Add Criteria
                  <Icon name='add' circular color='black' />
                </Button>
                <Dropdown
                  className='shareWith'
                  options={sharedToAccountsOptions}
                  placeholder='Share with...'
                  search
                  selection
                  multiple
                  value={this.state.sharedToAccounts}
                  onChange={this.handleChangeAccountShared}
                />
                <div>
                  <Button onClick={onClose}>
                    Close
                  </Button>
                  <Button color='green' onClick={this.applyFilter}>
                    Apply filter
                  </Button>
                </div>
              </div>
            </div>
          </Form>
        </Modal.Content>
      </Modal>
    )
  }
}

CriteriaInfoModal.propTypes = {
  isOpen: PropTypes.bool.isRequired,
  onClose: PropTypes.func.isRequired,
  criteriaInfo: PropTypes.shape({}).isRequired
};

CriteriaInfoModal.contextType = AppContext;

export default injectSheet(stylesheet)(CriteriaInfoModal);