import React from 'react';
import PropTypes from 'prop-types';
import injectSheet from 'react-jss';
import uuid from 'uuid/v4';
import { Select, Button, Icon } from 'semantic-ui-react';

import stylesheet from './stylesheet';
import SegmentGroup from "./SegmentGroup/SegmentGroup";
import AppContext from "../../../../../../Common/Constants/AppContext";
import ContactActionCreator from "../../../../../Actions/ContactActionCreator";

const operatorOptions = {
  and: 'and',
  or: 'or',
};

class FilterEditor extends React.Component {
  constructor(props) {
    super(props);
    const { criteriaInfo } = props;
    this.state = {
      operator: operatorOptions.and,
      segmentFilter: {},
      ...criteriaInfo,
      isHitSave: false,
    }
  }

  handleChange = (e, { name, value }) => {
    this.setState({
      operator: value,
    });
  }

  handleAddSegmentGroup = () => {
    const { segmentFilter } = this.state;
    const id = uuid();
    const childId = uuid();
    segmentFilter[id] = {
      id,
      operator: operatorOptions.and,
      segmentFilter: {
        [childId]: {
          id: childId,
          field: '',
          operator: '',
          value: ''
        }
      },
    };

    this.setState({
      segmentFilter,
    });
  }

  handleRemoveSegmentGroup = (id) => {
    const { segmentFilter } = this.state;
    delete segmentFilter[id];
    this.setState({
      segmentFilter,
    });
  }

  handleFilterChange = (item) => {
    const { segmentFilter } = this.state;
    segmentFilter[item.id] = item;
    this.setState({
      segmentFilter,
    });
  }

  checkValidFilterData() {
    const { operator, segmentFilter } = this.state;
    if (!operator) {
      return false;
    }
    let isValid = true;
    Object.values(segmentFilter).forEach(groupSegment => {
      const { operator: childOperator, segmentFilter: childSegmentFilter } = groupSegment;
      if (!childOperator) {
        isValid = false;
      }
      Object.values(childSegmentFilter).forEach(segment => {
        Object.values(segment).forEach(field => {
          if (!field) {
            isValid = false;
          }
        })
      })
    })
    return isValid;
  }

  handleApplyFilter = () => {
    const { operator, segmentFilter, isHitSave } = this.state;
    if (!isHitSave) {
      this.setState({
        isHitSave: true,
      })
    }
    if (this.checkValidFilterData()) {
      const { dispatch } = this.context;
      this.setState({
        isHitSave: false,
      })
      dispatch(ContactActionCreator.getContactsFilterSegment(1, {
        operator,
        segmentFilter,
      }))
    }
  }

  render() {
    const { classes } = this.props;
    const { intl: { messages } } = this.context
    const matches = Object.keys(operatorOptions).map(key => ({
      key,
      text: messages[`common.advanced_search.${key}`],
      value: operatorOptions[key],
    }));

    const { operator, segmentFilter, isHitSave } = this.state;
    return (
      <div className={classes.filterEditor}>
        <div className='match-operator-container'>
          <span>Matches</span>
          <Select
            className='match-operator'
            options={matches}
            value={operator}
            onChange={this.handleChange}
          />
          <span>of the following</span>
        </div>
        {
          Object.values(segmentFilter).map((item, index) => (
            <SegmentGroup
              key={item.id}
              index={index}
              parentOperator={operator}
              segmentGroup={item}
              handleChange={this.handleFilterChange}
              handleRemoveSegmentGroup={this.handleRemoveSegmentGroup}
              isHitSave={isHitSave}
            />
          ))
        }
        <Button
          fluid
          onClick={this.handleAddSegmentGroup}
          className='btn-add-segment-group'
        >
          Add Criteria
          <Icon size={"small"} name='add' circular color='black' />
        </Button>
        <Button color='green' floated='right' onClick={this.handleApplyFilter}>
          Apply Filter
        </Button>
      </div>
    );
  }
}

FilterEditor.propTypes = {
  criteriaInfo: PropTypes.shape({}),
};

FilterEditor.defaultProps = {
  criteriaInfo: {},
}

FilterEditor.contextType = AppContext;

export default injectSheet(stylesheet)(FilterEditor);

