import React from 'react';
import PropTypes from 'prop-types';
import isequal from 'lodash.isequal';
import debounce from 'lodash.debounce';
import { Form } from "semantic-ui-react";
import injectSheet from "react-jss";
import Enum from "../../../../Common/Constants/Enum";
import AppContext from "../../../../Common/Constants/AppContext";
import SearchGroup from './SearchGroup';
import stylesheet from './stylesheet';

const defaultFilter = {
  type: '',
  'volunteer.status': '',
  'volunteer.roles': '',
  selectedState: '',
  selectedVotingDistrict: {name: 'All'},
  'payments.membership_class': '',
  'payments.payment_type': ''
};

class ContactFilter extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      ...defaultFilter,
      ...props.initFilter,
    }
    this.handleChange = this.handleChange.bind(this);
    this.handleResetFilter = this.handleResetFilter.bind(this);
    this.onSearchContactChange = this.onSearchContactChange.bind(this);
    this.buildSearchQuery = debounce(this.buildSearchQuery.bind(this), 300);
    this.handleSearch = debounce(this.handleSearch.bind(this), 300);
  }

  handleResetFilter() {
    this.setState({
      ...defaultFilter,
    });
    const { onSearchFilterChange } = this.props;
    onSearchFilterChange(defaultFilter);
  }

  handleChange(e, { name, value }) {
    const filter = {
      ...this.state,
      [name]: value,
    };
    if (isequal(filter, this.state)) {
      return;
    }
    this.setState({
      ...filter,
    });
    this.handleSearch(filter);
  }

  handleSearch = (filter) => {
    const { onSearchFilterChange } = this.props;
    onSearchFilterChange(filter);
  };

  handleSelectElectorate = (selectedElectorate) => {
    const filter = {
      ...this.state,
      selectedVotingDistrict: selectedElectorate,
    };
    if (isequal(filter, this.state)) {
      return;
    }
    this.setState({
      ...filter,
    });

    const { onSearchFilterChange } = this.props;
    onSearchFilterChange(filter);

  }

  buildSearchQuery() {
    const {filter: oldFilter, selectedState, searchText, searchKey} = this.state;
    const filter = {...oldFilter, searchKey, searchText};
    if (selectedState || searchText) {
      filter["$and"] = [];
      if (selectedState) {
        const stateSearch = { "$or": [{ "home_address.state": selectedState }, { "work_address.state": selectedState }, { "postal_address.state": selectedState }] };
        filter["$and"].push(stateSearch);
      }
      if (searchText) {
        filter["$and"].push({"$text": {"$search": searchText}});
      }
    } else {
      filter["$and"] = null;
    }
    this.setState({
      filter,
    });
    const { onSearchFilterChange } = this.props;
    onSearchFilterChange(filter);
  }

  onSearchContactChange({searchKey, searchText}) {
    this.setState({searchText, searchKey}, this.buildSearchQuery);
  }

  render() {
    const { classes, showNumber, total, contactPaymentType } = this.props;
    const { intl: { messages } } = this.context;
    const emptyFilter = {
      key: 'all',
      text: messages.all,
      value: '',
    };
    const types = Object.values(Enum.ContactTypeEnum).map(item => ({
      key: item,
      text: messages[item],
      value: item
    }));
    types.unshift(emptyFilter);
    const volunteerStatus = Object.values(Enum.VolunteerStatus).map(item => ({
      key: item,
      text: messages[item],
      value: item
    }));
    volunteerStatus.unshift(emptyFilter);

    const volunteerRoles = Enum.VolunteerRoles.map(item=> ({
      key: item,
      text: item,
      value: item,
    }));
    volunteerRoles.unshift(emptyFilter);

    const stateList = Object.keys(Enum.StateAustralia).map(item => ({
      key: item,
      value: item,
      text: Enum.StateAustralia[item],
    }));
    stateList.unshift(emptyFilter);

    const membershipClasses = Object.values(Enum.MembershipClasses).map(item => ({
      key: item,
      value: item,
      text: item,
    }))
    membershipClasses.unshift(emptyFilter)

    const donorTypes = Object.values(Enum.DonationTypeEnum).map(item => ({
      key: item,
      value: item,
      text: messages[item],
    }))
    donorTypes.unshift(emptyFilter)

    const { type, selectedState } = this.state;
    return (
      <div className={classes.searchFilter}>
        <div className='reset-filter'>
          <span onClick={this.handleResetFilter}>Reset filters</span>
          <img src={require('../../../../Assets/reset.png')} alt='' onClick={this.handleResetFilter} />
        </div>
        <div className='search-container'>
          <div className='search-box'>
            <div className='text-title'>Search by</div>
            <div className='control'>
              <SearchGroup onSearch={this.onSearchContactChange}/>
            </div>
          </div>
          <div className='filter-box'>
            {
              (contactPaymentType === 'all' || contactPaymentType === 'donor') && 
              <div className='block-item'>
                <div className='text-title'>{messages.type}</div>
                <div className='control'>
                  <Form.Select
                    options={types}
                    placeholder={messages.all}
                    name='type'
                    value={type}
                    onChange={this.handleChange}
                  />
                </div>
              </div>
            }
            {
              contactPaymentType === 'donor' &&
              <div className='block-item'>
                <div className='text-title'>{messages['contact.list.filter.donor_type']}</div>
                <div className='control'>
                  <Form.Select
                    options={donorTypes}
                    placeholder={messages.all}
                    className='state-dropdown'
                    name='payments.payment_type'
                    value={this.state['payments.payment_type']}
                    onChange={this.handleChange}
                  />
                </div>
              </div>
            }
            {
              contactPaymentType === 'membership' &&
              <div className='block-item'>
                <div className='text-title'>{messages['contact.list.filter.membership_class']}</div>
                <div className='control'>
                  <Form.Select
                    options={membershipClasses}
                    placeholder={messages.all}
                    name='payments.membership_class'
                    value={this.state['payments.membership_class']}
                    onChange={this.handleChange}
                  />
                </div>
              </div>
            }
            <div className='block-item'>
              <div className='text-title'>{messages.state}</div>
              <div className='control'>
                <Form.Select
                  options={stateList}
                  placeholder={messages.all}
                  className='state-dropdown'
                  name='selectedState'
                  value={selectedState}
                  onChange={this.handleChange}
                />
              </div>
            </div>
            {/* {
              contactPaymentType === 'all' && 
              <div className='block-item'>
              <div className='text-title'>{messages.electorate}</div>
              <div className='control'>
                <ElectorateComponent
                  onSelect={this.handleSelectElectorate}
                  defaultSelected={selectedVotingDistrict}
                  allowAll
                />
              </div>
            </div>
            } */}
            { contactPaymentType === 'volunteer' &&
              <div className='block-item'>
              <div className='text-title'>{messages.volunteerStatus}</div>
              <div className='control'>
                <Form.Select
                  options={volunteerStatus}
                  placeholder={messages.all}
                  name='volunteer.status'
                  value={this.state['volunteer.status']}
                  onChange={this.handleChange}
                />
              </div>
            </div>
            }
            { contactPaymentType === 'volunteer' &&
              <div className='block-item volunteer-roles'>
              <div className='text-title'>{messages.volunteerRoles}</div>
              <div className='control'>
                <Form.Select
                  options={volunteerRoles}
                  placeholder={messages.all}
                  name='volunteer.roles'
                  value={this.state['volunteer.roles']}
                  onChange={this.handleChange}
                />
              </div>
            </div>
            }
          </div>
        </div>
        <div className='display-contact-number'>
          Showing <b>{showNumber}</b> of <b>{total}</b> contacts
        </div>
      </div>
    )
  }
}

ContactFilter.contextType = AppContext;

ContactFilter.propTypes = {
  onSearchFilterChange: PropTypes.func.isRequired,
  showNumber: PropTypes.number,
  total: PropTypes.number,
  initFilter: PropTypes.shape({}),
};

ContactFilter.defaultProps = {
  showNumber: 0,
  total: 0,
  initFilter: {},
}

export default injectSheet(stylesheet)(ContactFilter);
