import React from 'react';
import PropTypes from 'prop-types';
import isequal from "lodash.isequal";
import { Button, Form, Checkbox, Modal } from 'semantic-ui-react';
import injectSheet from "react-jss";
import AppContext from "../../../Common/Constants/AppContext";
import Utils from '../../../Common/Utils/Utils';
import Enum from '../../../Common/Constants/Enum';
import { ModalStyleSheet } from "../../../Common/GlobalStyle";
import {compose} from 'redux';
import {connect} from 'react-redux';

const style = {
  groupOfType: {
    justifyContent: 'center',
  },
  toggleContactType: {
    display: 'flex',
    alignItems: 'center',
    '& .ui.toggle.checkbox .box:before, .ui.toggle.checkbox label:before': {
      backgroundColor: '#0d71bb!important',
    }
  },
  helpText: {
    extend: ModalStyleSheet.helpText,
  },
  actionGroup: {
    extend: ModalStyleSheet.actionGroup,
  },
  header: {
    extend: ModalStyleSheet.header,
  },
}

class ContactInfoModal extends React.Component {
  constructor(props) {
    super(props);
    let emails = [
      {
        email: '',
        primary: true,
      }
    ];
    if(props.contactInfo.emails && props.contactInfo.emails.length > 0) {
      emails = props.contactInfo.emails;
    }
    this.state = {
      contactInfo: {
        type: 'person',
        ...props.contactInfo,
        emails,
      },
    };
    this.isFieldError = this.isFieldError.bind(this);
    this.handleChange = this.handleChange.bind(this);
    this.handleSubmit = this.handleSubmit.bind(this);
    this.handleChangeRadio = this.handleChangeRadio.bind(this);
  }

  handleChange({ target: { name, value } }) {
    if (name === 'email') {
      const emails = [...this.state.contactInfo.emails];
      emails[0] = {
        ...emails[0],
        email: value,
      };
      this.setState({
        contactInfo: {
          ...this.state.contactInfo,
          emails,
        }
      })
    } else {
      this.setState({
        contactInfo: {
          ...this.state.contactInfo,
          [name]: value,
        }
      })
    }
  }

  handleChangeRadio() {
    const { contactInfo } = this.state;
    this.setState({
      contactInfo: {
        ...contactInfo,
        type: contactInfo.type === Enum.ContactTypeEnum.organisation ?
          Enum.ContactTypeEnum.person : Enum.ContactTypeEnum.organisation,
      }
    })
  }

  handleSubmit() {
    if (!this.state.isHitSave) {
      this.setState({
        isHitSave: true,
      });
    }
    if (this.state.contactInfo.type === Enum.ContactTypeEnum.person) {
      if (this.isFieldError('email') ||
        this.isFieldError('first_name') ||
        this.isFieldError('last_name')
      ) {
        return;
      }
    } else {
      if (this.isFieldError('email') ||
        this.isFieldError('abn') ||
        this.isFieldError('organisation_name')
      ) {
        return;
      }
    }
    const { onOk } = this.props;
    onOk(this.state.contactInfo);
  }

  isFieldError(name) {
    const { intl: { messages } } = this.context;
    if (name === 'email') {
      if (this.state.contactInfo.emails[0].email && !Utils.validateEmail(this.state.contactInfo.emails[0].email)) {
        return 'Please input a valid email';
      }
    } else if (!this.state.contactInfo[name]) {
      return `${messages.pleaseEnter} ${messages[name]}`;
    }
    return '';
  }

  render() {
    const { classes, isOpen, onClose , contactTitles} = this.props;
    const { intl: { messages } } = this.context;
    const { contactInfo } = this.state;
    const emptyValue = {
      key: 'select',
      text: messages.select,
      value: '',
    };
    const prefix = contactTitles.map(item => ({
      key: item._id,
      value: item.title,
      text: item.title
    }));
    prefix.unshift(emptyValue);
    return (
      <Modal
        open={isOpen}
        onClose={onClose}
        size="mini"
      >
        <Modal.Header className={classes.header}>
          {
            contactInfo._id ? messages.updateContact : messages.addNewContact
          }
        </Modal.Header>
        <Modal.Content>
          <Form
            onSubmit={this.handleSubmit}
          >
            <Form.Group inline className={classes.groupOfType}>
              <Form.Field label={messages.person}></Form.Field>
              <Form.Field
                className={classes.toggleContactType}
                control={Checkbox}
                toggle
                name='type'
                checked={contactInfo.type !== Enum.ContactTypeEnum.person}
                onChange={this.handleChangeRadio}
              />
              <Form.Field label={messages.organisation}></Form.Field>
            </Form.Group>
            {
              contactInfo.type === Enum.ContactTypeEnum.person ? [
                <Form.Field>
                  <label>{messages.prefixName}</label>
                  <Form.Select
                    options={prefix}
                    placeholder={messages.select}
                    name='prefix_name'
                    value={contactInfo.prefix_name}
                    onChange={(e, target) => this.handleChange({ target: target })}
                  />
                </Form.Field>,
                <Form.Field required error={this.state.isHitSave && this.isFieldError('first_name')}>
                  <label>{messages.firstName}</label>
                  <input
                    placeholder={messages.firstName}
                    name='first_name'
                    value={contactInfo.first_name}
                    onChange={this.handleChange}
                  />
                  {
                    this.state.isHitSave && this.isFieldError('first_name') &&
                    <div className={classes.helpText}>{this.isFieldError('first_name')}</div>
                  }
                </Form.Field>,
                <Form.Field required error={this.state.isHitSave && this.isFieldError('last_name')}>
                  <label>{messages.lastName}</label>
                  <input
                    placeholder={messages.lastName}
                    name='last_name'
                    value={contactInfo.last_name}
                    onChange={this.handleChange}
                  />
                  {
                    this.state.isHitSave && this.isFieldError('last_name') &&
                    <div className={classes.helpText}>{this.isFieldError('last_name')}</div>
                  }
                </Form.Field>,
                <Form.Field>
                  <label>{messages.organisationName}</label>
                  <input
                    placeholder={messages.organisationName}
                    name='related_organisation'
                    value={contactInfo.related_organisation}
                    onChange={this.handleChange}
                  />
                </Form.Field>
              ] : [
                <Form.Field required error={this.state.isHitSave && this.isFieldError('organisation_name')}>
                  <label>{messages.organisationName}</label>
                  <input
                    placeholder={messages.organisationName}
                    name='organisation_name'
                    value={contactInfo.organisation_name}
                    onChange={this.handleChange}
                  />
                  {
                    this.state.isHitSave && this.isFieldError('organisation_name') &&
                    <div className={classes.helpText}>{this.isFieldError('organisation_name')}</div>
                  }
                </Form.Field>,
                <Form.Field required error={this.state.isHitSave && this.isFieldError('abn')}>
                  <label>{messages.abn}</label>
                  <input
                    placeholder={messages.abn}
                    name='abn'
                    value={contactInfo.abn}
                    onChange={this.handleChange}
                  />
                  {
                    this.state.isHitSave && this.isFieldError('abn') &&
                    <div className={classes.helpText}>{this.isFieldError('abn')}</div>
                  }
                </Form.Field>,
              ]
            }
            <Form.Field>
              <label>{messages.mobile}</label>
              <input
                placeholder={`${messages.mobile} (${messages.optional})`}
                name='mobile_number'
                value={contactInfo.mobile_number}
                onChange={this.handleChange}
              />
            </Form.Field>
            <Form.Field error={this.state.isHitSave && this.isFieldError('email')}>
              <label>{messages.email}</label>
              <input
                placeholder={messages.email}
                name='email'
                value={contactInfo.emails[0].email}
                onChange={this.handleChange}
              />
              {
                this.state.isHitSave && this.isFieldError('email') &&
                <div className={classes.helpText}>{this.isFieldError('email')}</div>
              }
            </Form.Field>
            <Form.Field className={classes.actionGroup}>
              <Button onClick={onClose}>
                {messages.cancel}
              </Button>
              <Button
                type='submit'
                color='green'
                disabled={isequal(this.props.contactInfo, contactInfo)}
              >
                {
                  contactInfo._id ? messages.update : messages.add
                }
              </Button>
            </Form.Field>
          </Form>
        </Modal.Content>
      </Modal>
    )
  }
}

ContactInfoModal.propTypes = {
  isOpen: PropTypes.bool.isRequired,
  onClose: PropTypes.func.isRequired,
  onOk: PropTypes.func.isRequired,
  contactInfo: PropTypes.shape({}).isRequired,
  contactData: PropTypes.object.isRequired,
  contactTitles: PropTypes.array.isRequired
};

ContactInfoModal.contextType = AppContext;

const mapStateToProps = ({settings}) => ({
  contactTitles: settings.contactTitles
})

export default compose(
  injectSheet(style),
  connect(mapStateToProps)
)(ContactInfoModal);
