import React from 'react'
import PropTypes from "prop-types"
import { Button, Modal, Form, Message, Input } from "semantic-ui-react"
import { CardNumberElement, CardExpiryElement, injectStripe } from 'react-stripe-elements'
import isequal from "lodash.isequal"
import injectSheet from "react-jss"
import { ModalStyleSheet } from "../../../Common/GlobalStyle"
import AppContext from "../../../Common/Constants/AppContext"
import Enum from "../../../Common/Constants/Enum"
import Checkbox from "../../../Common/Components/Base/Checkbox"
import { toast } from "react-toastify"
import PaymentSourceSelector from '../Common/PaymentSourceSelector'
import isEmpty from 'lodash.isempty'
import get from 'lodash/get'
import DatePicker from "react-datepicker/es";
import { config } from '../../../configuration';
import moment from 'moment'

const stylesheet = {
  ...ModalStyleSheet,
  cardElement: {
    border: '1px solid rgba(34,36,38,.15)',
    borderRadius: '3px',
    padding: '10px'
  },
  note: {
    fontSize: 13,
    color: "#7c7c7c",
    "& span": {
      fontWeight: 600
    }
  },
  modalContent: {
    '& .react-datepicker-wrapper': {
      width: '100%'
    }
  } 
}

class AssignMembershipModal extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      title: 'Assign Membership',
      actionButtonName: 'Assign',
      assignMembershipInfo: {},
      source: (this.props.contactDetail.stripe_customer && this.props.contactDetail.stripe_customer.default_source) || 'new_source'
    };
    this.handleChange = this.handleChange.bind(this);
    this.handleSubmit = this.handleSubmit.bind(this);
  }

  handleChange({ target: { name, value, checked } }) {
    this.setState({
      assignMembershipInfo: {
        ...this.state.assignMembershipInfo,
        [name]: name === 'auto_renew' ? checked : value || ''
      }
    }, () => {
      if(name === 'membership_id') {
        //Prefill price
        const selectedMembership = this.props.memberships
          .find(membership => membership._id === value)
          selectedMembership && this.setState({
          assignMembershipInfo: {
            ...this.state.assignMembershipInfo,
            price: selectedMembership.price
          }  
        })

        // Prefill period
        if(!this.state.assignMembershipInfo.start_date) {
          this.setState((prev) => ({
            assignMembershipInfo: {
              ...prev.assignMembershipInfo,
              start_date: moment().startOf('day').toDate().getTime(),
            }
          }));
        }
        const latestMembershipOrders = this.props.membershipOrders
          .sort((a,b) => b.membership_period.end_date - a.membership_period.end_date)
          .filter(membershipOrder => membershipOrder.membership._id === value)
        if(latestMembershipOrders[0]) {
          this.setState({
            title: 'Renew Membership', 
            actionButtonName: 'Renew', 
          })
          this.handleChangeStartDate(latestMembershipOrders[0].membership_period.end_date)  
          // Set start date from the latest expiry
        } else {
          this.setState({title: 'Assign Membership', actionButtonName: 'Assign'})
          this.handleChangeStartDate(this.state.assignMembershipInfo.start_date)
        }
      }
    })
  }

  getPeriodType(period){
    if(period === 'daily') return 'day';
    if(period === 'monthly') return 'month';
    if(period === 'yearly') return 'year';

    return 'day'
  }

  handleChangeStartDate = (value) => {
    const {assignMembershipInfo: {membership_id}} = this.state
    const membership = this.props.memberships.find(item => item._id == membership_id) || {};
    const periodType = this.getPeriodType(membership.period)

    this.setState((prev) => ({
      assignMembershipInfo: {
        ...prev.assignMembershipInfo,
        start_date: moment(value).startOf('day').toDate().getTime(),
        expiry_date: moment(value).add(1, periodType).endOf('day').toDate().getTime()
      }
    }));
  }

  handleChangeExpiryDate = (value) => {
    this.setState((prev) => ({
      assignMembershipInfo: {
        ...prev.assignMembershipInfo,
        expiry_date: moment(value).toDate().getTime()
      }
    }));
  }

  async handleSubmit() {
    if (!this.state.isHitSave) {
      this.setState({
        isHitSave: true,
      });
    }
    const assignMembershipInfo = this.state.assignMembershipInfo;
    if (!assignMembershipInfo.tender || !assignMembershipInfo.membership_id) return;
    const { onAssign } = this.props;
    if(assignMembershipInfo.tender === 'credit_card_stripe') {
      const { contactDetail: {first_name, last_name} } = this.props;
      try {
        let source = this.state.source
        if(this.state.source === 'new_source') {
          const { token, error } = await this.props.stripe.createToken({name: first_name + ' ' + last_name});
          if(error) {
            toast.error(error.message)
          }
          if(!token) return;
          source = token.id
        }
        onAssign({...assignMembershipInfo, ...{ stripe_token_id: source }});
      } catch (error) {
        toast.error(error.message)
      }
    } else {
      onAssign({...assignMembershipInfo});
    }
  }

  onSelectSource = (source) => {
    this.setState({source})
  }

  render() {
    const { classes, isOpen, onClose, memberships, contactDetail } = this.props;
    const { intl: { messages } } = this.context;
    const { assignMembershipInfo } = this.state;
    const emptyFilter = {
      key: 'select',
      text: 'Select Tender',
      value: '',
    };
    const tenderList = Object.keys(Enum.Tender).map(item => ({
      key: item,
      value: item,
      text: Enum.Tender[item],
    }));
    tenderList.unshift(emptyFilter);

    const isHasAddress = 
      !isEmpty(contactDetail?.home_address) || 
      !isEmpty(contactDetail?.postal_address) || 
      !isEmpty(contactDetail?.work_address);

    const isHasEmail = !isEmpty(contactDetail?.emails);

    const contactName = [contactDetail?.first_name, contactDetail?.last_name]
      .filter((item) => item)
      .join(" ");

    const startDate = get(this.state.assignMembershipInfo, 'start_date', '')
    const expiryDate = get(this.state.assignMembershipInfo, 'expiry_date', '')

    return (
      <Modal
        open={isOpen}
        onClose={onClose}
        size="tiny"
      >
        <Modal.Header className={classes.header}>
          {this.state.title}
        </Modal.Header>
        <Modal.Content className={classes.modalContent}>
        {!isHasAddress && (
            <Message error>
              <Message.Header>Unable to Proceed</Message.Header>
              <p>The contact must have a current address</p>
            </Message>
          )}
          {!isHasEmail && (
            <p className={classes.note}>
              <span>Note.</span> {contactName} dose not have an email address listed and will
              not receive receipts or follow-up communications by email.
            </p>
          )}
          <Form
            onSubmit={this.handleSubmit}
          >
            <Form.Group widths='equal'>
              <Form.Field required error={this.state.isHitSave && !assignMembershipInfo.membership_id}>
                <label>Select membership</label>
                <Form.Select
                  options={memberships && memberships.map(m => { return {key: m._id, value: m._id, text: `${m.state} ${m.membership_class} ($${m.price})`}})}
                  placeholder={messages.memberships}
                  name='membership_id'
                  value={assignMembershipInfo.membership_id}
                  onChange={(e, target) => this.handleChange({ target: target })}
                />
              </Form.Field>
              <Form.Field required error={this.state.isHitSave && !assignMembershipInfo.price}>
                <label>Price</label>
                <Input 
                  type="number" 
                  placeholder='Price'
                  name='price'
                  value={assignMembershipInfo.price}
                  onChange={this.handleChange}
                  label={{ basic: true, content: '$' }}
                  labelPosition='left'
                  >
                </Input>
              </Form.Field>
            </Form.Group>
            <Form.Group widths='equal'>
              <Form.Field required error={this.state.isHitSave && !assignMembershipInfo.tender}>
                <label>{messages.tender}</label>
                <Form.Select
                  options={tenderList}
                  placeholder={messages.tender}
                  className='state-dropdown'
                  name='tender'
                  value={assignMembershipInfo.tender}
                  onChange={(e, target) => this.handleChange({ target: target })}
                />
              </Form.Field>
              <Form.Field>
                <label>Campaign ID</label>
                <input
                  placeholder='Campaign ID (optional)'
                  name='campaign_id'
                  value={assignMembershipInfo.campaign_id}
                  onChange={this.handleChange}
                />
              </Form.Field>
            </Form.Group>
            <Form.Group widths='equal'>
              <Form.Field>
                <label>{messages.startDate}</label>
                <DatePicker
                  className="datepicker"
                  selected={startDate ? moment(startDate).toDate() : new Date()}
                  onChange={this.handleChangeStartDate}
                  placeholderText={config.DATE_FORMAT}
                  value={startDate ? moment(startDate).toDate() : ''}
                  dateFormat='dd/MM/yyyy'
                  todayButton={'Today'}
                />
              </Form.Field>
              <Form.Field>
                <label>{messages.expiryDate}</label>
                <DatePicker
                  className="datepicker"
                  selected={expiryDate ? moment(expiryDate).toDate() : new Date()}
                  onChange={this.handleChangeExpiryDate}
                  dateFormat='dd/MM/yyyy'
                  placeholderText={config.DATE_FORMAT}
                  value={expiryDate ? moment(expiryDate).toDate() : ''}
                  todayButton={'Today'}
                />
              </Form.Field>
            </Form.Group>
            {
              assignMembershipInfo.tender === 'credit_card_stripe' && 
              <PaymentSourceSelector contact={contactDetail} onSelectSource={this.onSelectSource} currentSource={this.state.source} />
            }
            {
              assignMembershipInfo.tender === 'credit_card_stripe' && this.state.source === 'new_source' &&
              <Form.Group>
                <Form.Field width={12}>
                  <label>Credit or debit card</label>
                  <div className={classes.cardElement}><CardNumberElement/></div>
                </Form.Field>
                <Form.Field width={4}>
                  <label>Expiry</label>
                  <div className={classes.cardElement}><CardExpiryElement/></div>
                </Form.Field>
              </Form.Group>
            }
            {
              assignMembershipInfo.tender === 'credit_card_stripe' && <Form.Field>
                <Checkbox 
                  name='auto_renew'
                  label="Auto renew"
                  checked={assignMembershipInfo.autoRenew}
                  onChange={(e, target) => this.handleChange({ target: target })} />
              </Form.Field>
            }
            <Form.Field className={classes.actionGroup}>
              <Button onClick={onClose}>
                {messages.cancel}
              </Button>
              <Button
                type='submit'
                color={!isHasAddress ? "grey" : "green"}
                disabled={!isHasAddress || isequal(this.props.assignMembershipInfo, assignMembershipInfo)}
              >
                {this.state.actionButtonName}
              </Button>
            </Form.Field>
          </Form>
        </Modal.Content>
      </Modal>
    );
  }
}

AssignMembershipModal.propTypes = {
  isOpen: PropTypes.bool.isRequired,
  onClose: PropTypes.func.isRequired
};

AssignMembershipModal.contextType = AppContext;

export default injectSheet(stylesheet)(injectStripe(AssignMembershipModal));
