import React from 'react'
import PropTypes from "prop-types"
import { Button, Modal, Form } from "semantic-ui-react"
import isequal from "lodash.isequal"
import injectSheet from "react-jss"
import { ModalStyleSheet } from "../../../Common/GlobalStyle"
import AppContext from "../../../Common/Constants/AppContext"
import get from 'lodash/get'
import DatePicker from "react-datepicker/es";
import { config } from '../../../configuration';
import moment from 'moment'

const stylesheet = {
  ...ModalStyleSheet,
  cardElement: {
    border: '1px solid rgba(34,36,38,.15)',
    borderRadius: '3px',
    padding: '10px'
  },
  note: {
    fontSize: 13,
    color: "#7c7c7c",
    "& span": {
      fontWeight: 600
    }
  },
  modalContent: {
    '& .react-datepicker-wrapper': {
      width: '100%'
    }
  } 
}

class NewMembershipModal extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      renewMembershipInfo: this._calculateStartAndExpiryDate({startDate: new Date(), period:this.props.selectedOrder.membership.period}),
      selectedMembership: this.props.selectedOrder.membership
    };
    this.handleChange = this.handleChange.bind(this);
    this.handleSubmit = this.handleSubmit.bind(this);
  }

  _calculateStartAndExpiryDate = ({startDate, period}) => {
    const periodType = this.getPeriodType(period)
    return {
      start_date: moment(startDate).startOf('day').toDate().getTime(),
      expiry_date: moment(startDate).add(1, periodType).endOf('day').toDate().getTime()
    }
  }

  handleChange({ target: { name, value, checked } }) {
    this.setState({
      renewMembershipInfo: {
        ...this.state.renewMembershipInfo,
        [name]: value ? value : checked,
      }
    }, () => {
      if(name === 'membership_id') {
        if(!this.state.renewMembershipInfo.start_date) {
          this.setState((prev) => ({
            renewMembershipInfo: {
              ...prev.renewMembershipInfo,
              start_date: moment().startOf('day').toDate().getTime(),
            }
          }));
        }
        this.handleChangeStartDate(this.state.renewMembershipInfo.start_date)
      }
    })
  }

  getPeriodType(period){
    if(period === 'daily') return 'day';
    if(period === 'monthly') return 'month';
    if(period === 'yearly') return 'year';

    return 'day'
  }

  handleChangeStartDate = (value) => {
    const periodType = this.getPeriodType(this.state.selectedMembership.period)
    this.setState((prev) => ({
      renewMembershipInfo: {
        ...prev.renewMembershipInfo,
        start_date: moment(value).startOf('day').toDate().getTime(),
        expiry_date: moment(value).add(1, periodType).endOf('day').toDate().getTime()
      }
    }));
  }

  handleChangeExpiryDate = (value) => {
    this.setState((prev) => ({
      renewMembershipInfo: {
        ...prev.renewMembershipInfo,
        expiry_date: moment(value).toDate().getTime()
      }
    }));
  }

  async handleSubmit() {
    if (!this.state.isHitSave) {
      this.setState({
        isHitSave: true,
      });
    }
    const renewMembershipInfo = this.state.renewMembershipInfo;
    const { onRenew } = this.props;
    // Only allow renew once off membership. For subscription membership, need to cancel and re-create new membership
    onRenew({...renewMembershipInfo});
  }

  render() {
    const { classes, isOpen, onClose } = this.props;
    const { intl: { messages } } = this.context;
    const { renewMembershipInfo } = this.state;

    const startDate = get(this.state.renewMembershipInfo, 'start_date', '')
    const expiryDate = get(this.state.renewMembershipInfo, 'expiry_date', '')
    
    return (
      <Modal
        open={isOpen}
        onClose={onClose}
        size="tiny"
      >
        <Modal.Header className={classes.header}>
          Renew Membership - {
            this.props.selectedOrder && this.props.selectedOrder.membership.membership_class + ' ' + 
            this.props.selectedOrder.membership.state
          }
        </Modal.Header>
        <Modal.Content className={classes.modalContent}>
          <Form
            onSubmit={this.handleSubmit}
          >
            <Form.Group widths='equal'>
              <Form.Field>
                <label>{messages.startDate}</label>
                <DatePicker
                  className="datepicker"
                  selected={startDate ? moment(startDate).toDate() : new Date()}
                  onChange={this.handleChangeStartDate}
                  placeholderText={config.DATE_FORMAT}
                  value={startDate ? moment(startDate).toDate() : ''}
                  dateFormat='dd/MM/yyyy'
                  todayButton={'Today'}
                />
              </Form.Field>
              <Form.Field>
                <label>{messages.expiryDate}</label>
                <DatePicker
                  className="datepicker"
                  selected={expiryDate ? moment(expiryDate).toDate() : new Date()}
                  onChange={this.handleChangeExpiryDate}
                  dateFormat='dd/MM/yyyy'
                  placeholderText={config.DATE_FORMAT}
                  value={expiryDate ? moment(expiryDate).toDate() : ''}
                  todayButton={'Today'}
                />
              </Form.Field>
            </Form.Group>
            <Form.Field className={classes.actionGroup}>
              <Button onClick={onClose}>
                {messages.cancel}
              </Button>
              <Button
                type='submit'
                color={isequal(this.props.renewMembershipInfo, renewMembershipInfo) ? "grey" : "green"}
                disabled={isequal(this.props.renewMembershipInfo, renewMembershipInfo)}
              >
                Renew
              </Button>
            </Form.Field>
          </Form>
        </Modal.Content>
      </Modal>
    );
  }
}

NewMembershipModal.propTypes = {
  isOpen: PropTypes.bool.isRequired,
  onClose: PropTypes.func.isRequired
};

NewMembershipModal.contextType = AppContext;

export default injectSheet(stylesheet)(NewMembershipModal);
