import React from 'react';
import PropTypes from 'prop-types';
import isequal from "lodash.isequal";
import { Button, Form, Modal } from 'semantic-ui-react';
import injectSheet from "react-jss";
import { ModalStyleSheet } from "../../../../Common/GlobalStyle";
import AppContext from "../../../../Common/Constants/AppContext";
import ContactTaskActionCreator from "../../../Actions/ContactTaskActionCreator";
import Enum from "../../../../Common/Constants/Enum";
import DatePicker from "react-datepicker/es";

const stylesheet = {
  ...ModalStyleSheet,
  taskModal: {
    '& .react-datepicker-wrapper': {
      width: '100%',
      '& .react-datepicker__input-container': {
        width: '100%',
      }
    }
  }
}

class TaskInfoModal extends React.Component {
  constructor(props) {
    super(props);
    const taskInfo = { ...props.taskInfo };
    if (taskInfo.due_date) {
      taskInfo.due_date = new Date(taskInfo.due_date);
    }
    this.state = {
      taskInfo,
    };
    this.isFieldError = this.isFieldError.bind(this);
    this.handleChange = this.handleChange.bind(this);
    this.handleChangeDate = this.handleChangeDate.bind(this);
    this.handleSubmit = this.handleSubmit.bind(this);
  }

  handleChange({ target: { name, value } }) {
    this.setState({
      taskInfo: {
        ...this.state.taskInfo,
        [name]: value,
      }
    })
  }

  handleChangeDate(value) {
    const taskInfo = { ...this.state.taskInfo };
    const date = value || new Date();
    taskInfo.due_date = date;
    this.setState({
      taskInfo,
    });
  }

  handleSubmit() {
    if (!this.state.isHitSave) {
      this.setState({
        isHitSave: true,
      });
    }
    if (this.isFieldError('task') || this.isFieldError('due_date')) {
      return;
    }
    const taskInfo = {
      ...this.state.taskInfo,
      due_date: this.state.taskInfo.due_date.getTime(),
    };
    const { dispatch } = this.context;
    const { onClose } = this.props;

    if (!taskInfo._id) {
      dispatch(ContactTaskActionCreator.createContactTasks(taskInfo));
    } else {
      dispatch(ContactTaskActionCreator.saveContactTasks(taskInfo));
    }
    onClose();
  }

  isFieldError(name) {
    if (!this.state.taskInfo[name]) {
      return true;
    }
    return false;
  }

  render() {
    const { classes, isOpen, onClose } = this.props;
    const { intl: { messages } } = this.context;
    const { taskInfo } = this.state;
    const contactNotes = Object.values(Enum.ContactNotes).map(item => ({
      key: item,
      value: item,
      text: messages[item],
    }));
    return (
      <Modal
        open={isOpen}
        onClose={onClose}
        size="tiny"
        className={classes.taskModal}
      >
        <Modal.Header className={classes.header}>
          {
            taskInfo._id ? messages.editTask : messages.addTask
          }
        </Modal.Header>
        <Modal.Content>
          <Form
            onSubmit={this.handleSubmit}
          >
            <Form.Field required error={this.state.isHitSave && this.isFieldError('task')}>
              <Form.TextArea
                rows={5}
                placeholder={messages.addTask}
                name='task'
                value={taskInfo.task}
                onChange={this.handleChange}
              />
            </Form.Field>
            <Form.Group widths='equal'>
              <Form.Field required error={this.state.isHitSave && this.isFieldError('category')}>
                <label>{messages.category}</label>
                <Form.Select
                  options={contactNotes}
                  placeholder={messages.category}
                  name='category'
                  value={taskInfo.category}
                  onChange={(e, target) => this.handleChange({ target: target })}
                />
              </Form.Field>
              <Form.Field required error={this.state.isHitSave && this.isFieldError('due_date')}>
                <label>Due date</label>
                <DatePicker
                  selected={taskInfo.due_date}
                  onChange={this.handleChangeDate}
                  dateFormat='dd/MM/yyyy'
                  placeholderText='Due date'
                  todayButton={'Today'}
                />
              </Form.Field>
            </Form.Group>
            <Form.Field className={classes.actionGroup}>
              <Button onClick={onClose}>
                {messages.cancel}
              </Button>
              <Button
                type='submit'
                color='green'
                disabled={isequal(this.props.taskInfo, taskInfo) && taskInfo._id}
              >
                {
                  taskInfo._id ? messages.update : messages.create
                }
              </Button>
            </Form.Field>
          </Form>
        </Modal.Content>
      </Modal>
    )
  }
}

TaskInfoModal.propTypes = {
  isOpen: PropTypes.bool.isRequired,
  onClose: PropTypes.func.isRequired,
  taskInfo: PropTypes.shape({}).isRequired,
  selectedContactId: PropTypes.string.isRequired,
};

TaskInfoModal.contextType = AppContext;

export default injectSheet(stylesheet)(TaskInfoModal);
