import React from 'react';
import { Loader, List, Button, Icon, Pagination, Input } from 'semantic-ui-react';
import injectSheet from "react-jss";

import ReduxStatus from "../../../../Common/Constants/ReduxStatus";
import stylesheet from "./stylesheet";
import AppContext from "../../../../Common/Constants/AppContext";
import moment from "moment/moment";
import TaskInfoModal from './TaskInfoModal';
import ContactTaskActionCreator from "../../../Actions/ContactTaskActionCreator";
import Confirmation from '../../../../Common/Components/Modal/ConfirmModal';
import Utils from "../../../../Common/Utils/Utils";
import Checkbox from "../../../../Common/Components/Base/Checkbox";

class Tasks extends React.Component {
  constructor() {
    super();
    this.state = {
      toggleEdit: false,
      taskInfo: {},

      selectedDeleteTaskId: '',
      toggleDelete: false,
    }
    this.handleDeleteTask = this.handleDeleteTask.bind(this);
    this.handlePaginationChange = this.handlePaginationChange.bind(this);
    this.onSearchFilterChange = this.onSearchFilterChange.bind(this);
  }

  componentDidMount() {
    const { dispatch } = this.context;
    const { selectedContact } = this.props;
    const filter = {
      contact: selectedContact._id,
    }
    dispatch(ContactTaskActionCreator.getContactTasks(1, filter));
  }

  handlePaginationChange(e, { activePage }) {
    const { dispatch } = this.context;
    const { contactTasks: { filter } } = this.props;
    dispatch(ContactTaskActionCreator.getContactTasks(activePage, filter))
  }

  onSearchFilterChange(e, { value }) {
    const { dispatch } = this.context;
    const { contactTasks } = this.props;
    const newFilter = { ...contactTasks.filter };
    if (value) {
      newFilter.task = { "$regex": value, "$options": "i" };
    } else {
      newFilter.task = null;
    }
    dispatch(ContactTaskActionCreator.getContactTasks(1, newFilter))
  }

  handleDeleteTask() {
    const { dispatch } = this.context;
    dispatch(ContactTaskActionCreator.deleteContactTasks(this.state.selectedDeleteTaskId));
    this.setState({
      selectedDeleteTaskId: '',
      toggleDelete: false,
    })
  }

  handleChangeTaskStatus = (task) => {
    const { dispatch } = this.context;
    const savedTask = {
      ...task,
      is_completed: !task.is_completed,
    }
    dispatch(ContactTaskActionCreator.saveContactTasks(savedTask));
  }

  renderDueDate = (task) => {
    const m = moment(task.due_date);
    const today = moment().startOf('day');
    const days = Math.round((m - today) / 86400000);
    if (days < 0) {
      return (
        <div className='due_date overdue'>
          Overdue
        </div>
      )
    } else if (days === 0) {
      return (
        <div className='due_date'>
          Today
        </div>
      )
    } else if (days === 1) {
      return (
        <div className='due_date'>
          Due in a day
        </div>
      )
    } else {
      return (
        <div className='due_date'>
          Due in {days} days
        </div>
      )
    }
  }

  render() {
    const { intl: { messages }, currentUser } = this.context;
    const { classes, selectedContact, contactTasks: { data, status, activePage, totalPage } } = this.props;
    return (
      <div className={classes.contactOverviewTask}>
        {
          this.state.toggleEdit &&
          <TaskInfoModal
            isOpen={this.state.toggleEdit}
            onClose={() => this.setState({ toggleEdit: false })}
            taskInfo={this.state.taskInfo}
          />
        }
        {
          this.state.toggleDelete &&
          <Confirmation
            isOpen={true}
            onClose={() => this.setState({ toggleDelete: false })}
            onConfirm={this.handleDeleteTask}
            content={{
              mainText: messages.deleteText.replace('{item}', 'this task'),
              description: messages.messageConfirm.replace('{name}',
                `${currentUser.first_name}`)
            }}
            title={messages.deleteTask}
            cancelText={messages.keepThisTask}
            confirmText={messages.delete}
          />
        }
        <div className='contact-task-header'>
          <Input
            icon='search'
            iconPosition='left'
            placeholder='Search task...'
            onChange={this.onSearchFilterChange}
          />
          <Button
            icon
            labelPosition='right'
            color='green'
            className='add-task'
            onClick={() => this.setState({
              toggleEdit: true,
              taskInfo: {
                contact: selectedContact._id,
                author: currentUser._id,
              }
            })}
          >
            {messages.addTask}
            <Icon name='right add' circular inverted />
          </Button>
        </div>
        <List selection celled>
          {
            data.map((task) => (
              <List.Item
                key={task._id}
              >
                <List.Content>
                  <Checkbox
                    className='task-status'
                    onChange={() => this.handleChangeTaskStatus(task)}
                    checked={task.is_completed}
                  />
                  <div className='date-add'>
                    <div className='due_date'>
                      {this.renderDueDate(task)}
                    </div>
                    <div>
                      {moment(new Date(task.due_date)).format('DD MMM YYYY')}
                    </div>
                  </div>
                  <div className='task-content'>
                    <div className='task'>{task.task}</div>
                    <div className='author'>Added by {Utils.combineName(task.author)}</div>
                    <div className='category'>{messages[task.category]}</div>
                    <div className='item-action'>
                      <div
                        onClick={(e) => {
                          e.stopPropagation();
                          this.setState({
                            selectedDeleteTaskId: task._id,
                            toggleDelete: true,
                          })
                        }}
                      >
                        {messages.delete}
                      </div>
                      <div
                        onClick={(e) => {
                          e.stopPropagation();

                          this.setState({
                            toggleEdit: true,
                            taskInfo: {
                              ...task,
                              contact: selectedContact._id,
                              author: currentUser._id,
                            }
                          })
                        }}
                      >
                        {messages.edit}
                      </div>
                    </div>
                  </div>
                </List.Content>
              </List.Item>
            ))
          }
        </List>
        {
          data.length > 0 && totalPage > 1 &&
          <Pagination
            activePage={activePage}
            totalPages={totalPage}
            onPageChange={this.handlePaginationChange}
          />
        }
        {
          status === ReduxStatus.loading &&
          <Loader active />
        }
      </div>
    )
  }
}

Tasks.contextType = AppContext;

export default injectSheet(stylesheet)(Tasks);