import { takeEvery, put, call, select, takeLatest, delay } from "redux-saga/effects";
import { toast } from 'react-toastify';
import isempty from 'lodash.isempty';
import { deleteClient, get, patch, post } from '../../Common/Utils/AxiosClient';
import Enum from '../../Common/Constants/Enum';
import {
  GET_CONTACTS,
  GET_CONTACTS_SUCCESS,
  GET_CONTACTS_FAILED,
  CREATE_CONTACT,
  SAVE_CONTACT,
  DELETE_CONTACT,
  GET_CONTACT_BY_ID,
  GET_CONTACT_BY_ID_SUCCESS,
  GET_CONTACT_BY_ID_FAILED,
  GET_PAYMENT_BY_CONTACT,
  GET_PAYMENT_BY_CONTACT_SUCCESS,
  GET_PAYMENT_BY_CONTACT_FAILED,
  GET_PAYMENT_DONATION_BY_CONTACT_SUCCESS,
  GET_PAYMENT_MEMBERSHIPS_BY_CONTACT_SUCCESS,
  GET_PAYMENT_MEMBERSHIPS_BY_CONTACT_FAILED,
  GET_PAYMENT_DONATION_BY_CONTACT_FAILED,
  GET_PAYMENT_MEMBERSHIPS_BY_CONTACT,
  GET_PAYMENT_DONATION_BY_CONTACT,
  GET_VOLUNTEER_CONTACTS,
  SAVE_CONTACT_FAILED,
  SEARCH_VOLUNTEER,
  DELETE_MULTIPLE_CONTACTS, SEARCH_VOLUNTEER_ON_MAP, SEARCH_VOLUNTEER_ON_MAP_SUCCESS,
  EXPORT_CONTACTS, GET_CONTACT_FILTER_SEGMENT, GET_CONTACT_FILTER_SEGMENT_SUCCESS, GET_CONTACT_FILTER_SEGMENT_FAILED, 
  ADD_PAYMENT_SOURCE, REMOVE_PAYMENT_SOURCE, SET_DEFAULT_PAYMENT_SOURCE,
  UPLOAD_CONTACT_LIST, UPLOAD_CONTACT_LIST_SUCCESS, UPLOAD_CONTACT_LIST_FAILED, GET_CONTACT_TEMPLATE
} from "../Actions/ContactActionCreator";
import omit from "lodash.omit";
import Utils from "../../Common/Utils/Utils";
import ContactActionCreator from "../Actions/ContactActionCreator";
import ContactUtils from "../ContactUtils";

function* watchGetContacts({ payload: { activePage, filter, type, opts } }) {
  try {
    let newFilter = ContactUtils.buildSearchContactQuery(filter);

    // Using at filter by tabs
    if(type === 'donor' && !newFilter['payments.payment_type']){
      newFilter = {...newFilter, ...{'payments.payment_type': {$in: ['once-off-donation', 'recurring-donation']}}}
    }
    if(type === 'membership'){
      newFilter = {...newFilter, ...{'payments.payment_type': 'membership', 'membership_status': 'active'}}
    }
    
    const skip = (activePage - 1) * Enum.ContactPerPage;
    const params = {
      filter: newFilter,
      limit: Enum.ContactPerPage,
      skip,
    };

    const url = '/contacts';
    const res = yield call(get, url, params);
    const totalPage = Utils.getTotalPage(res.data.metadata);
    yield put({
      type: opts.successAction || GET_CONTACTS_SUCCESS,
      payload: {
        activePage,
        filter: filter,
        totalPage,
        totalRecord: res.data.metadata.total,
        data: res.data.data,
      }
    });
  } catch (e) {
    yield put({
      type: opts.failureAction || GET_CONTACTS_FAILED,
      payload: {
        ...e
      },
    });
    const { messages } = yield select(state => state.intl);
    toast.error(messages['contact.list.get.error']);
  }
}

function* watchGetContactsFilterSegment({ payload: { activePage, filter } }) {
  try {
    const contactFilter = Utils.buildAdvancedQueryString(filter);
    const skip = (activePage - 1) * Enum.ContactPerPage;
    const url = '/contacts';
    const params = {
      filter: contactFilter,
      limit: Enum.ContactPerPage,
      skip
    };
    const res = yield call(get, url, params);
    const totalPage = Utils.getTotalPage(res.data.metadata);
    yield put({
      type: GET_CONTACT_FILTER_SEGMENT_SUCCESS,
      payload: {
        activePage,
        filter,
        totalPage,
        totalRecord: res.data.metadata.total,
        data: res.data.data,
      }
    });
  } catch (e) {
    yield put({
      type: GET_CONTACT_FILTER_SEGMENT_FAILED,
      payload: {
        ...e
      },
    });
    const { messages } = yield select(state => state.intl);
    toast.error(messages['contact.list.get.error']);
  }
}

function* watchGetContact({ payload: { id } }) {
  try {
    const url = `/contacts/${id}`;
    const res = yield call(get, url);
    yield put({ type: GET_CONTACT_BY_ID_SUCCESS, payload: res.data.data });
  } catch (e) {
    yield put({
      type: GET_CONTACT_BY_ID_FAILED,
      payload: {
        ...e
      },
    });
    const { messages } = yield select(state => state.intl);
    toast.error(messages['contact.list.get.error']);
  }
}

function* watchCreateContact({ payload: { contact, history } }) {
  const { messages } = yield select(state => state.intl);
  try {
    const url = '/contacts';
    const res = yield call(post, url, contact);
    toast.success(messages['contact.list.created.success']);
    history.push(`/contact-detail/${res.data.data._id}/detail`);
  } catch (e) {
    toast.error(messages['contact.list.created.failure']);
  }
}

function* watchSaveContact({ payload: { contact, reloadList } }) {
  const { messages } = yield select(state => state.intl);
  try {
    const url = `/contacts/${contact._id}`;
    yield call(patch, url, omit(contact, ['profile_picture']));
    toast.success(messages['contact.details.updated.success']);
    if (reloadList) {
      const contactList = yield select(state => state.contacts.contactList);
      yield put({
        type: GET_CONTACTS,
        payload: {
          activePage: contactList.activePage,
          filter: contactList.filter,
          opts: {}
        }
      });
    } else {
      yield put({
        type: GET_CONTACT_BY_ID,
        payload: {
          id: contact._id
        }
      })
    }
  } catch (e) {
    yield put({
      type: SAVE_CONTACT_FAILED,
      payload: {
        ...e
      },
    });
    if(e?.response?.data?.error?.message.includes('nationbuilder_id_1_deletedAt_1 dup key')) {
      toast.error(messages['contact.details.updated.duplicate_nationbuilder_id']);
    } else {
      toast.error(messages['contact.details.updated.failure']);
    }
  }
}

function* watchDeleteContact({ payload: { id, opts } }) {
  const { messages } = yield select(state => state.intl);
  try {
    const url = `/contacts/delete/${id}`;
    yield call(patch, url, {reason: opts.reason});
    toast.success(messages['contact.list.deleted.success']);
    let contactList = yield select(state => state.contacts.contactList);
    if (opts.type === 'volunteer') {
      contactList = yield select(state => state.contacts.contactVolunteerList);
      yield put(ContactActionCreator.getVolunteerContacts(contactList.activePage, contactList.filter));
    } else {
      yield put(ContactActionCreator.getContacts(contactList.activePage, contactList.filter));
    }
  } catch (e) {
    toast.error(messages['contact.list.deleted.failure']);
  }
}

function* watchDeleteMultipleContact({ payload: { ids, opts } }) {
  const { messages } = yield select(state => state.intl);
  try {
    const url = `/contacts-delete-multi?contact_ids=${ids.join(',')}`;
    yield call(patch, url, {reason: opts.reason});
    toast.success(messages['contact.list.deleted.success']);
    let contactList = yield select(state => state.contacts.contactList);
    if (opts.type === 'volunteer') {
      contactList = yield select(state => state.contacts.contactVolunteerList);
      yield put(ContactActionCreator.getVolunteerContacts(contactList.activePage, contactList.filter));
    } else {
      yield put(ContactActionCreator.getContacts(contactList.activePage, contactList.filter));
    }
  } catch (e) {
    toast.error(messages['contact.list.deleted.failure']);
  }
}

function* watchGetPaymentByContact({ payload: { id, activePage } }) {
  try {
    const skip = (activePage - 1) * Enum.PaymentPerPage;
    const url = `/contacts/${id}/payments?limit=${Enum.PaymentPerPage}&skip=${skip}`;
    const res = yield call(get, url);
    const totalPage = Utils.getTotalPage(res.data.metadata)
    yield put({
      type: GET_PAYMENT_BY_CONTACT_SUCCESS,
      payload: {
        activePage,
        totalPage,
        data: res.data.data,
      }
    });
  } catch (e) {
    yield put({
      type: GET_PAYMENT_BY_CONTACT_FAILED,
      payload: {
        ...e
      },
    });
  }
}

function* watchGetPaymentDonationByContact({ payload: { id, activePage } }) {
  try {
    const filter = encodeURIComponent(JSON.stringify({ type: Enum.PaymentEnum.donationPayment }));
    const skip = (activePage - 1) * Enum.PaymentPerPage;
    const url = `/contacts/${id}/payments?filter=${filter}&limit=${Enum.PaymentPerPage}&skip=${skip}`;
    const res = yield call(get, url);
    const totalPage = Utils.getTotalPage(res.data.metadata)
    yield put({
      type: GET_PAYMENT_DONATION_BY_CONTACT_SUCCESS,
      payload: {
        activePage,
        totalPage,
        data: res.data.data,
      }
    });
  } catch (e) {
    yield put({
      type: GET_PAYMENT_DONATION_BY_CONTACT_FAILED,
      payload: {
        ...e
      },
    });
  }
}

function* watchGetPaymentMembershipsByContact({ payload: { id, activePage } }) {
  try {
    const filter = encodeURIComponent(JSON.stringify({ type: Enum.PaymentEnum.membershipPayment }));
    const skip = (activePage - 1) * Enum.PaymentPerPage;
    const url = `/contacts/${id}/payments?filter=${filter}&limit=${Enum.PaymentPerPage}&skip=${skip}`;
    const res = yield call(get, url);
    const totalPage = Utils.getTotalPage(res.data.metadata)
    yield put({
      type: GET_PAYMENT_MEMBERSHIPS_BY_CONTACT_SUCCESS,
      payload: {
        activePage,
        totalPage,
        data: res.data.data,
      }
    });
  } catch (e) {
    yield put({
      type: GET_PAYMENT_MEMBERSHIPS_BY_CONTACT_FAILED,
      payload: {
        ...e
      },
    });
  }
}

function* watchSearchVolunteer({ payload: { searchText, status, handleSearchResult } }) {
  try {
    yield delay(300);
    const filter = {
      "$and": [
        {
          "volunteer": { "$exists": true }
        },
        { is_deceased: { $ne: true } },
        { do_not_contact: { $ne: true } },
      ],
    };
    if (searchText) {
      filter["$and"].push({
        "$text": { "$search": searchText }
      });
    }
    filter["$and"].push({
      "volunteer.status": { "$in": status }
    })
    const url = '/contacts';
    const params = {
      filter,
      limit: 10,
      skip: 0,
    };
    const res = yield call(get, url, params);
    handleSearchResult(res.data.data);
  } catch (e) {
  }
}

function* watchSearchVolunteerOnMap({ payload: { status, geoInfo } }) {
  try {
    const filter = {
      "$and": [
        {
          "volunteer": { "$exists": true }
        },
        { is_deceased: { $ne: true } },
        { do_not_contact: { $ne: true } },
      ],
    };
    filter["$and"].push({
      "volunteer.status": { "$in": status }
    })
    if (geoInfo) {
      filter["$and"].push({
        "home_address.location": { "$geoWithin": { "$center": geoInfo } },
      })
    }
    const url = '/contacts';
    const params = {
      filter,
      limit: 50,
      skip: 0,
    };
    const res = yield call(get, url, params);
    yield put({
      type: SEARCH_VOLUNTEER_ON_MAP_SUCCESS,
      payload: {
        data: res.data.data,
        filter: { status, geoInfo }
      }
    });

  } catch (e) {
  }
}

function* watchExportContactsToCSV({ payload: { type, filter, opts } }) {
  const { messages } = yield select(state => state.intl);
  let newFilter = ContactUtils.buildSearchContactQuery(filter);
  // Using at filter by tabs
  if(type === 'donor' && !newFilter['payments.payment_type']){
    newFilter = {...newFilter, ...{'payments.payment_type': {$in: ['once-off-donation', 'recurring-donation']}}}
  }
  if(type === 'membership'){
    newFilter = {...newFilter, ...{'payments.payment_type': 'membership', 'membership_status': 'active'}}
  }
  if(type === 'volunteer'){
    newFilter = {...newFilter, ...{'volunteer':{'$exists':true},'volunteer.status':{'$ne':'inactive'}}}
  }
  if(type === 'advanced-filter') newFilter = filter
  if(isempty(newFilter)) {
    toast.error(messages['contact.list.generate_csv.unallowed']);
    return
  }
  
  try {
    toast.info(messages['contact.list.generate_csv.processing'], { toastId: 'generateCsvNotification', autoClose: false });
    const contactFilter = Utils.cleanObject(newFilter);
    const url = `/export-contacts-to-csv`;
    const params = {
      filter: contactFilter,
      ...opts,
    };
    const config = {
      responseType: 'blob',
    };
    const res = yield call(get, url, params, config);
    
    Utils.downloadFile(res.data, `contacts_${(new Date()).toLocaleDateString("en-AU")}.csv`);
    toast.dismiss('generateCsvNotification');
  } catch (e) {
    toast.dismiss('generateCsvNotification');
    toast.error(messages['contact.list.generate_csv.failure']);
  }

}

function* watchAddPaymentSource({ payload: { contact_id, source_id } }) {
  const { messages } = yield select(state => state.intl);
  try {
    const url = `/contacts/${contact_id}/stripe-sources`
    yield call(post, url, { source_id });
    yield put({
      type: GET_CONTACT_BY_ID,
      payload: {
        id: contact_id
      }
    })
    toast.success(messages['contact.details.updated.success']);
  } catch (e) {
    toast.error(messages['contact.details.updated.failure']);
  }
}

function* watchRemovePaymentSource({ payload: { contact_id, source_id } }) {
  const { messages } = yield select(state => state.intl);
  try {
    const url = `/contacts/${contact_id}/stripe-sources/${source_id}`
    yield call(deleteClient, url);
    yield put({
      type: GET_CONTACT_BY_ID,
      payload: {
        id: contact_id
      }
    })
    toast.success(messages['contact.details.updated.success']);
  } catch (e) {
    toast.error(messages['contact.details.updated.failure']);
  }
}

function* watchSetDefaultPaymentSource({ payload: { contact_id, source_id } }) {
  const { messages } = yield select(state => state.intl);
  try {
    const url = `/contacts/${contact_id}/set-stripe-default-source`
    yield call(patch, url, { source_id });
    yield put({
      type: GET_CONTACT_BY_ID,
      payload: {
        id: contact_id
      }
    })
    toast.success(messages['contact.details.updated.success']);
  } catch (e) {
    toast.error(messages['contact.details.updated.failure']);
  }
}

function* watchUploadContactList({payload: {file}}) {
    try {
      const url = `/contacts/import-csv`
      const formData = new FormData()
      formData.append('type','contact')
      formData.append('file',file)
      const config = {
        headers: {
          'content-type': 'multipart/form-data',
        }
      }
      const resp = yield call(post, url, formData, config);
      yield put({type: UPLOAD_CONTACT_LIST_SUCCESS, payload: {filename: resp.data.data}})
    } catch (e) {
      yield put({type: UPLOAD_CONTACT_LIST_FAILED})
    }
}

function* watchGetContactTemplate() {
  try {
    const url = `/contacts/export-template`
    const result = yield call(get, url);
    Utils.downloadFile(result.data, 'contact-template.csv')
  } catch (e) {
    console.log(e)
  }
}

function* ContactSaga() {
  yield takeEvery(GET_CONTACTS, watchGetContacts);
  yield takeEvery(GET_VOLUNTEER_CONTACTS, watchGetContacts);
  yield takeEvery(GET_CONTACT_FILTER_SEGMENT, watchGetContactsFilterSegment);
  yield takeEvery(GET_CONTACT_BY_ID, watchGetContact)
  yield takeEvery(CREATE_CONTACT, watchCreateContact);
  yield takeEvery(SAVE_CONTACT, watchSaveContact);
  yield takeEvery(DELETE_CONTACT, watchDeleteContact);
  yield takeEvery(DELETE_MULTIPLE_CONTACTS, watchDeleteMultipleContact);
  yield takeEvery(GET_PAYMENT_BY_CONTACT, watchGetPaymentByContact);
  yield takeEvery(GET_PAYMENT_DONATION_BY_CONTACT, watchGetPaymentDonationByContact);
  yield takeEvery(GET_PAYMENT_MEMBERSHIPS_BY_CONTACT, watchGetPaymentMembershipsByContact);
  yield takeLatest(SEARCH_VOLUNTEER, watchSearchVolunteer);
  yield takeEvery(SEARCH_VOLUNTEER_ON_MAP, watchSearchVolunteerOnMap);
  yield takeEvery(EXPORT_CONTACTS, watchExportContactsToCSV);
  yield takeEvery(ADD_PAYMENT_SOURCE, watchAddPaymentSource)
  yield takeEvery(SET_DEFAULT_PAYMENT_SOURCE, watchSetDefaultPaymentSource)
  yield takeEvery(REMOVE_PAYMENT_SOURCE, watchRemovePaymentSource)
  yield takeEvery(UPLOAD_CONTACT_LIST, watchUploadContactList)
  yield takeLatest(GET_CONTACT_TEMPLATE, watchGetContactTemplate)
}

export default ContactSaga;
