import { takeEvery, put, call, select } from "redux-saga/effects";
import { toast } from 'react-toastify';
import { deleteClient, get, patch, post } from '../../Common/Utils/AxiosClient';
import Enum from '../../Common/Constants/Enum';
import Utils from "../../Common/Utils/Utils";
import {
  CREATE_CRITERIA,
  GET_CRITERIA_LIST_SUCCESS,
  GET_CRITERIA_LIST_FAILED,
  DELETE_CRITERIA, GET_CRITERIA_LIST,
  SAVE_CRITERIA
} from "../Actions/CriteriaActionCreator";

function* watchGetCriteriaList({ payload: { activePage, accountId, filter } }) {
  try {
    const newFilter = Utils.cleanObject(filter);
    const skip = (activePage - 1) * Enum.SegmentFilterPerPage;
    const url = `/accounts/${accountId}/search-criteria`;
    const params = {
      filter: newFilter,
      limit: Enum.SegmentFilterPerPage,
      skip,
      populate: { "path": "author" },
      sort: { "name": 1 },
    };
    const res = yield call(get, url, params);
    const totalPage = Utils.getTotalPage(res.data.metadata);
    const data = res.data.data.map(item => ({
      ...item,
      criteria: JSON.parse(item.criteria),
    }));
    yield put({
      type: GET_CRITERIA_LIST_SUCCESS,
      payload: {
        activePage,
        filter: filter,
        totalPage,
        data,
      }
    });
  } catch (e) {
    yield put({
      type: GET_CRITERIA_LIST_FAILED,
      payload: {
        ...e
      },
    });
  }
}

function* watchCreateCriteria({ payload: { accountId, criteria, opts } }) {
  try {
    const url = `/accounts/${accountId}/search-criteria`;
    const res = yield call(post, url, criteria);
    const { callback } = opts;
    if (callback) {
      callback(res.data.data);
    }
    const criteriaList = yield select(state => state.criteria.criteriaList);
    yield put({
      type: GET_CRITERIA_LIST,
      payload: {
        accountId,
        activePage: criteriaList.activePage,
        filter: criteriaList.filter,
      }
    });
    toast.success(`Create criteria successful`);
  } catch (e) {
    const { response: { data: { error } } } = e;
    toast.error(error.message);
  }
}

function* watchSaveCriteria({ payload: { accountId, criteria, opts } }) {
  try {
    const url = `/accounts/${accountId}/search-criteria/${criteria._id}`;
    yield call(patch, url, criteria);
    const { callback } = opts;
    if (callback) {
      callback();
    }
    const criteriaList = yield select(state => state.criteria.criteriaList);
    yield put({
      type: GET_CRITERIA_LIST,
      payload: {
        accountId,
        activePage: criteriaList.activePage,
        filter: criteriaList.filter,
      }
    });
    toast.success(`Save criteria successful`);
  } catch (e) {
    const { response: { data: { error } } } = e;
    toast.error(error.message);
  }
}

function* watchDeleteCriteria({ payload: { accountId, id } }) {
  try {
    const url = `/accounts/${accountId}/search-criteria/${id}`;
    yield call(deleteClient, url);
    const criteriaList = yield select(state => state.criteria.criteriaList);
    yield put({
      type: GET_CRITERIA_LIST,
      payload: {
        accountId,
        activePage: criteriaList.activePage,
        filter: criteriaList.filter,
      }
    });
    toast.success(`Delete criteria successful`);
  } catch (e) {
    const { response: { data: { error } } } = e;
    toast.error(error.message);
  }
}

function* CriteriaSaga() {
  yield takeEvery(GET_CRITERIA_LIST, watchGetCriteriaList);
  yield takeEvery(CREATE_CRITERIA, watchCreateCriteria);
  yield takeEvery(SAVE_CRITERIA, watchSaveCriteria);
  yield takeEvery(DELETE_CRITERIA, watchDeleteCriteria);
}

export default CriteriaSaga;
