import React from "react"
import { Accordion, List, ListContent, Loader, Pagination, Select, Input, Icon } from 'semantic-ui-react'
import injectsheet from 'react-jss'
import stylesheet from './stylesheet'
import moment from 'moment'
import ReduxStatus from "../../../Common/Constants/ReduxStatus"
import DashboardActionCreator from "../../DashboardActionCreator"
import AccountActionCreator from "../../../Account/AccountActionCreator"
import AppContext from "../../../Common/Constants/AppContext"
import Confirmation from '../../../Common/Components/Modal/ConfirmModal'
import Utils from "../../../Common/Utils/Utils"
import Checkbox from "../../../Common/Components/Base/Checkbox"
import { Link } from 'react-router-dom'

const inCompleteFilter = { is_completed: { $ne: true } }
class TaskList extends React.Component {
  state = {
    isShowIncompleteTasks: true,
    addedBy: '',
    selectedTaskIds: []
  }

  componentDidMount() {
    const { dispatch } = this.context
    dispatch(DashboardActionCreator.getTasks(1, inCompleteFilter))
    dispatch(AccountActionCreator.getAccounts(1))
  }

  handlePaginationChange = (e, { activePage }) => {
    const { dispatch } = this.context
    const { taskList: { filter } } = this.props;
    dispatch(DashboardActionCreator.getTasks(activePage, { ...filter, ...inCompleteFilter }))
  }

  onSearchFilterChange(e, { value }) {
    const { dispatch } = this.context
    const { taskList: { filter } } = this.props
    dispatch(DashboardActionCreator.getTasks(1, { ...filter, ...inCompleteFilter }))
  }

  handleChangeTaskStatus = (task) => {
    const { dispatch } = this.context
    const savedTask = {
      _id: task._id,
      is_completed: !this.state.selectedTaskIds.includes(task._id),
    }
    dispatch(DashboardActionCreator.saveTask(savedTask))

    if (!this.state.selectedTaskIds.includes(task._id)) {
      this.setState({ selectedTaskIds: [...this.state.selectedTaskIds, task._id] })
    } else {
      this.setState({ selectedTaskIds: this.state.selectedTaskIds.filter(t => t !== task._id) })
    }
  }

  handleChangeAddedBy = (e, { name, value }) => {
    if (value !== this.state.addedBy) {
      const { dispatch } = this.context
      const newFilter = { [name]: value }
      this.setState({ addedBy: value })
      dispatch(DashboardActionCreator.getTasks(1, { ...newFilter, ...inCompleteFilter }))
    }
  }

  handleDeleteTask = (id) => {
    const { dispatch } = this.context
    dispatch(DashboardActionCreator.deleteTask(id))
  }

  renderDateStatus = (task) => {
    const m = moment(task.due_date);
    const today = moment().startOf('day');
    const days = Math.round((m - today) / 86400000);
    if (days < 0) {
      return (
        <div className='status overdue'>
          Overdue
        </div>
      )
    } else if (days === 0) {
      return (
        <div className='status'>
          Today
        </div>
      )
    } else if (days === 1) {
      return (
        <div className='status'>
          Due in a day
        </div>
      )
    } else {
      return (
        <div className='status'>
          Due in {days} days
        </div>
      )
    }
  }

  render() {
    const { classes, taskList: { data, status, activePage, totalPage } } = this.props
    const { accountList } = this.props
    const { intl: { messages }, currentUser } = this.context
    const dataIncludedStatus = data.map(task => {
      if (this.state.selectedTaskIds.includes(task._id)) {
        task['task_selected'] = true
      } else {
        task['task_selected'] = false
      }
      return task
    })
    const accountOptions = accountList.data.map(a => {
      return {
        key: a._id,
        value: a._id,
        text: a.first_name + ' ' + a.last_name
      }
    })
    const emptyFilter = {
      key: 'select',
      text: 'Added By',
      value: '',
    };
    accountOptions.unshift(emptyFilter)
    return (
      <Accordion fluid className={classes.taskList}>
        <Accordion.Title onClick={() => this.setState({ isShowIncompleteTasks: !this.state.isShowIncompleteTasks })}>INCOMPLETE TASKS</Accordion.Title>
        <Accordion.Content active={this.state.isShowIncompleteTasks}>
          {
            status === ReduxStatus.loading &&
            <Loader active />
          }
          {/* {
            this.state.toggleEdit &&
            <TaskInfoModal
              isOpen={this.state.toggleEdit}
              onClose={() => this.setState({toggleEdit: false})}
              taskInfo={this.state.taskInfo}
            />
          } */}
          {
            this.state.toggleDelete &&
            <Confirmation
              isOpen={true}
              onClose={() => this.setState({ toggleDelete: false })}
              onConfirm={this.handleDeleteTask}
              content={{
                mainText: messages.deleteText.replace('{item}', 'this task'),
                description: messages.messageConfirm.replace('{name}',
                  `${currentUser.first_name}`)
              }}
              title={messages.deleteTask}
              cancelText={messages.keepThisTask}
              confirmText={messages.delete}
            />
          }
          <div className='task-filter'>
            {/* <Input
              icon='search'
              iconPosition='left'
              placeholder='Search task...'
              onChange={this.onSearchFilterChange}
            /> */}
            <Select
              className='author-filter'
              name='author'
              options={accountOptions}
              value={this.state.addedBy}
              onChange={this.handleChangeAddedBy}
              placeholder='Added By'
            />
          </div>
          <List selection celled>
            {
              dataIncludedStatus.map((task) => (
                <List.Item key={task._id} className={task.task_selected ? 'strike-through' : ''}>
                  <List.Content floated='left' verticalAlign='middle'>
                    <Checkbox
                      className='task-status'
                      onChange={() => this.handleChangeTaskStatus(task)}
                    />
                  </List.Content>
                  <List.Content verticalAlign='middle'>
                    <div className='task-content'>
                      {
                        task.contact &&
                        <Link className='contact' to={`/contact-detail/${task.contact._id}/overview`}>
                          {Utils.combineName(task.contact)}
                        </Link>
                      }
                      <div className='task'>{task.task}</div>
                      {task.author && <div className='author'>Added by <b>{task.author.first_name + ' ' + task.author.last_name}</b></div>}
                      <div className='category'>{messages[task.category]}</div>
                    </div>
                    <div className='date-add'>
                      {this.renderDateStatus(task)}
                      <div className='due_date'>
                        {moment(new Date(task.due_date)).format('DD MMM YYYY')}
                      </div>
                    </div>
                  </List.Content>
                  <div className='item-action'>
                    <div onClick={() => this.handleDeleteTask(task._id)}>
                      {messages.delete}
                    </div>
                  </div>
                </List.Item>
              ))
            }
          </List>
          {
            data.length > 0 && totalPage > 1 &&
            <Pagination
              activePage={activePage}
              totalPages={totalPage}
              onPageChange={this.handlePaginationChange}
            />
          }
        </Accordion.Content>
      </Accordion>
    );
  }
}

TaskList.contextType = AppContext;
export default injectsheet(stylesheet)(TaskList)

