import React from 'react';
import { Icon, Loader, Popup, Table, Pagination } from "semantic-ui-react";
import injectSheet from "react-jss";
import PropTypes from "prop-types";
import AppContext from "../../../../Common/Constants/AppContext";
import stylesheet from './stylesheet';
import ReduxStatus from "../../../../Common/Constants/ReduxStatus";
import InventoryActionCreator from "../../../Actions/InventoryActionCreator";
import InventoryInfoModal from './InventoryInfoModal';
import SetDeliveryModal from './SetDeliveryModal';
import LocationActionCreator from "../../../Actions/LocationActionCreator";
import ConfirmModal from "../../../../Common/Components/Modal/ConfirmModal";
import AddButton from '../../../../Common/Components/Base/AddButton';
import Utils from "../../../../Common/Utils/Utils";
import EventLocationFilter from "../EventLocations/EventLocationFilter/EventLocationFilter";

class EventInventory extends React.Component {
  constructor() {
    super();
    this.state = {
      toggleInventoryModal: false,
      inventoryInfo: {},

      toggleDeleteInventory: false,
      selectedDeleteInventory: null,

      toggleSetDelivery: false,
      selectedLocationDelivery: {},
    };
    this.handlePaginationChange = this.handlePaginationChange.bind(this);
    this.handleSaveInventory = this.handleSaveInventory.bind(this);
    this.handleDeleteInventory = this.handleDeleteInventory.bind(this);
    this.handleSetDelivery = this.handleSetDelivery.bind(this);
    this.handleAdjustQuantity = this.handleAdjustQuantity.bind(this);
    this.renderDeliveryStatus = this.renderDeliveryStatus.bind(this);
  }

  componentDidMount() {
    const { dispatch } = this.context;
    const { selectedEvent, locationList: { activePage, filter: {status, ...filter} = {}, sort } } = this.props;
    dispatch(InventoryActionCreator.getInventories(selectedEvent._id, 1));
    dispatch(LocationActionCreator.getLocations(selectedEvent._id, activePage, filter, sort));
  }

  handlePaginationChange(e, { activePage }) {
    const { dispatch } = this.context;
    const { locationList: { filter, sort }, selectedEvent } = this.props;
    dispatch(LocationActionCreator.getLocations(selectedEvent._id, activePage, filter, sort));
  }

  handleSaveInventory(inventoryInfo) {
    const { dispatch } = this.context;
    const { selectedEvent } = this.props;
    if (inventoryInfo._id) {
      dispatch(InventoryActionCreator.saveInventory(selectedEvent._id, inventoryInfo));
    } else {
      dispatch(InventoryActionCreator.createInventory(selectedEvent._id, inventoryInfo));
    }
    this.setState({
      toggleInventoryModal: false,
      inventoryInfo: {},
    });
  }

  handleSetDelivery(delivery) {
    const { dispatch } = this.context;
    const { selectedEvent } = this.props;
    const saveLocation = {
      _id: this.state.selectedLocationDelivery._id,
      name: this.state.selectedLocationDelivery.name,
      delivery,
    }
    dispatch(LocationActionCreator.saveLocation(selectedEvent._id, saveLocation));

    this.setState({
      toggleSetDelivery: false,
      selectedLocationDelivery: {},
    })
  }

  handleDeleteInventory() {
    const { dispatch } = this.context;
    const { selectedEvent } = this.props;
    dispatch(InventoryActionCreator.deleteInventory(selectedEvent._id, this.state.selectedDeleteInventory._id));
    this.setState({
      toggleDeleteInventory: false,
      selectedDeleteInventory: null,
    })
  }

  getLocationQuantity(inventory, locationId) {
    if (inventory.locations && inventory.locations.length > 0) {
      const location = inventory.locations.find(item => item.location === locationId);
      if (location) {
        return location.quantity;
      }
      return 0;
    }
    return 0;
  }

  handleAdjustQuantity(inventory, locationId, quantity, increase) {
    const { dispatch } = this.context;
    const { selectedEvent } = this.props;
    if (increase) {
      if (increase && inventory.quantity > inventory.usedQuantity) {
        dispatch(InventoryActionCreator.updateInventoryLocationQuantity(selectedEvent._id, inventory._id, locationId, quantity + 1));
      }
    } else {
      if (quantity > 0) {
        dispatch(InventoryActionCreator.updateInventoryLocationQuantity(selectedEvent._id, inventory._id, locationId, quantity - 1));
      }
    }
  }

  renderDeliveryStatus(location) {
    const { intl: { messages } } = this.context;
    let status = 'deliveryStatus';
    if (location.delivery && location.delivery.status) {
      status = location.delivery.status;
    }
    return (
      <span className={`status ${status}`}>{messages[status]}</span>
    )
  }

  render() {
    const {
      classes,
      selectedEvent,
      eventInventories: { data: inventoryData, status: inventoryStatus },
      locationList: { data: locationData, status, filter, sort, activePage, totalPage, totalRecord },
    } = this.props;
    const { intl: { messages }, currentUser } = this.context;
    return (
      <div className={classes.eventInventoryPage}>
        {
          (status === ReduxStatus.loading || inventoryStatus === ReduxStatus.loading) && <Loader active />
        }
        {
          this.state.toggleInventoryModal &&
          <InventoryInfoModal
            isOpen={this.state.toggleInventoryModal}
            onClose={() => this.setState({ toggleInventoryModal: false })}
            onOk={this.handleSaveInventory}
            inventoryInfo={this.state.inventoryInfo}
          />
        }
        {
          this.state.toggleSetDelivery &&
          <SetDeliveryModal
            isOpen={this.state.toggleSetDelivery}
            onClose={() => this.setState({ toggleSetDelivery: false })}
            onOk={this.handleSetDelivery}
            location={this.state.selectedLocationDelivery}
            eventId={selectedEvent._id}
          />
        }
        {
          this.state.toggleDeleteInventory &&
          <ConfirmModal
            isOpen={true}
            onClose={() => this.setState({ toggleDeleteInventory: false })}
            onConfirm={this.handleDeleteInventory}
            content={{
              mainText: messages.deleteText.replace('{item}', this.state.selectedDeleteInventory.name),
              description: messages.messageConfirm.replace('{name}',
                `${currentUser.first_name}`)
            }}
            title={messages.deleteItem}
            cancelText={messages.keepThisItem}
            confirmText={messages.delete}
          />
        }
        <AddButton
          onClick={() => this.setState({
            toggleInventoryModal: true,
            inventoryInfo: {}
          })}
          content={messages.addItem}
        />
        <EventLocationFilter
          selectedEvent={selectedEvent}
          initFilter={filter}
          sort={sort}
          showNumber={locationData.length}
          total={totalRecord}
        />
        <Table selectable basic>
          <Table.Header>
            <Table.Row verticalAlign='top'>
              <Table.HeaderCell>{messages.location}</Table.HeaderCell>
              <Table.HeaderCell>{messages.deliveryStatus}</Table.HeaderCell>
              <Table.HeaderCell textAlign='center'>{messages.volunteers}</Table.HeaderCell>
              {
                inventoryData.map(inventory => (
                  <Table.HeaderCell key={inventory._id}>
                    <Popup
                      trigger={(
                        <div className='header-action'>
                          <div>
                            {inventory.name}
                          </div>
                          <div><Icon name='ellipsis horizontal' /></div>
                        </div>
                      )}
                      content={(
                        <div className={classes.inventoryHeaderActions}>
                          <div
                            className='action'
                            onClick={(e) => {
                              e.stopPropagation();
                              this.setState({
                                toggleInventoryModal: true,
                                inventoryInfo: inventory
                              })
                              document.getElementsByTagName('body')[0].click();
                            }}
                          >
                            {messages.edit}
                          </div>
                          <div
                            className='action'
                            onClick={(e) => {
                              e.stopPropagation();
                              this.setState({
                                toggleDeleteInventory: true,
                                selectedDeleteInventory: inventory,
                              })
                              document.getElementsByTagName('body')[0].click();
                            }}
                          >
                            {messages.delete}
                          </div>
                        </div>
                      )}
                      on='click'
                      position='bottom center'
                    />
                  </Table.HeaderCell>
                ))
              }
              <Table.HeaderCell>
              </Table.HeaderCell>
            </Table.Row>
          </Table.Header>
          <Table.Body>
            <Table.Row className='inventory-info-row'>
              <Table.Cell>
              </Table.Cell>
              <Table.Cell>
              </Table.Cell>
              <Table.Cell textAlign='center'>
              </Table.Cell>
              {
                inventoryData.map(inventory => (
                  <Table.Cell textAlign='center' key={inventory._id}>
                    {inventory.usedQuantity}/{inventory.quantity}
                  </Table.Cell>
                ))
              }
              <Table.Cell>
              </Table.Cell>
            </Table.Row>
            {
              locationData.map(location => (
                <Table.Row key={location._id}>
                  <Table.Cell>
                    <div className='location-name'>
                      {location.name}
                    </div>
                    <div className='location-address'>
                      {Utils.combineAddress(location.address)}
                    </div>
                    <div className='recipient'>
                      {
                        location.delivery && location.delivery.recipient &&
                        <a target="_blank" rel="noopener noreferrer"
                           href={`/contact-detail/${location.delivery.recipient._id}/overview`}>
                          {Utils.combineName(location.delivery.recipient, messages)}
                        </a>
                      }
                    </div>
                  </Table.Cell>
                  <Table.Cell>
                    <div className='delivery-box' onClick={() => this.setState({
                      toggleSetDelivery: true,
                      selectedLocationDelivery: location
                    })}>
                      {this.renderDeliveryStatus(location)}
                      <Icon name='ellipsis horizontal' />
                    </div>
                  </Table.Cell>
                  <Table.Cell textAlign='center'>
                    {location.volunteers.length}
                  </Table.Cell>
                  {
                    inventoryData.map(inventory => {
                      const locationQuantity = this.getLocationQuantity(inventory, location._id);
                      return (
                        <Table.Cell textAlign='center' key={inventory._id}>
                          <div className='inventory-quantity'>
                            <Icon color="grey" name='minus'
                                  onClick={() => this.handleAdjustQuantity(inventory, location._id, locationQuantity, false)} />
                            <div className='quantity-value'>
                              {locationQuantity}
                            </div>
                            <Icon color="grey" name='plus'
                                  onClick={() => this.handleAdjustQuantity(inventory, location._id, locationQuantity, true)} />
                          </div>
                        </Table.Cell>
                      )
                    })
                  }
                  <Table.Cell>
                  </Table.Cell>
                </Table.Row>
              ))
            }
          </Table.Body>
        </Table>
        {
          locationData.length > 0 && totalPage > 1 &&
          <Pagination
            activePage={activePage}
            totalPages={totalPage}
            onPageChange={this.handlePaginationChange}
          />
        }
      </div>
    )
  }
}

EventInventory.propTypes = {
  selectedEvent: PropTypes.shape({}).isRequired,
  eventInventories: PropTypes.shape({}).isRequired,
  locationList: PropTypes.shape({}).isRequired,
};

EventInventory.contextType = AppContext;

export default injectSheet(stylesheet)(EventInventory);
