import React from 'react';
import PropTypes from 'prop-types';
import isequal from "lodash.isequal";
import { Button, Form, Modal, Icon } from 'semantic-ui-react';
import injectSheet from "react-jss";
import AppContext from "../../../../Common/Constants/AppContext";
import Enum from "../../../../Common/Constants/Enum";
import { ModalStyleSheet } from "../../../../Common/GlobalStyle";

const stylesheet = {
  helpText: {
    extend: ModalStyleSheet.helpText,
  },
  actionGroup: {
    extend: ModalStyleSheet.actionGroup,
  },
  header: {
    extend: ModalStyleSheet.header,
  },
  quantityContainer: {
    display: 'flex',
    alignItems: 'center',
    justifyContent: 'flex-end',
    '& .quantity-content': {
      marginLeft: 20,
      display: 'flex',
      alignItems: 'stretch',
      border: '1px solid rgba(34,36,38,.15)',
      '& .decrease-increase': {
        padding: [0, 10],
        display: 'flex',
        alignItems: 'center',
        cursor: 'pointer',
        '&.decrease': {
          borderRight: '1px solid rgba(34,36,38,.15)',
        },
        '&.increase': {
          borderLeft: '1px solid rgba(34,36,38,.15)',
        }
      },
      '& input': {
        border: 'none !important',
        width: '90px !important',
        textAlign: 'center',
      },
      '& input[type=number]::-webkit-inner-spin-button': {
        '-webkit-appearance': 'none',
        margin: 0,
      },
      '& input[type=number]::-webkit-outer-spin-button': {
        '-webkit-appearance': 'none',
        margin: 0,
      }
    }
  }
}

class InventoryInfoModal extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      inventoryInfo: {
        quantity: 0,
        ...props.inventoryInfo,
      },
    };
    this.isFieldError = this.isFieldError.bind(this);
    this.handleChange = this.handleChange.bind(this);
    this.handleSubmit = this.handleSubmit.bind(this);
    this.handleAdjustQuantity = this.handleAdjustQuantity.bind(this);
  }

  handleChange({ target: { name, value } }) {
    this.setState({
      inventoryInfo: {
        ...this.state.inventoryInfo,
        [name]: value,
      }
    })
  }

  isFieldError(name) {
    const { intl: { messages } } = this.context;
    if (!this.state.inventoryInfo[name]) {
      let fieldText = messages[name];
      if (name === 'name') {
        fieldText = messages.itemName;
      }
      return `${messages.pleaseEnter} ${fieldText}`;
    }
    return '';
  }

  handleSubmit() {
    if (!this.state.isHitSave) {
      this.setState({
        isHitSave: true,
      });
    }
    if (this.isFieldError('name') ||
      this.isFieldError('distribution_method')
    ) {
      return;
    }
    const { onOk } = this.props;
    onOk(this.state.inventoryInfo);
  }

  handleAdjustQuantity(increase) {
    if(increase) {
      this.setState({
        inventoryInfo: {
          ...this.state.inventoryInfo,
          quantity: this.state.inventoryInfo.quantity + 1
        }
      })
    } else {
      this.setState({
        inventoryInfo: {
          ...this.state.inventoryInfo,
          quantity: this.state.inventoryInfo.quantity - 1
        }
      })
    }
  }

  render() {
    const { classes, isOpen, onClose } = this.props;
    const { intl: { messages } } = this.context;
    const { inventoryInfo } = this.state;
    const distributionMethods = Object.values(Enum.DistributionMethods).map(item => ({
      key: item,
      value: item,
      text: messages[item],
    }));
    return (
      <Modal
        open={isOpen}
        onClose={onClose}
        size="mini"
      >
        <Modal.Header className={classes.header}>
          {
            inventoryInfo._id ? messages.updateItem : messages.addItem
          }
        </Modal.Header>
        <Modal.Content>
          <Form
            onSubmit={this.handleSubmit}
          >
            <Form.Field required error={this.state.isHitSave && this.isFieldError('name')}>
              <input
                placeholder={messages.itemName}
                name='name'
                value={inventoryInfo.name}
                onChange={this.handleChange}
              />
              {
                this.state.isHitSave && this.isFieldError('name') &&
                <div className={classes.helpText}>{this.isFieldError('name')}</div>
              }
            </Form.Field>
            <Form.Field className={classes.quantityContainer} required
                        error={this.state.isHitSave && this.isFieldError('quantity')}>
              <span>{messages.quantity}</span>
              <div className='quantity-content'>
                <div className='decrease-increase decrease' onClick={()=> this.handleAdjustQuantity(false)}>
                  <Icon color="grey" name='minus' />
                </div>
                <input
                  name='quantity'
                  type='number'
                  value={inventoryInfo.quantity}
                  onChange={this.handleChange}
                  min={0}
                />
                <div className='decrease-increase increase' onClick={()=> this.handleAdjustQuantity(true)}>
                  <Icon color="grey" name='plus' />
                </div>
              </div>
            </Form.Field>
            <Form.Field required error={this.state.isHitSave && this.isFieldError('distribution_method')}>
              <Form.Select
                options={distributionMethods}
                placeholder={messages.distribution_method}
                name='distribution_method'
                value={inventoryInfo.distribution_method}
                onChange={(e, target) => this.handleChange({ target: target })}
              />
              {
                this.state.isHitSave && this.isFieldError('distribution_method') &&
                <div className={classes.helpText}>{this.isFieldError('distribution_method')}</div>
              }
            </Form.Field>
            <Form.Field className={classes.actionGroup}>
              <Button onClick={onClose}>
                {messages.cancel}
              </Button>
              <Button
                type='submit'
                color='green'
                disabled={isequal(this.props.inventoryInfo, inventoryInfo) && inventoryInfo._id}
              >
                {
                  inventoryInfo._id ? messages.update : messages.create
                }
              </Button>
            </Form.Field>
          </Form>
        </Modal.Content>
      </Modal>
    )
  }
}

InventoryInfoModal.propTypes = {
  isOpen: PropTypes.bool.isRequired,
  onClose: PropTypes.func.isRequired,
  onOk: PropTypes.func.isRequired,
  inventoryInfo: PropTypes.shape({}).isRequired,
};

InventoryInfoModal.contextType = AppContext;

export default injectSheet(stylesheet)(InventoryInfoModal);