import React from 'react';
import PropTypes from 'prop-types';
import debounce from 'lodash.debounce';
import { Form, Input } from "semantic-ui-react";
import injectSheet from "react-jss";
import stylesheet from './stylesheet';
import LocationActionCreator from "../../../../Actions/LocationActionCreator";
import ElectorateComponent from "../../../../../Common/Components/ElectorateComponent/ElectorateComponent";
import AppContext from "../../../../../Common/Constants/AppContext";
import Enum from "../../../../../Common/Constants/Enum";
import Utils from "../../../../../Common/Utils/Utils";
import { eventLocationFilter } from "../../../../../Common/Constants/AppKey";

const defaultElectorate = { name: 'All' };

class EventLocationFilter extends React.Component {
  constructor(props) {
    super(props);
    const filter = props.initFilter;
    const { selectedEvent } = props;
    const currentSelectedElectorate = this.getInitElectorate(selectedEvent._id, filter.voting_district);
    this.state = {
      filter: {
        ...filter,
        status: null
      },
      searchText: this.getSearchTextFromFilter(filter),
      currentSelectedElectorate,
    };

    this.electorateFilterRef = React.createRef();

    this.onSearchFilterChange = debounce(this.onSearchFilterChange.bind(this), 300);
    this.handleSelectElectorate = this.handleSelectElectorate.bind(this);
    this.handleChange = this.handleChange.bind(this);
    this.handleResetFilter = this.handleResetFilter.bind(this);
    this.handleSearch = this.handleSearch.bind(this);
  }

  getInitElectorate(eventId, voting_district) {
    let currentSelectedElectorate = { ...defaultElectorate };
    const preSelectedData = Utils.getLocalStorage(eventLocationFilter);
    if (preSelectedData) {
      if (preSelectedData[eventId]) {
        currentSelectedElectorate = preSelectedData[eventId].currentSelectedElectorate;
        if (!voting_district) {
          currentSelectedElectorate = { ...defaultElectorate };
          Utils.setLocalStorage(eventLocationFilter, preSelectedData);
        }
      } else {
        preSelectedData[eventId] = {
          currentSelectedElectorate,
        }
        Utils.setLocalStorage(eventLocationFilter, preSelectedData);
      }
    } else {
      Utils.setLocalStorage(eventLocationFilter, {});
    }
    return currentSelectedElectorate;
  }

  getSearchTextFromFilter(filter) {
    let currentFilter = filter || this.state.filter;
    if (currentFilter["$or"] && currentFilter["$or"].length > 0) {
      const nameFilter = currentFilter["$or"].find(item => item["name"]);
      if (nameFilter && nameFilter.name) {
        return nameFilter.name["$regex"];
      }
    }
    return '';
  }

  handleSearch() {
    const { dispatch } = this.context;
    const { selectedEvent, sort } = this.props;
    dispatch(LocationActionCreator.getLocations(selectedEvent._id, 1, this.state.filter, sort));
  }

  onSearchFilterChange(e, { value }) {
    let search = null;
    if (value) {
      search = [
        {
          "name": {
            "$regex": value,
            "$options": "i",
          },
        },
        {
          "address.line1": {
            "$regex": value,
            "$options": "i"
          }
        },
        {
          "address.line2": {
            "$regex": value,
            "$options": "i"
          }
        },
        {
          "address.line3": {
            "$regex": value,
            "$options": "i"
          }
        },
        {
          "address.city": {
            "$regex": value,
            "$options": "i"
          }
        },

        {
          "address.state": {
            "$regex": value,
            "$options": "i"
          }
        },
        {
          "address.country": {
            "$regex": value,
            "$options": "i"
          }
        },
      ];
    }
    this.setState({
      filter: {
        ...this.state.filter,
        '$or': search,
      }
    }, () => {
      this.handleSearch();
    });
  }

  handleSelectElectorate(selectedElectorate) {
    const preSelectedData = Utils.getLocalStorage(eventLocationFilter);
    const { selectedEvent } = this.props;
    preSelectedData[selectedEvent._id] = {
      currentSelectedElectorate: selectedElectorate,
    };
    Utils.setLocalStorage(eventLocationFilter, preSelectedData);
    this.setState({
      currentSelectedElectorate: selectedElectorate,
      filter: {
        ...this.state.filter,
        voting_district: selectedElectorate._id,
      }
    }, () => {
      this.handleSearch();
    });
  }

  handleResetFilter() {
    this.electorateFilterRef.current.setState({
      currentSearchText: '',
      currentSelected: { ...defaultElectorate },
    });
    const preSelectedData = Utils.getLocalStorage(eventLocationFilter);
    const { selectedEvent } = this.props;
    preSelectedData[selectedEvent._id] = {
      currentSelectedElectorate: { ...defaultElectorate },
    };
    Utils.setLocalStorage(eventLocationFilter, preSelectedData);
    this.setState({
      filter: {
        '$or': null,
        status: '',
        voting_district: '',
      },
      searchText: '',
      currentSelectedElectorate: { ...defaultElectorate },
    }, () => {
      this.handleSearch();
    });
  }

  handleChange(e, { name, value }) {
    const statusFilter = window.location.href.includes('inventory') ? {"delivery.status": value} : {"status": value}
    if (value !== this.state.filter.status) {
      this.setState({
        filter: {
          ...this.state.filter,
          ...statusFilter
        },
      }, () => {
        this.handleSearch();
      });
    }
  }

  render() {
    const { classes, showNumber, total } = this.props;
    const { intl: { messages } } = this.context;
    const emptyFilter = {
      key: 'all',
      text: messages.all,
      value: '',
    };

    let dropdownValues = Enum.EventLocationStatus
    if(window.location.href.includes('inventory')) {
      dropdownValues = Enum.DeliveryStatus
    }
    const filterStatus = Object.values(dropdownValues).map(item => ({
      key: item,
      text: messages[item],
      value: item
    }));
    filterStatus.unshift(emptyFilter);

    const status = this.state.filter.status ? this.state.filter.state : this.state.filter['delivery.status']

    return (
      <div className={classes.searchFilter}>
        <div className='search-container'>
          <div className='search-box'>
            <Input
              icon='search'
              iconPosition='left'
              placeholder='Search...'
              value={this.state.searchText}
              onChange={(e, { value }) => {
                this.setState({
                  searchText: value,
                });
                this.onSearchFilterChange(e, { value });
              }}
            />
          </div>
          <div className='filter-box'>
            <div className='block-item'>
              <div className='text-title'>{messages.electorate}</div>
              <div className='control event-location-search-electorate'>
                <ElectorateComponent
                  onSelect={this.handleSelectElectorate}
                  forwardedRef={this.electorateFilterRef}
                  allowClear
                  allowAll
                  defaultSelected={this.state.currentSelectedElectorate}
                />
              </div>
            </div>
            <div className='block-item'>
              <div className='text-title'>{messages.status}</div>
              <div className='control'>
                <Form.Select
                  options={filterStatus}
                  placeholder={messages.all}
                  name='status'
                  value={status}
                  onChange={this.handleChange}
                />
              </div>
            </div>
          </div>
          <div className='reset-filter'>
            <span onClick={this.handleResetFilter}>Reset filters</span>
            <img src={require('../../../../../Assets/reset.png')} alt='' onClick={this.handleResetFilter} />
          </div>
        </div>
        <div className='display-contact-number'>
          Showing <b>{showNumber}</b> of <b>{total}</b> locations
        </div>
      </div>
    )
  }
}

EventLocationFilter.contextType = AppContext;

EventLocationFilter.propTypes = {
  selectedEvent: PropTypes.shape({}).isRequired,
  showNumber: PropTypes.number,
  total: PropTypes.number,
  initFilter: PropTypes.shape({}),
  sort: PropTypes.shape({}),
};

EventLocationFilter.defaultProps = {
  showNumber: 0,
  total: 0,
  initFilter: {
    '$or': null,
    status: '',
    voting_district: '',
  },
  sort: {},
}

export default injectSheet(stylesheet)(EventLocationFilter);
