import React from 'react';
import ReduxStatus from "../../../../Common/Constants/ReduxStatus";
import { Table, Loader, Pagination } from "semantic-ui-react";
import ConfirmModal from "../../../../Common/Components/Modal/ConfirmModal";
import AddButton from "../../../../Common/Components/Base/AddButton";
import Utils from "../../../../Common/Utils/Utils";
import LocationInfoModal from "../EventLocationMap/LocationList/LocationInfoModal";
import EventLocationFilter from "./EventLocationFilter/EventLocationFilter";

import stylesheet from './stylesheet';
import ImportModal from './ImportModal';
import injectSheet from "react-jss";
import AppContext from "../../../../Common/Constants/AppContext";
import LocationActionCreator from "../../../Actions/LocationActionCreator";
import ImportButton from '../../../../Common/Components/Base/ImportButton';


const SORT_DIRECTION = {
  asc: 'ascending',
  des: 'descending',
}

class EventLocations extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      toggleDeleteLocation: false,
      selectedDeleteLocation: null,

      toggleLocationModal: false,
      locationInfo: {},
    }
    this.handlePaginationChange = this.handlePaginationChange.bind(this);
    this.handleDeleteLocation = this.handleDeleteLocation.bind(this);
    this.changeSort = this.changeSort.bind(this);
    this.onImportSuccess = this.onImportSuccess.bind(this);
  }

  componentDidMount() {
    const { dispatch } = this.context;
    const { selectedEvent, locationList: { activePage, filter, sort } } = this.props;
    dispatch(LocationActionCreator.getLocations(selectedEvent._id, activePage, filter, sort));
  }

  onImportSuccess() {
    const {dispatch} = this.context;
    const {selectedEvent, locationList: {sort}} = this.props;
    dispatch(LocationActionCreator.getLocations(selectedEvent._id, 1, {}, sort));
  }

  handlePaginationChange(e, { activePage }) {
    const { dispatch } = this.context;
    const { selectedEvent, locationList: { filter, sort } } = this.props;
    dispatch(LocationActionCreator.getLocations(selectedEvent._id, activePage, filter, sort));
  }

  handleDeleteLocation() {
    const { dispatch } = this.context;
    const { selectedEvent } = this.props;
    dispatch(LocationActionCreator.deleteLocation(selectedEvent._id, this.state.selectedDeleteLocation._id));
    this.setState({
      toggleDeleteLocation: false,
      selectedDeleteLocation: null,
    })
  }

  changeSort(name) {
    const { dispatch } = this.context;
    let sortDirection = SORT_DIRECTION.asc;
    const { selectedEvent, locationList: { filter, sort } } = this.props;
    const currentSort = Utils.parseSortConfig(sort);
    if (currentSort.sortColumn === name) {
      sortDirection = currentSort.sortDirection === SORT_DIRECTION.asc ? SORT_DIRECTION.des : SORT_DIRECTION.asc;
    }
    const newSort = Utils.getSortConfig(name, sortDirection);
    dispatch(LocationActionCreator.getLocations(selectedEvent._id, 1, filter, newSort));
  }

  setImportModalRef = (ref) => {
    this.importModalRef = ref;
  };

  render() {
    const {
      classes,
      selectedEvent,
      locationList: { data, status, activePage, totalPage, totalRecord, filter, sort },
    } = this.props;
    const {intl: {messages}, currentUser} = this.context;

    const { sortColumn, sortDirection } = Utils.parseSortConfig(sort);
    ;
    return (
      <div className={classes.locationListPage}>
        {
          status === ReduxStatus.loading && <Loader active />
        }
        {
          this.state.toggleLocationModal &&
          <LocationInfoModal
            isOpen={this.state.toggleLocationModal}
            onClose={() => this.setState({ toggleLocationModal: false })}
            locationInfo={this.state.locationInfo}
            selectedEvent={selectedEvent}
          />
        }
        {
          this.state.toggleDeleteLocation &&
          <ConfirmModal
            isOpen={true}
            onClose={() => this.setState({ toggleDeleteLocation: false })}
            onConfirm={this.handleDeleteLocation}
            content={{
              mainText: messages.deleteText.replace('{item}', this.state.selectedDeleteLocation.name),
              description: messages.messageConfirm.replace('{name}',
                `${currentUser.first_name}`)
            }}
            title={messages.deleteLocation}
            cancelText={messages.keepThisLocation}
            confirmText={messages.delete}
          />
        }
        <ImportModal
          onImportSuccess={this.onImportSuccess}
          eventId={selectedEvent._id}
          ref={this.setImportModalRef}
        />
        <ImportButton
          onClick={() => this.importModalRef.openModal()}
          content="Import Location"
        />
        <AddButton
          onClick={() => this.setState({
            toggleLocationModal: true,
            locationInfo: {},
          })}
          content={messages.addLocation}
        />
        <EventLocationFilter
          selectedEvent={selectedEvent}
          showNumber={data.length}
          total={totalRecord}
          initFilter={filter}
          sort={sort}
        />
        <Table sortable selectable basic>
          <Table.Header>
            <Table.Row>
              <Table.HeaderCell
                sorted={sortColumn === 'name' ? sortDirection : null}
                onClick={() => this.changeSort('name')}
                width={3}>{messages.name}</Table.HeaderCell>
              <Table.HeaderCell width={3}>{messages.address}</Table.HeaderCell>
              <Table.HeaderCell>{messages.electorate}</Table.HeaderCell>
              <Table.HeaderCell
                sorted={sortColumn === 'expected_votes' ? sortDirection : null}
                onClick={() => this.changeSort('expected_votes')}
              >
                {messages.expected_votes}
              </Table.HeaderCell>
              <Table.HeaderCell>{messages.volunteer}</Table.HeaderCell>
              <Table.HeaderCell
                sorted={sortColumn === 'status' ? sortDirection : null}
                onClick={() => this.changeSort('status')}
              >
                {messages.status}
              </Table.HeaderCell>
              <Table.HeaderCell width={3}></Table.HeaderCell>
            </Table.Row>
          </Table.Header>
          <Table.Body>
            {data.map(location => (
              <Table.Row key={location._id} className="contact-row">
                <Table.Cell>
                  {location.name}
                </Table.Cell>
                <Table.Cell>
                  {Utils.combineAddress(location.address)}
                </Table.Cell>
                <Table.Cell>
                  {location.voting_district && location.voting_district.name ? location.voting_district.name : '-'}
                </Table.Cell>
                <Table.Cell>{location.expected_votes}</Table.Cell>
                <Table.Cell>
                  {location.volunteers ? location.volunteers.length : 0}/
                  {location.volunteers_required} {messages.volunteers}
                </Table.Cell>
                <Table.Cell>
                  <div className={`location-status ${location.status}`}>{messages[location.status]}</div>
                </Table.Cell>
                <Table.Cell width={3}>
                  <div className="user-action">
                    <div onClick={() => this.setState({
                      toggleDeleteLocation: true,
                      selectedDeleteLocation: location,
                    })}>
                      {messages.delete}
                    </div>
                    <div onClick={() => this.setState({
                      toggleLocationModal: true,
                      locationInfo: location,
                    })}>
                      {messages.edit}
                    </div>
                  </div>
                </Table.Cell>
              </Table.Row>
            ))}
          </Table.Body>
        </Table>
        {
          data.length > 0 && totalPage > 1 &&
          <Pagination
            activePage={activePage}
            totalPages={totalPage}
            onPageChange={this.handlePaginationChange}
          />
        }
      </div>
    )
  }
}

EventLocations.contextType = AppContext;

export default injectSheet(stylesheet)(EventLocations);
