import React from "react";
import PropTypes from "prop-types";
import {
  Button,
  Modal,
  List,
  Icon,
} from "semantic-ui-react";
import injectSheet from "react-jss";
import Dropzone from "react-dropzone";
import _ from "lodash";

import {ModalStyleSheet} from "../../../../../Common/GlobalStyle";
import colors from "../../../../../Common/Constants/color";
import AppContext from "../../../../../Common/Constants/AppContext"
import {get, post} from "../../../../../Common/Utils/AxiosClient";
import Utils from "../../../../../Common/Utils/Utils";

const style = {
  groupOfType: {
    justifyContent: "center",
  },
  toggleContactType: {
    display: "flex",
    alignItems: "center",
    "& .ui.toggle.checkbox .box:before, .ui.toggle.checkbox label:before": {
      backgroundColor: "#0d71bb!important",
    },
  },
  helpText: {
    extend: ModalStyleSheet.helpText,
  },
  actionGroup: {
    extend: ModalStyleSheet.actionGroup,
    justifyContent: "space-bettwen",
  },
  header: {
    extend: ModalStyleSheet.header,
  },
  warning: {
    color: colors.primaryRed1,
    marginLeft: 25,
    marginTop: 5,
  },
  dropzoneContainer: {
    "& .dropzone": {
      margin: "1em 0 1em 0",
      padding: "4em 0em",
      borderRadius: ".3em",
      backgroundColor: `${colors.primaryGray4} !important`,
      color: `${colors.primaryBlue} !important`,
      fontWeight: "bold",
      fontSize: "9pt",
      textAlign: "center",
      cursor: "pointer",
    },
  },
  downloadText: {
    color: colors.primaryBlue,
    textDecoration: "underline",
    fontSize: 13,
    cursor: "pointer",
    marginLeft: 25,
  },
  note: {
    marginLeft: 25,
    marginBottom: 20,
    marginTop: 10,
    "& .title": {
      display: "flex",
      justifyContent: "space-between",
      alignItems: "center",
      cursor: "pointer",
    },
  },
};

class ImportModal extends React.Component {
  state = {
    isVisible: false,
    isLoading: false,
    expand: false,
    files: [],
  };

  openModal = () => this.setState({isVisible: true});

  hideModal = () => this.setState({isVisible: false, files: []});

  onDrop = (files) => this.setState({files});

  onImportData = async () => {
    const {files} = this.state;
    const {eventId, onImportSuccess} = this.props;

    this.setState({isLoading: true});
    const formData = new FormData()
    formData.append('file', files[0])
    const config = {
      header: {
        'content-type': 'multipart/form-data',
      }
    }
    const results = await post(`events/${eventId}/locations/import-location`, formData, config)
    this.setState({isLoading: false})
    if (_.get(results, 'data.data.success')) {
      onImportSuccess()
    }
    this.hideModal();
  };

  downloadTemplate = async () => {
    const result = await get('get-import-template');
    Utils.downloadFile(result.data, 'location.csv')
  };

  render() {
    const {classes} = this.props;
    const {
      intl: {messages},
    } = this.context;

    const {isVisible, isLoading, files, expand} = this.state;

    return (
      <Modal open={isVisible} onClose={this.hideModal} size="tiny">
        <Modal.Header className={classes.header}>Import Locations</Modal.Header>
        <Modal.Content>
          <Dropzone accept=".csv" onDrop={this.onDrop}>
            {({getRootProps, getInputProps}) => (
              <section className={classes.dropzoneContainer}>
                <div {...getRootProps({className: "dropzone"})}>
                  <input {...getInputProps()} />
                  <p>Drag & drop some files here, or click to select files</p>
                </div>
              </section>
            )}
          </Dropzone>
          <List>
            {files.map((file) => (
              <List.Item key={file.name}>
                <List.Icon name="file alternate" />
                <List.Content>{file.name}</List.Content>
              </List.Item>
            ))}
          </List>
        </Modal.Content>
        <div className={classes.downloadText} onClick={this.downloadTemplate}>
          Download import template
        </div>
        <div className={classes.note}>
          <div
            className="title"
            onClick={() =>
              this.setState((prev) => ({expand: !prev.expand}))
            }
          >
            <span>Notes for the special fields</span>
            <Icon name={`chevron ${expand ? "down" : "up"}`} />
          </div>
          {expand ? (
            <List>
              {[
                {
                  field: "location_name",
                  description: "Free text [Required]",
                },
                {
                  field: "electorate_name",
                  description: "Free text [Required]",
                },
                {
                  field: "electorate_state",
                  description: "Possible values: ['NSW', 'VIC', 'WA', 'QLD', 'ACT', 'TAS', 'NT', 'SA'] [Required]",
                },
                {
                  field: "electorate_type",
                  description:
                    "Possible values: ['federal-upper', 'federal-lower', 'state-upper', 'state-lower'] [Required]"
                },
                {
                  field: "expected_votes",
                  description: "Number [Required]"
                },
                {
                  field: "volunteers_required",
                  description: "Number [Optional]"
                },
                {
                  field: "line1, line2, line3",
                  description: "One of those fields must have data [Required]",
                },
                {
                  field: "city",
                  description: "Free text [Required]",
                },
                {
                  field: "postal_code",
                  description: "Number [Required]"
                },
                {
                  field: "state",
                  description: "Possible values: ['NSW', 'VIC', 'WA', 'QLD', 'ACT', 'TAS', 'NT', 'SA'] [Required]"
                },
                {
                  field: "country",
                  description: "Should be 'Australia' [Required]"
                }
              ].map((item) => (
                <List.Item>
                  <List.Header>{item.field}</List.Header>
                  <List.Description>{item.description}</List.Description>
                </List.Item>
              ))}
            </List>
          ) : null}
        </div>
        <Modal.Actions>
          <Button onClick={this.hideModal}>{messages.close}</Button>
          <Button
            positive
            loading={isLoading}
            content="Import Location"
            disabled={files.length === 0}
            onClick={this.onImportData}
          />
        </Modal.Actions>
      </Modal>
    );
  }
}

ImportModal.propTypes = {};

ImportModal.contextType = AppContext;

export default injectSheet(style)(ImportModal);
