import React from 'react';
import injectSheet from "react-jss";
import debounce from 'lodash.debounce';
import { Input, Loader, Pagination } from 'semantic-ui-react';
import stylesheet from "../stylesheet";
import EventCard from "./EventCard";
import EventInfoModal from "./EventInfoModal";
import AppContext from "../../../Common/Constants/AppContext";
import EventActionCreator from "../../Actions/EventActionCreator";
import ConfirmModal from "../../../Common/Components/Modal/ConfirmModal";
import ReduxStatus from "../../../Common/Constants/ReduxStatus";
import AddButton from '../../../Common/Components/Base/AddButton';

class UpcomingEvents extends React.Component {
  constructor() {
    super();
    this.state = {
      toggleDeleteEvent: false,
      selectedDeleteEvent: null,
      toggleEventModal: false,
      eventInfo: {},
    };
    this.handlePaginationChange = this.handlePaginationChange.bind(this);
    this.onSearchFilterChange = debounce(this.onSearchFilterChange.bind(this), 300);
    this.handleSaveEvent = this.handleSaveEvent.bind(this);
    this.handleDeleteEvent = this.handleDeleteEvent.bind(this);
  }

  componentDidMount() {
    const { dispatch } = this.context;
    const filter = {
      end_time: {$gte : new Date().getTime()}
    };

    dispatch(EventActionCreator.getUpcomingEvents(1, filter));
  }

  handlePaginationChange(e, { activePage }) {
    const { dispatch } = this.context;
    const { events: { filter } } = this.props;
    dispatch(EventActionCreator.getUpcomingEvents(activePage, filter));
  }

  onSearchFilterChange(e, { value }) {
    const { dispatch } = this.context;
    const { events: { filter } } = this.props;

    const newFilter = {
      ...filter,
    };
    if (value) {
      newFilter.name = { "$regex": value, "$options": "i" };
    } else {
      newFilter.name = null;
    }
    dispatch(EventActionCreator.getUpcomingEvents(1, newFilter));
  }

  handleDeleteEvent() {
    const { dispatch } = this.context;
    dispatch(EventActionCreator.deleteEvent(this.state.selectedDeleteEvent._id));
    this.setState({
      toggleDeleteEvent: false,
      selectedDeleteEvent: null,
    })
  }

  handleSaveEvent(eventInfo) {
    const { dispatch } = this.context;
    if (eventInfo._id) {
      dispatch(EventActionCreator.saveEvent(eventInfo));
    } else {
      dispatch(EventActionCreator.createEvent(eventInfo));
    }
    this.setState({
      toggleEventModal: false,
      eventInfo: {},
    });
  }

  render() {
    const { classes, events: { data, status, activePage, totalPage } } = this.props;
    const { intl: { messages }, currentUser } = this.context;
    return (
      <div className={classes.eventPageContainer}>
        {
          status === ReduxStatus.loading && <Loader active />
        }
        {
          this.state.toggleEventModal &&
          <EventInfoModal
            isOpen={this.state.toggleEventModal}
            onClose={() => this.setState({ toggleEventModal: false })}
            onOk={this.handleSaveEvent}
            eventInfo={this.state.eventInfo}
          />
        }
        {
          this.state.toggleDeleteEvent &&
          <ConfirmModal
            isOpen={true}
            onClose={() => this.setState({ toggleDeleteEvent: false })}
            onConfirm={this.handleDeleteEvent}
            content={{
              mainText: messages.deleteText.replace('{item}', this.state.selectedDeleteEvent.name),
              description: messages.messageConfirm.replace('{name}',
                `${currentUser.first_name}`)
            }}
            title={messages.deleteEvent}
            cancelText={messages.keepThisEvent}
            confirmText={messages.delete}
          />
        }
        <div className={classes.eventSearch}>
          <Input
            icon='search'
            iconPosition='left'
            placeholder='Search event name...'
            onChange={this.onSearchFilterChange}
          />
        </div>
        <div className={classes.eventListPage}>
          <AddButton
            onClick={() => this.setState({
              toggleEventModal: true,
              eventInfo: {},
            })}
            content={messages.addEvent}
          />
          {
            data.map(item => (
              <EventCard
                key={item._id}
                event={item}
                onEdit={() => {
                  this.setState({
                    toggleEventModal: true,
                    eventInfo: item,
                  })
                }}
                onDuplicate={() => {
                  this.setState({
                    toggleEventModal: true,
                    eventInfo: {
                      ...item,
                      _id: null,
                    },
                  })
                }}
                onDelete={() => {
                  this.setState({
                    toggleDeleteEvent: true,
                    selectedDeleteEvent: item,
                  })
                }}
              />
            ))
          }
          {
            data.length > 0 && totalPage > 1 &&
            <Pagination
              activePage={activePage}
              totalPages={totalPage}
              onPageChange={this.handlePaginationChange}
            />
          }
        </div>
      </div>
    );
  }
}

UpcomingEvents.contextType = AppContext;

export default injectSheet(stylesheet)(UpcomingEvents);
