import { takeEvery, put, call, select, takeLatest, delay } from "redux-saga/effects";
import { toast } from 'react-toastify';
import { deleteClient, get, patch, post } from '../../Common/Utils/AxiosClient';
import Enum from '../../Common/Constants/Enum';
import {
  GET_UPCOMING_EVENTS,
  GET_UPCOMING_EVENTS_SUCCESS,
  GET_UPCOMING_EVENTS_FAILED,
  GET_PAST_EVENTS,
  CREATE_EVENT,
  SAVE_EVENT,
  SAVE_EVENT_FAILED,
  DELETE_EVENT,
  GET_EVENT_BY_ID,
  GET_EVENT_BY_ID_SUCCESS,
  GET_EVENT_BY_ID_FAILED,
  GET_EVENT_VOLUNTEERS,
  GET_EVENT_VOLUNTEERS_SUCCESS,
  GET_EVENT_VOLUNTEERS_FAILED,
  ADD_VOLUNTEER_TO_EVENT, REMOVE_VOLUNTEER_FROM_EVENT, SEARCH_VOLUNTEER_IN_EVENT, SEARCH_EVENT,
} from "../Actions/EventActionCreator";
import Utils from "../../Common/Utils/Utils";
import EventActionCreator from "../Actions/EventActionCreator";

function* watchGetEvents({ payload: { activePage, filter, opts } }) {
  try {
    const cleanObject = Utils.cleanObject(filter);
    const skip = (activePage - 1) * Enum.PerPage;
    const url = '/events';
    const params = {
      filter: cleanObject,
      limit: Enum.PerPage,
      skip,
    };
    const res = yield call(get, url, params);
    const totalPage = Utils.getTotalPage(res.data.metadata)
    yield put({
      type: opts.successAction || GET_UPCOMING_EVENTS_SUCCESS,
      payload: {
        activePage,
        filter: cleanObject,
        totalPage,
        data: res.data.data,
      }
    });
  } catch (e) {
    yield put({
      type: opts.failureAction || GET_UPCOMING_EVENTS_FAILED,
      payload: {
        ...e
      },
    });
  }
}

function* watchSearchEvent({ payload: { searchText, handleSearchResult } }) {
  try {
    yield delay(300);
    const url = '/events';
    const filter = {};
    if (searchText) {
      filter.name = { "$regex": searchText, "$options": "i" };
    }
    const params = {
      filter,
      limit: 20,
      skip: 0,
    };
    const res = yield call(get, url, params);
    handleSearchResult(res.data.data);
  } catch (e) {
  }
}

function* watchGetEvent({ payload: { id } }) {
  try {
    const url = `/events/${id}`;
    const res = yield call(get, url);
    yield put({ type: GET_EVENT_BY_ID_SUCCESS, payload: res.data.data });
  } catch (e) {
    yield put({
      type: GET_EVENT_BY_ID_FAILED,
      payload: {
        ...e
      },
    });
  }
}

function* watchCreateEvent({ payload: { event, isPastEvent } }) {
  try {
    const url = '/events';
    yield call(post, url, event);
    let eventList = yield select(state => state.events.upcomingEvents);
    if (isPastEvent) {
      eventList = yield select(state => state.events.pastEvents);
    }
    if (isPastEvent) {
      yield put(EventActionCreator.getPastEvents(eventList.activePage, eventList.filter))
    } else {
      yield put(EventActionCreator.getUpcomingEvents(eventList.activePage, eventList.filter))
    }
    toast.success(`Create event ${event.name} successful`);
  } catch (e) {
    toast.error(`Create event failed`);
  }
}

function* watchSaveEvent({ payload: { event, isPastEvent } }) {
  try {
    const url = `/events/${event._id}`;
    yield call(patch, url, event);
    let eventList = yield select(state => state.events.upcomingEvents);
    if (isPastEvent) {
      eventList = yield select(state => state.events.pastEvents);
    }
    if (isPastEvent) {
      yield put(EventActionCreator.getPastEvents(eventList.activePage, eventList.filter))
    } else {
      yield put(EventActionCreator.getUpcomingEvents(eventList.activePage, eventList.filter))
    }
    toast.success(`Update event ${event.name} successful`);
  } catch (e) {
    yield put({
      type: SAVE_EVENT_FAILED,
      payload: {
        ...e
      },
    });
    toast.error(`Update event failed`);
  }
}

function* watchDeleteEvent({ payload: { id, isPastEvent } }) {
  try {
    const url = `/events/${id}`;
    yield call(deleteClient, url);
    let eventList = yield select(state => state.events.upcomingEvents);
    if (isPastEvent) {
      eventList = yield select(state => state.events.pastEvents);
    }
    if (isPastEvent) {
      yield put(EventActionCreator.getPastEvents(eventList.activePage, eventList.filter))
    } else {
      yield put(EventActionCreator.getUpcomingEvents(eventList.activePage, eventList.filter))
    }
    toast.success(`Delete event successful`);
  } catch (e) {
    toast.error(`Delete event failed`);
  }
}

function* watchGetEventVolunteers({ payload: { eventId, activePage, filter } }) {
  try {
    const skip = (activePage - 1) * Enum.VolunteerPerPage;
    const url = `/events/${eventId}/volunteers`;
    const newFilter = {};
    if (filter["$text"]) {
      newFilter["$text"] = { "$search": filter["$text"] };
    }
    if (filter.assignmentStatus) {
      newFilter["events.event"] = eventId;
      newFilter["events.location"] = { "$exists": filter.assignmentStatus === 'assigned' ? "true" : "false" };
    } else {
      if (filter.selectedVotingDistrict && filter.selectedVotingDistrict._id) {
        newFilter.voting_district = filter.selectedVotingDistrict._id;
      }
      if (filter.selectedLocation && filter.selectedLocation._id) {
        newFilter["events.location"] = filter.selectedLocation._id;
      }
    }

    const params = {
      filter: newFilter,
      limit: Enum.VolunteerPerPage,
      skip,
      populate: { "path": "events.location", "populate": { "path": "voting_district" } },
    };
    const res = yield call(get, url, params);
    const totalPage = Utils.getTotalPage(res.data.metadata);
    yield put({
      type: GET_EVENT_VOLUNTEERS_SUCCESS,
      payload: {
        activePage,
        filter,
        totalPage,
        totalRecord: res.data.metadata.total,
        data: res.data.data,
      }
    });
  } catch (e) {
    yield put({
      type: GET_EVENT_VOLUNTEERS_FAILED,
      payload: {
        ...e
      },
    });
  }
}

function* watchSearchVolunteer({ payload: { eventId, searchText, handleSearchResult } }) {
  try {
    yield delay(300);
    const url = `/events/${eventId}/volunteers`;
    const filter = {};
    if (searchText) {
      filter["$text"] = { "$search": searchText };
    }
    const params = {
      filter,
      limit: 20,
      skip: 0,
    };
    const res = yield call(get, url, params);
    handleSearchResult(res.data.data);
  } catch
    (e) {
  }
}

function* watchAddVolunteerToEvent({ payload: { eventId, volunteerIds } }) {
  try {
    const url = `/events/${eventId}/volunteers`;
    yield call(post, url, volunteerIds);
    toast.success(`Add volunteer successful`);
    let eventVolunteersList = yield select(state => state.events.eventVolunteers);
    yield put(EventActionCreator.getEventVolunteers(eventId, eventVolunteersList.activePage, eventVolunteersList.filter));
  } catch (e) {
    toast.error(`Something went wrong`);
  }
}

function* watchRemoveVolunteerFromEvent({ payload: { eventId, volunteerId } }) {
  try {
    const url = `/events/${eventId}/volunteers/${volunteerId}`;
    yield call(deleteClient, url);
    toast.success(`Remove volunteer successful`);
    let eventVolunteersList = yield select(state => state.events.eventVolunteers);
    yield put(EventActionCreator.getEventVolunteers(eventId, eventVolunteersList.activePage, eventVolunteersList.filter));
  } catch (e) {
    toast.error(`Something went wrong`);
  }
}

function* EventSaga() {
  yield takeEvery(GET_UPCOMING_EVENTS, watchGetEvents);
  yield takeEvery(GET_PAST_EVENTS, watchGetEvents);
  yield takeLatest(SEARCH_EVENT, watchSearchEvent);
  yield takeEvery(GET_EVENT_BY_ID, watchGetEvent)
  yield takeEvery(CREATE_EVENT, watchCreateEvent);
  yield takeEvery(SAVE_EVENT, watchSaveEvent);
  yield takeEvery(DELETE_EVENT, watchDeleteEvent);
  yield takeEvery(GET_EVENT_VOLUNTEERS, watchGetEventVolunteers);
  yield takeEvery(ADD_VOLUNTEER_TO_EVENT, watchAddVolunteerToEvent);
  yield takeEvery(REMOVE_VOLUNTEER_FROM_EVENT, watchRemoveVolunteerFromEvent);
  yield takeLatest(SEARCH_VOLUNTEER_IN_EVENT, watchSearchVolunteer);
}

export default EventSaga;
