import React from 'react';
import PropTypes from 'prop-types';
import isequal from 'lodash.isequal';
import debounce from 'lodash.debounce';
import { Form, Input } from "semantic-ui-react";
import injectSheet from "react-jss";
import stylesheet from './stylesheet';
import AppContext from "../../../../Common/Constants/AppContext";
import DatePicker from 'react-datepicker/es';
import Enum from '../../../../Common/Constants/Enum';
import { config } from '../../../../configuration';

const FILTER_TYPE = {
  all: 'all',
  donation: 'donation',
  none: '',
}

class FinanceFilter extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      searchText: '',
      payment_type: '',
      payment_method: '',
      selectedVotingDistrict: {name: 'All'},
      status: '',
      partyBranch: '',
      state: '',
      receivedAfter: null,
      receivedBefore: null,

      ...props.initFilter,
    };
    this.handleChange = this.handleChange.bind(this);
    this.handleResetFilter = this.handleResetFilter.bind(this);
    this.handleSearch = debounce(this.handleSearch.bind(this), 300);
  }

  handleResetFilter() {
    const filter = {
      searchText: '',
      payment_type: '',
      payment_method: '',
      selectedVotingDistrict: {name: 'All'},
      status: '',
      partyBranch: '',
      state: '',
      receivedAfter: null,
      receivedBefore: null,
    };
    this.setState({
      ...filter,
    });
    const { onSearchFilterChange } = this.props;
    onSearchFilterChange(filter);
  }

  handleChange(e, { name, value }) {
    const filter = {
      ...this.state,
      [name]: value,
    };
    if (isequal(filter, this.state)) {
      return;
    }
    this.setState({
      ...filter,
    });
    this.handleSearch(filter);
  }

  handleChangeDate = ({name, value}) => {
    const filter = { ...this.state };
    if (value) {
      filter[name] = (new Date(value.getFullYear(), value.getMonth(), value.getDate(), value.getHours(), value.getMinutes(), 0)).getTime();
    }
    this.setState({...filter});
    this.handleSearch(filter);
  }

  handleSearch(filter) {
    const { onSearchFilterChange } = this.props;
    onSearchFilterChange(filter);
  }

  handleSelectElectorate = (selectedElectorate) => {
    const filter = {
      ...this.state,
      selectedVotingDistrict: selectedElectorate,
    };
    if (isequal(filter, this.state)) {
      return;
    }
    this.setState({
      ...filter,
    });

    const { onSearchFilterChange } = this.props;
    onSearchFilterChange(filter);
  }

  render() {
    const { classes, showNumber, total, filterType } = this.props;
    const { intl: { messages } } = this.context;
    const emptyFilter = {
      key: 'all',
      text: messages.all,
      value: '',
    };

    let PaymentTypeEnum = Enum.PaymentTypeEnum;
    if(filterType === FILTER_TYPE.donation) {
      PaymentTypeEnum = Enum.DonationTypeEnum;
    }
    const paymentTypes = Object.values(PaymentTypeEnum).map(item => ({
      key: item,
      text: messages[item],
      value: item
    }));
    paymentTypes.unshift(emptyFilter);

    const paymentMethods = Object.values(Enum.PaymentMethodEnum).map(item => ({
      key: item,
      text: messages[item],
      value: item
    }));
    paymentMethods.unshift(emptyFilter);

    const financeStatus = Object.values(Enum.FinanceStatus).map(item => ({
      key: item,
      text: messages[item],
      value: item
    }));
    financeStatus.unshift(emptyFilter);

    const partyBranches = Object.values(Enum.PartyBranches).map(item => ({
      key: item,
      text: item,
      value: item
    }));
    partyBranches.unshift(emptyFilter);

    const states = Object.keys(Enum.StateAustralia).map(item => ({
      key: item,
      text: Enum.StateAustralia[item],
      value: item
    }));
    states.unshift(emptyFilter);

    return (
      <div className={classes.searchFilter}>
        <div className='reset-filter'>
          <span onClick={this.handleResetFilter}>Reset filters</span>
          <img src={require('../../../../Assets/reset.png')} alt='' onClick={this.handleResetFilter} />
        </div>
        <div className='search-container'>
          <div className='search-box'>
            <Input
              icon='search'
              iconPosition='left'
              placeholder='Search...'
              name={'searchText'}
              value={this.state.searchText}
              onChange={this.handleChange}
            />
          </div>
          <div className='filter-box'>
            {
              filterType &&
              <div className='block-item'>
                <div className='text-title'>
                  {
                    filterType === FILTER_TYPE.all && 'Payment Type'
                  }
                  {
                    filterType === FILTER_TYPE.donation && 'Donation Type'
                  }
                </div>
                <div className='control'>
                  <Form.Select
                    options={paymentTypes}
                    placeholder={messages.all}
                    name='payment_type'
                    value={this.state.payment_type}
                    onChange={this.handleChange}
                  />
                </div>
              </div>
            }
            <div className='block-item'>
              <div className='text-title'>Payment Method</div>
              <div className='control'>
                <Form.Select
                  options={paymentMethods}
                  placeholder={messages.all}
                  name='payment_method'
                  value={this.state.payment_method}
                  onChange={this.handleChange}
                />
              </div>
            </div>
            {/* <div className='block-item'>
              <div className='text-title'>{messages.electorate}</div>
              <div className='control'>
                <ElectorateComponent
                  className='electorate'
                  onSelect={this.handleSelectElectorate}
                  defaultSelected={this.state.selectedVotingDistrict}
                  allowAll
                />
              </div>
            </div> */}

            <div className='block-item'>
              <div className='text-title'>Status</div>
              <div className='control'>
                <Form.Select
                  options={financeStatus}
                  placeholder={messages.all}
                  name='status'
                  value={this.state.status}
                  onChange={this.handleChange}
                />
              </div>
            </div>

            <div className='block-item dropdown'>
              <div className='text-title'>State</div>
              <div className='control'>
                <Form.Select
                  options={states}
                  placeholder={messages.all}
                  name='state'
                  value={this.state.state}
                  onChange={this.handleChange}
                />
              </div>
            </div>
            
            <div className='block-item date-picker'>
              <div className='text-title'>Received after</div>
              <Form>
                <Form.Field>
                  <DatePicker
                    selected={this.state.receivedAfter && new Date(this.state.receivedAfter)}
                    onChange={(value) => this.handleChangeDate({name: 'receivedAfter', value})}
                    dateFormat='dd/MM/yyyy'
                    placeholderText={config.DATE_FORMAT}
                    todayButton={'Today'}
                  />
                </Form.Field>
              </Form>
            </div>

            <div className='block-item date-picker'>
              <div className='text-title'>Received before</div>
              <Form>
                <Form.Field>
                  <DatePicker
                    selected={this.state.receivedBefore && new Date(this.state.receivedBefore)}
                    onChange={(value) => this.handleChangeDate({name: 'receivedBefore', value})}
                    dateFormat='dd/MM/yyyy'
                    placeholderText={config.DATE_FORMAT}
                    todayButton={'Today'}
                  />
                </Form.Field>
              </Form>
            </div>
          
          </div>
        </div>
        <div className='search-container'>
          
        </div>
        <div className='display-contact-number'>
          Showing <b>{showNumber}</b> of <b>{total}</b> payments
        </div>
      </div>
    )
  }
}

FinanceFilter.contextType = AppContext;

FinanceFilter.propTypes = {
  onSearchFilterChange: PropTypes.func.isRequired,
  initFilter: PropTypes.shape({}),
  showNumber: PropTypes.number,
  total: PropTypes.number,
  filterType: PropTypes.string,
};

FinanceFilter.defaultProps = {
  initFilter: {},
  showNumber: 0,
  total: 0,
  filterType: FILTER_TYPE.all,
};

export default injectSheet(stylesheet)(FinanceFilter);