import React from 'react'
import PropTypes from 'prop-types'
import injectSheet from 'react-jss'

import stylesheet from './stylesheet'

import { ContactFields, OperatorOptions, ValueDataOption } from './Config'
import AppContext from "../../../../../../../Common/Constants/AppContext"
import { Icon, Input, Select, Form } from "semantic-ui-react"
import DatePicker from 'react-datepicker/es'
import {config} from '../../../../../../../configuration'

class Segment extends React.Component {

  handleChangeField = (e, { name, value }) => {
    const { segment, handleChange } = this.props;
    handleChange({
      ...segment,
      field: value,
      operator: '',
    });
  }

  handleSelectChange = (e, { name, value }) => {
    const { segment, handleChange } = this.props;
    handleChange({
      ...segment,
      [name]: value,
      ...{value: name === 'operator' ? '' : value} // Clear input when changing operator
    });
  }

  renderOperator = () => {
    const { segment, isHitSave } = this.props;
    if (!segment.field) {
      return null;
    }
    const fieldInfo = ContactFields[segment.field];
    return (
      <Form.Field className='ui-form-field' error={isHitSave && !segment.operator}>
        <Select
          className={`operator-dropdown ${fieldInfo.type}-type`}
          options={OperatorOptions[fieldInfo.type]}
          name='operator'
          value={segment.operator}
          onChange={this.handleSelectChange}
        />
      </Form.Field>
    )
  }

  renderWidget = () => {
    const { segment, isHitSave } = this.props;
    const { operator } = segment;
    const { intl: { messages } } = this.context;
    if (!segment.field) {
      return null;
    }
    const fieldInfo = ContactFields[segment.field];
    switch (fieldInfo.type) {
      case 'dropdown':
        return (
          <Form.Field className='ui-form-field' error={isHitSave && !segment.value}>
            <Select
              className={`value-dropdown ${segment.field}-dropdown`}
              options={ValueDataOption(segment.field, messages)}
              name='value'
              value={segment.value}
              onChange={this.handleSelectChange}
            />
          </Form.Field>
        )
      case 'boolean':
        return (
          <Form.Field className='ui-form-field' error={isHitSave && (typeof segment.value !== "boolean" && !segment.value)}>
            <Select
              className={`value-dropdown ${segment.field}-dropdown`}
              options={ValueDataOption(segment.field, messages)}
              name='value'
              value={segment.value}
              onChange={this.handleSelectChange}
            />
          </Form.Field>
        )
      case 'date_fy':
        if(operator !== 'last_fiscal_year' && operator !== 'this_fiscal_year') {
          return (
            <Form.Field className='ui-form-field' error={isHitSave && !segment.value}>
              <DatePicker
                selected={segment.value ? new Date(segment.value) : null}
                onChange={(value) => this.handleSelectChange(null, {name: 'value', value})}
                dateFormat='dd/MM/yyyy'
                placeholderText={config.DATE_FORMAT}
                todayButton={'Today'}
              />
            </Form.Field>
          )
        }
        break;
      default:
        return (
          <Form.Field className='ui-form-field' error={isHitSave && !segment.value}>
            <Input
              name='value'
              value={segment.value}
              type={fieldInfo.type === 'number' ? 'number' : 'text'}
              onChange={this.handleSelectChange}
            />
          </Form.Field>
        );
    }
  }

  render() {
    const { classes, parentOperator, segment, length, index, handleRemoveSegment, isHitSave } = this.props;
    const { intl: { messages } } = this.context;

    const contactFields = Object.keys(ContactFields).map(key => ({
      key,
      text: messages[key],
      value: key,
    }));

    return (
      <div className={classes.segment}>
        <div className='parent-operator'>
          <span className={`text ${index === 0 ? 'hidden' : ''}`}>{parentOperator}</span>
        </div>
        <Form.Field className='ui-form-field' error={isHitSave && !segment.field}>
          <Select
            options={contactFields}
            value={segment.field}
            onChange={this.handleChangeField}
            className='field-dropdown'
            required
          />
        </Form.Field>
        {this.renderOperator()}
        {this.renderWidget()}
        {
          length > 1 &&
          <Icon
            name='remove'
            style={{ cursor: 'pointer' }}
            color='red'
            onClick={() => handleRemoveSegment(segment.id)}
          />
        }
      </div>
    );
  }
}

Segment.propType = {
  segment: PropTypes.shape({}).isRequired,
  handleChange: PropTypes.func.isRequired,
};

Segment.contextType = AppContext;

export default injectSheet(stylesheet)(Segment);