import React from 'react';
import PropTypes from 'prop-types';
import isequal from "lodash.isequal";
import { Button, Form, Message, Modal } from 'semantic-ui-react';
import injectSheet from "react-jss";
import AppContext from "../../../Common/Constants/AppContext";
import ReduxStatus from "../../../Common/Constants/ReduxStatus";
import Utils from '../../../Common/Utils/Utils';
import { ModalStyleSheet } from "../../../Common/GlobalStyle";
import Enum from "../../../Common/Constants/Enum";

class TeamMemberInfoModal extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      accountInfo: { ...props.accountInfo },
    };
    this.isFieldError = this.isFieldError.bind(this);
    this.handleChange = this.handleChange.bind(this);
    this.handleSubmit = this.handleSubmit.bind(this);
  }

  handleChange({ target: { name, value } }) {
    this.setState({
      accountInfo: {
        ...this.state.accountInfo,
        [name]: value,
      }
    })
  }

  handleSubmit() {
    if (this.isFieldError('email') ||
      this.isFieldError('first_name') ||
      this.isFieldError('last_name') ||
      this.isFieldError('group')
    ) {
      if (!this.state.isHitSave) {
        this.setState({
          isHitSave: true,
        });
      }
      return;
    }
    const { onOk } = this.props;
    onOk(this.state.accountInfo);
  }

  isFieldError(name) {
    const { intl: { messages } } = this.context;
    if (!this.state.accountInfo[name]) {
      return `${messages.pleaseEnter} ${messages[name]}`;
    }
    if (name === 'email') {
      if (!Utils.validateEmail(this.state.accountInfo[name])) {
        return 'Please input a valid email';
      }
      if(!this.state.accountInfo._id) {
        const { accountData } = this.props;
        const isExistEmail = accountData.map(item => item.email).includes(this.state.accountInfo.email);
        if(isExistEmail) {
          return 'This email is already used!';
        }
      }
    }
    return '';
  }

  render() {
    const { classes, isOpen, onClose, status, message } = this.props;
    const { intl: { messages } } = this.context;
    const { accountInfo } = this.state;
    const groups = Object.values(Enum.UserRole).map(item => ({
      key: item, text: messages[item], value: item,
    })) ;
    return (
      <Modal
        open={isOpen}
        onClose={onClose}
        size="mini"
      >
        <Modal.Header className={classes.header}>
          {
            accountInfo._id ? messages.updateTeamMember : messages.addTeamMember
          }
        </Modal.Header>
        <Modal.Content>
          <Form
            className={classes.teamMemberInfo}
            error={status === ReduxStatus.saveFailed}
            onSubmit={this.handleSubmit}
          >
            <Message
              error
              header={messages.failed}
              content={message}
            />
            <Form.Field required error={this.state.isHitSave && this.isFieldError('first_name')}>
              <label>{messages.firstName}</label>
              <input
                placeholder={messages.firstName}
                name='first_name'
                value={accountInfo.first_name}
                onChange={this.handleChange}
              />
              {
                this.state.isHitSave && this.isFieldError('first_name') &&
                <div className={classes.helpText}>{this.isFieldError('first_name')}</div>
              }
            </Form.Field>
            <Form.Field required error={this.state.isHitSave && this.isFieldError('last_name')}>
              <label>{messages.lastName}</label>
              <input
                placeholder={messages.lastName}
                name='last_name'
                value={accountInfo.last_name}
                onChange={this.handleChange}
              />
              {
                this.state.isHitSave && this.isFieldError('last_name') &&
                <div className={classes.helpText}>{this.isFieldError('last_name')}</div>
              }
            </Form.Field>
            <Form.Field required error={this.state.isHitSave && this.isFieldError('email')}>
              <label>{messages.email}</label>
              <input
                placeholder={messages.email}
                name='email'
                value={accountInfo.email}
                onChange={this.handleChange}
                disabled={accountInfo._id}
              />
              {
                this.state.isHitSave && this.isFieldError('email') &&
                <div className={classes.helpText}>{this.isFieldError('email')}</div>
              }
            </Form.Field>
            <Form.Field>
              <label>{messages.phone}</label>
              <input
                placeholder={`${messages.phone} (${messages.optional})`}
                name='phone'
                value={accountInfo.phone}
                onChange={this.handleChange}
              />
            </Form.Field>
            <Form.Field required error={this.state.isHitSave && this.isFieldError('group')}>
              <label>{messages.group}</label>
              <Form.Select
                options={groups}
                placeholder={messages.group}
                name='group'
                value={accountInfo.group}
                onChange={(e, target) => this.handleChange({ target: target })}
              />
              {
                this.state.isHitSave && this.isFieldError('group') &&
                <div className={classes.helpText}>{this.isFieldError('group')}</div>
              }
            </Form.Field>
            <Form.Field className={classes.actionGroup}>
              <Button onClick={onClose}>
                {messages.cancel}
              </Button>
              <Button
                type='submit'
                color='green'
                loading={status === ReduxStatus.saving}
                disabled={isequal(this.props.accountInfo, accountInfo)}
              >
                {
                  accountInfo._id ? messages.update : messages.add
                }
              </Button>
            </Form.Field>
          </Form>
        </Modal.Content>
      </Modal>
    )
  }
}

TeamMemberInfoModal.propTypes = {
  isOpen: PropTypes.bool.isRequired,
  onClose: PropTypes.func.isRequired,
  onOk: PropTypes.func.isRequired,
  accountInfo: PropTypes.shape({}).isRequired,
  accountData: PropTypes.shape([]).isRequired,
};

TeamMemberInfoModal.contextType = AppContext;

export default injectSheet(ModalStyleSheet)(TeamMemberInfoModal);