import React from 'react';
import PropTypes from 'prop-types';
import { Table, Loader, Pagination } from 'semantic-ui-react';
import AppContext from '../../../Common/Constants/AppContext';
import SettingsActionCreator from "../../SettingsActionCreator";
import ReduxStatus from "../../../Common/Constants/ReduxStatus";
import injectSheet from "react-jss";
import colors from "../../../Common/Constants/color";
import ConfirmModal from '../../../Common/Components/Modal/ConfirmModal';
import TeamMemberInfoModal from "./TeamMemberInfoModal";
import AddButton from '../../../Common/Components/Base/AddButton';
import Enum from "../../../Common/Constants/Enum";

const style = {
  teamSettingPage: {
    '& .user-action': {
      display: 'flex',
      justifyContent: 'flex-end',
      '& div': {
        marginRight: '3rem',
        cursor: 'pointer',
        color: colors.primaryBlue,
      }
    },
    '& .ui.pagination.menu': {
      marginLeft: '50%',
      transform: 'translateX(-50%)',
      maxWidth: '100%',
    },
  },
  profileImage: {
    width: 34,
    height: 34,
    objectFit: 'cover',
    borderRadius: '50%',
  },
};

class TeamSettings extends React.Component {
  constructor() {
    super();
    this.state = {
      toggleDeleteAccount: false,
      selectedDeleteAccount: null,
      toggleResetPassword: false,
      selectedResetPasswordAccount: null,
      toggleTeamMemberModal: false,
      accountInfo: {},
    };
    this.handleDeleteAccount = this.handleDeleteAccount.bind(this);
    this.handleResetPasswordAccount = this.handleResetPasswordAccount.bind(this);
    this.handleSaveAccount = this.handleSaveAccount.bind(this);
    this.handlePaginationChange = this.handlePaginationChange.bind(this);
  }

  componentDidMount() {
    const { dispatch } = this.context;
    dispatch(SettingsActionCreator.getAccount(1));
  }

  handlePaginationChange(e, { activePage }) {
    const { dispatch } = this.context;
    dispatch(SettingsActionCreator.getAccount(activePage));
  }

  handleDeleteAccount() {
    const { dispatch } = this.context;
    dispatch(SettingsActionCreator.deleteAccount(this.state.selectedDeleteAccount._id));
    this.setState({
      toggleDeleteAccount: false,
      selectedDeleteAccount: null,
    })
  }

  handleResetPasswordAccount() {
    const { dispatch } = this.context;
    dispatch(SettingsActionCreator.sendEmailResetPassword(this.state.selectedResetPasswordAccount.email));

    this.setState({
      toggleResetPassword: false,
      selectedResetPasswordAccount: null,
    })
  }

  handleSaveAccount(accountInfo) {
    const { dispatch } = this.context;
    if (accountInfo._id) {
      dispatch(SettingsActionCreator.saveAccount(accountInfo));
    } else {
      dispatch(SettingsActionCreator.createAccount(accountInfo));
    }
    this.setState({
      toggleTeamMemberModal: false,
      accountInfo: {},
    });
  }

  renderAccountAction = (item) => {
    const { intl: { messages }, userRole } = this.context;
    if (userRole.role === Enum.UserRole.owner) {
      return (
        <div className="user-action">
          <div onClick={() => this.setState({
            toggleDeleteAccount: true,
            selectedDeleteAccount: item,
          })}>
            {messages.delete}
          </div>
          <div onClick={() => this.setState({
            toggleTeamMemberModal: true,
            accountInfo: item,
          })}>
            {messages.edit}
          </div>
          <div onClick={() => this.setState({
            toggleResetPassword: true,
            selectedResetPasswordAccount: item,
          })}>
            {messages.resetPassword}
          </div>
        </div>
      )
    } else {
      return (
        <div className="user-action">
          {
            item.group !== Enum.UserRole.owner &&
            <div onClick={() => this.setState({
              toggleDeleteAccount: true,
              selectedDeleteAccount: item,
            })}>
              {messages.delete}
            </div>
          }
          {
            item.group !== Enum.UserRole.owner &&
            <div onClick={() => this.setState({
              toggleTeamMemberModal: true,
              accountInfo: item,
            })}>
              {messages.edit}
            </div>
          }
          <div onClick={() => this.setState({
            toggleResetPassword: true,
            selectedResetPasswordAccount: item,
          })}>
            {messages.resetPassword}
          </div>
        </div>
      )
    }
  }

  render() {
    const { classes, team: { data, status, activePage, totalPage } } = this.props;
    const { intl: { messages }, currentUser } = this.context;
    return (
      <div className={classes.teamSettingPage}>
        {
          this.state.toggleDeleteAccount &&
          <ConfirmModal
            isOpen={this.state.toggleDeleteAccount}
            onClose={() => this.setState({ toggleDeleteAccount: false })}
            onConfirm={this.handleDeleteAccount}
            content={{
              mainText: messages.deleteAccountText.replace('{email}',
                `${this.state.selectedDeleteAccount.email}`),
              description: messages.messageConfirm.replace('{name}',
                `${currentUser.first_name}`)
            }}
            title={messages.deleteTeamMember}
            cancelText={messages.keepThisTeamMember}
            confirmText={messages.delete}
          />
        }
        {
          this.state.toggleResetPassword &&
          <ConfirmModal
            isOpen={this.state.toggleResetPassword}
            onClose={() => this.setState({ toggleResetPassword: false })}
            onConfirm={this.handleResetPasswordAccount}
            content={{
              mainText: messages.resetPasswordAccountText.replace('{0}',
                `${this.state.selectedResetPasswordAccount.email}`),
              description: messages.messageConfirm.replace('{name}',
                `${currentUser.first_name}`)
            }}
            title={messages.resetPassword}
            cancelText={messages.no}
            confirmText={messages.yes}
          />
        }
        {
          this.state.toggleTeamMemberModal &&
          <TeamMemberInfoModal
            isOpen={this.state.toggleTeamMemberModal}
            onClose={() => this.setState({ toggleTeamMemberModal: false })}
            onOk={this.handleSaveAccount}
            accountInfo={this.state.accountInfo}
            accountData={data}
          />
        }
        {
          status === ReduxStatus.loading && <Loader active />
        }
        <AddButton
          onClick={() => this.setState({
            toggleTeamMemberModal: true,
            accountInfo: {},
          })}
          content={messages.addTeamMember}
        />
        <Table singleLine selectable basic>
          <Table.Header>
            <Table.Row>
              <Table.HeaderCell></Table.HeaderCell>
              <Table.HeaderCell>{messages.name}</Table.HeaderCell>
              <Table.HeaderCell>{messages.mobile}</Table.HeaderCell>
              <Table.HeaderCell>{messages.email}</Table.HeaderCell>
              <Table.HeaderCell>{messages.group}</Table.HeaderCell>
              <Table.HeaderCell width={6}></Table.HeaderCell>
            </Table.Row>
          </Table.Header>
          <Table.Body>
            {data.map(item => (
              <Table.Row key={item._id}>
                <Table.Cell collapsing>
                  {item.profile_picture && item.profile_picture.original ?
                    <img className={classes.profileImage} src={item.profile_picture.original} alt="" />
                    : <img className={classes.profileImage} src={require("../../../Assets/avatar.png")} alt='' />
                  }
                </Table.Cell>
                <Table.Cell>{`${item.first_name} ${item.last_name}`}</Table.Cell>
                <Table.Cell>{item.phone}</Table.Cell>
                <Table.Cell>
                  <a href={`mailto: ${item.email}`} target="_top">{item.email}</a>
                </Table.Cell>
                <Table.Cell>{messages[item.group]}</Table.Cell>
                <Table.Cell>
                  {
                    this.renderAccountAction(item)
                  }
                </Table.Cell>
              </Table.Row>
            ))}
          </Table.Body>
        </Table>
        {
          data.length > 0 && totalPage > 1 &&
          <Pagination
            activePage={activePage}
            totalPages={totalPage}
            onPageChange={this.handlePaginationChange}
          />
        }
      </div>
    )
  }
}

TeamSettings.contextType = AppContext;

TeamSettings.propTypes = {
  team: PropTypes.shape({}).isRequired,
}

export default injectSheet(style)(TeamSettings);