import { takeEvery, takeLatest, put, call, select } from "redux-saga/effects";
import omit from 'lodash.omit';
import { toast } from "react-toastify";
import { get, deleteClient, post, patch } from '../Common/Utils/AxiosClient';
import SettingsActionCreator, {
  CREATE_ACCOUNT, CREATE_ACCOUNT_FAILED, 
  DELETE_ACCOUNT, DELETE_ACCOUNT_FAILED,
  GET_ACCOUNT, GET_ACCOUNT_FAILED, GET_ACCOUNT_SUCCESS, 
  SAVE_ACCOUNT, SAVE_ACCOUNT_FAILED,
  RESET_PASSWORD,
  RESET_CONTACTED_FOR_NEXT_ELECTION,
  SET_DEFAULT_ADMIN,
  GET_DEFAULT_ADMIN,
  GET_DEFAULT_ACCOUNT_SUCCESS,
  GET_MEMBERSHIPS, GET_MEMBERSHIPS_SUCCESS, GET_MEMBERSHIPS_FAILURE,
  SET_AUTO_EMAILS_STATUS,
  GET_AUTO_EMAILS_STATUS,
  GET_AUTO_EMAILS_STATUS_SUCCESS,
  GET_CONTACT_TITLE_SUCCESS,
  GET_CONTACT_TITLE,
  CREATE_CONTACT_TITLE,
  DELETE_CONTACT_TITLE,
  DELETE_CONTACT_TITLE_SUCCESS,
  CREATE_CONTACT_TITLE_SUCCESS
} from "./SettingsActionCreator";
import Enum from "../Common/Constants/Enum";
import Utils from "../Common/Utils/Utils";

function* watchGetAccount({ payload: { activePage, filter } }) {
  try {
    const skip = (activePage - 1) * Enum.TeamSettingPerPage;
    const url = `/accounts`;
    const params = {
      filter,
      limit: Enum.TeamSettingPerPage,
      skip,
      sort: {first_name: 1}
    }
    const res = yield call(get, url, params);
    const totalPage = Utils.getTotalPage(res.data.metadata);
    yield put({
      type: GET_ACCOUNT_SUCCESS,
      payload: {
        activePage,
        filter,
        totalPage,
        totalRecord: res.data.metadata.total,
        data: res.data.data,
      }
    });
  } catch (e) {
    yield put({
      type: GET_ACCOUNT_FAILED,
      payload: {
        ...e
      },
    });
  }
}

function* watchDeleteAccount({ payload: { id } }) {
  try {
    const url = `/accounts/${id}`;
    yield call(deleteClient, url);
    let teamSettingList = yield select(state => state.settings.team);
    yield put(SettingsActionCreator.getAccount(teamSettingList.activePage));
  } catch (e) {
    yield put({
      type: DELETE_ACCOUNT_FAILED,
      payload: {
        ...e
      },
    });
  }
}

function* watchCreateAccount({ payload: { account } }) {
  try {
    const url = '/accounts';
    yield call(post, url, account);
    let teamSettingList = yield select(state => state.settings.team);
    yield put(SettingsActionCreator.getAccount(teamSettingList.activePage));
  } catch (e) {
    yield put({
      type: CREATE_ACCOUNT_FAILED,
      payload: {
        ...e
      },
    });
  }
}

function* watchSaveAccount({ payload: { account } }) {
  try {
    const url = `/accounts/${account._id}`;
    yield call(patch, url, omit(account, ['profile_picture']));
    let teamSettingList = yield select(state => state.settings.team);
    yield put(SettingsActionCreator.getAccount(teamSettingList.activePage));
  } catch (e) {
    yield put({
      type: SAVE_ACCOUNT_FAILED,
      payload: e,
    });
  }
}

function* watchResetPassword({ payload: { email } }) {
  try {
    const url = `/authentication/send-password-reset-link?email=${email}`;
    yield call(post, url, {});
    toast.success('Send email reset password successful');
  } catch (e) {
    toast.error('Send email reset password failed');
  }
}

function* watchResetContactedForNextElection() {
  try {
    const url = `/contacts/reset-contacted-for-next-election`;
    yield call(post, url, {});
    toast.success('Reset successful');
  } catch (e) {
    toast.error('Reset failed');
  }
}

function* watchSetDefaultAdmin({ payload: { accountId } }) {
  try {
    const url = `/accounts/set-default/${accountId}`;
    yield call(post, url, {});
    toast.success('Set default admin successful');
  } catch (error) {
    toast.error('Failed to set default admin')
  }
}

function* watchGetDefaultAdmin() {
  try {
    const url = `/accounts`;
    const params = {
      filter: {isMembershipPointPerson: true}
    }
    const res = yield call(get, url, params);
    
    yield put({
      type: GET_DEFAULT_ACCOUNT_SUCCESS,
      payload: {
        data: res.data.data[0],
      }
    });
  } catch (e) {
    console.log('watchGetDefaultAdmin error', e.message)
  }
}

function* watchGetMemberships({ payload: { activePage, filter } }) {
  try {
    const skip = (activePage - 1) * Enum.MembershipSettingPerPage
    const url = `/memberships`;
    const params = {
      filter,
      limit: Enum.MembershipSettingPerPage,
      skip,
      sort: {state: 1}
    }
    const res = yield call(get, url, params);
    const totalPage = Utils.getTotalPage(res.data.metadata);
    yield put({
      type: GET_MEMBERSHIPS_SUCCESS,
      payload: {
        activePage,
        filter,
        totalPage,
        totalRecord: res.data.metadata.total,
        data: res.data.data,
      }
    });
  } catch (e) {
    yield put({
      type: GET_MEMBERSHIPS_FAILURE,
      payload: {
        ...e
      },
    });
  }
}

function* watchSetAutoEmailStatus({ payload: { isAutoEmail } }) {
  try {
    const url = `/general-settings/update-auto-emails`;
    yield call(post, url, {auto_emails: isAutoEmail});
    
    yield put({
      type: GET_AUTO_EMAILS_STATUS_SUCCESS,
      payload: {
        data: isAutoEmail,
      }
    });
    toast.success('Set auto email status successful');
  } catch (error) {
    toast.error('Failed to set auto email status')
  }
}

function* watchGetAutoEmailStatus() {
  try {
    const url = `/general-settings/auto-emails-status`;
    const res = yield call(get, url);
    yield put({
      type: GET_AUTO_EMAILS_STATUS_SUCCESS,
      payload: {
        data: res.data.data,
      }
    });
  } catch (error) {
    console.log(error.message)
  }
}

function* watchGetContactTitle() {
  try {
    const url = `contact-title`;
    const res = yield call(get, url);
    yield put({
      type: GET_CONTACT_TITLE_SUCCESS,
      payload: {
        data: res.data.data,
      }
    });
  } catch (error) {
    console.log(error.message)
  }
}

function* watchCreateContactTitle({payload: {title}}) {
  try {
    const url = `contact-title`;
    const res = yield call(post, url, {title});
    yield put({
      type: CREATE_CONTACT_TITLE_SUCCESS,
      payload: {
        data: res.data.data,
      }
    });
    toast.success('Title option added successfully')
    return res
  } catch (error) {
    toast.error('Title option already exists')
    console.log(error.message)
  }
}

function* watchDeleteContactTitle({payload: {id}}) {
  try {
    const url = `contact-title/${id}`;
    const result = yield call(deleteClient, url);
    yield put({
      type: DELETE_CONTACT_TITLE_SUCCESS,
      payload: {id}
    });
    toast.success('Title option archived successfully')
    return result
  } catch (error) {
    console.log(error.message)
  }
}

function* SettingsSaga() {
  yield takeEvery(GET_ACCOUNT, watchGetAccount)
  yield takeEvery(DELETE_ACCOUNT, watchDeleteAccount)
  yield takeEvery(CREATE_ACCOUNT, watchCreateAccount)
  yield takeEvery(SAVE_ACCOUNT, watchSaveAccount)
  yield takeEvery(RESET_PASSWORD, watchResetPassword)
  yield takeEvery(RESET_CONTACTED_FOR_NEXT_ELECTION, watchResetContactedForNextElection)
  yield takeEvery(SET_DEFAULT_ADMIN, watchSetDefaultAdmin)
  yield takeEvery(GET_DEFAULT_ADMIN, watchGetDefaultAdmin)
  yield takeLatest(GET_MEMBERSHIPS, watchGetMemberships)
  yield takeLatest(SET_AUTO_EMAILS_STATUS, watchSetAutoEmailStatus)
  yield takeEvery(GET_AUTO_EMAILS_STATUS, watchGetAutoEmailStatus)
  yield takeLatest(GET_CONTACT_TITLE, watchGetContactTitle)
  yield takeLatest(CREATE_CONTACT_TITLE, watchCreateContactTitle)
  yield takeLatest(DELETE_CONTACT_TITLE, watchDeleteContactTitle)
}

export default SettingsSaga;
