import React from "react";
import injectSheet from "react-jss";
import { connect } from "react-redux";
import { Link } from "react-router-dom";

import stylesheet from "./stylesheet";
import ElectoralRollFilter from "./ElectoralFilter/ElectoralRollFilter";
import { Table, Pagination, Loader } from "semantic-ui-react";
import ImportButton from "../../../Common/Components/Base/ImportButton";

import ToolsActionCreator from "../../ToolsActionCreator";
import ReduxStatus from "../../../Common/Constants/ReduxStatus";
import AppContext from "../../../Common/Constants/AppContext";
import Utils from "../../../Common/Utils/Utils";
import { post } from "../../../Common/Utils/AxiosClient";
import FileUpload from "../../../Common/Components/FileUpload/FileUpload";

import ImportModal from "./ImportModal";
import moment from "moment";

class ElectoralRoll extends React.Component {
  isFirstUploaded = false;

  state = {
    isDeleting: false,
  };

  componentDidMount() {
    const { dispatch } = this.context;
    const {
      tools: {
        electoralRolls: { activePage, filter },
      },
    } = this.props;
    dispatch(ToolsActionCreator.getElectoralRolls(activePage, filter));
  }

  handlePaginationChange = (e, { activePage }) => {
    const { dispatch } = this.context;
    const {
      tools: {
        electoralRolls: { filter },
      },
    } = this.props;
    dispatch(ToolsActionCreator.getElectoralRolls(activePage, filter));
  };

  onSearchFilterChange = (filter) => {
    const { dispatch } = this.context;
    dispatch(ToolsActionCreator.getElectoralRolls(1, filter));
  };

  handleUploadCSV = (files) => {
    const { dispatch } = this.context;
    const [file] = files;
    if (!file) return;
    dispatch(
      ToolsActionCreator.uploadFileElectoralList(
        file,
        this.state.isRemoveAllVoter
      )
    );
  };

  handleCloseImportModal = () => {
    const { dispatch } = this.context;
    this.setState({ toggleImportModal: false, isRemoveAllVoter: false });
    dispatch(ToolsActionCreator.resetFileElectoralListStatus());
  };

  handleOnGoToImportList = () => {
    const { history } = this.props;
    history.push(`/tools/importHistory`);
  };

  onUploadSuccess = ({ fileUrl, fileName }) =>
    post("/electoral-rolls/import-data", {
      fileUrl,
      fileName,
    });

  removeAllVoters = async () => {
    const { dispatch } = this.context;
    this.setState({ isDeleting: true });
    await dispatch(ToolsActionCreator.removeAllVoters());
    await post("/electoral-rolls/delete-all");
    this.setState({ isDeleting: false }); 
    await dispatch(ToolsActionCreator.removeAllVotersSuccess());
  };

  setImportModalRef = (ref) => {
    this.importModalRef = ref;
  };

  render() {
    const { classes, tools } = this.props;
    const { isDeleting } = this.state;
    const {
      electoralRolls: {
        data,
        status,
        activePage,
        totalPage,
        totalRecord,
        filter,
      },
    } = tools;
    const {
      intl: { messages },
    } = this.context;
    return (
      <div className={classes.toolElectoralRoll}>
        {status === ReduxStatus.loading && <Loader active />}
        <ElectoralRollFilter
          initFilter={filter}
          onSearchFilterChange={this.onSearchFilterChange}
          showNumber={data.length}
          total={totalRecord}
        />
        <div className="tool-electoral-content">
          <Table singleLine selectable basic>
            <Table.Header>
              <Table.Row>
                <Table.HeaderCell>Given Names</Table.HeaderCell>
                <Table.HeaderCell>Last Name</Table.HeaderCell>
                <Table.HeaderCell>Linked Profile</Table.HeaderCell>
                <Table.HeaderCell>Address</Table.HeaderCell>
                <Table.HeaderCell>DOB</Table.HeaderCell>
                <Table.HeaderCell>NSW Voter ID</Table.HeaderCell>
              </Table.Row>
            </Table.Header>
            <Table.Body>
              {data.map((item) => (
                <Table.Row key={item._id} className="contact-row">
                  <Table.Cell>{item.given_name}</Table.Cell>
                  <Table.Cell>{item.last_name}</Table.Cell>
                  <Table.Cell>
                  { 
                    item.linked_profile ? <Link to={`/contact-detail/${item.linked_profile._id}/overview`}>{Utils.combineName(item.linked_profile, messages)}</Link> : "-"
                  }
                  </Table.Cell>
                  <Table.Cell>{Utils.combineAddress(item.address)}</Table.Cell>
                  <Table.Cell>{item.day_of_birth ? moment(item.day_of_birth).format('DD/MM/YYYY') : ''}</Table.Cell>
                  <Table.Cell>{item.voter_id}</Table.Cell>
                </Table.Row>
              ))}
            </Table.Body>
          </Table>

          <ImportButton
            onClick={() => this.importModalRef.openModal()}
            content="Import"
          />
          {
            <FileUpload onUploadSuccess={this.onUploadSuccess}>
              {(props) => (
                <ImportModal
                  loading={isDeleting}
                  onUpload={props.onUploadFiles}
                  ref={this.setImportModalRef}
                  removeAllVoters={this.removeAllVoters}
                />
              )}
            </FileUpload>
          }
          {data.length > 0 && totalPage > 1 && (
            <Pagination
              activePage={activePage}
              totalPages={totalPage}
              onPageChange={this.handlePaginationChange}
            />
          )}
        </div>
      </div>
    );
  }
}

const mapStateToProps = ({ tools }) => ({
  tools,
});

ElectoralRoll.contextType = AppContext;
export default connect(mapStateToProps)(injectSheet(stylesheet)(ElectoralRoll));
