import { put, call, takeEvery, takeLatest, select } from "redux-saga/effects";
import { toast } from "react-toastify";
import { get, post, deleteClient } from '../Common/Utils/AxiosClient';
import Enum from '../Common/Constants/Enum';
import ToolsActionCreator, {
  DOWNLOAD_RECEIPT, SEND_EMAIL_RECEIPT, SEND_EMAIL_LETTER,
  GET_LETTERS, GET_LETTERS_SUCCESS, GET_LETTERS_FAILURE, 
  GENERATE_LETTERS,
  DOWNLOAD_LETTER,
  GET_PRINTINGS, GET_PRINTINGS_SUCCESS, GET_PRINTINGS_FAILURE,
  DELETE_PRINTING,
  GET_ELECTORAL_ROLLS, GET_ELECTORAL_ROLLS_SUCCESS, GET_ELECTORAL_ROLLS_FAILURE,
  GET_IMPORT_HISTORY, GET_IMPORT_HISTORY_SUCCESS, GET_IMPORT_HISTORY_FAILURE
} from "./ToolsActionCreator";
import Utils from "../Common/Utils/Utils";

function* watchSendEmailReceipt({ payload: { data: { payment_id, email } } }) {
  try {
    const url = `/send-email-receipt?payment_id=${payment_id}&email=${email}`;
    yield call(post, url);
    toast.success('Send email successfully');
  } catch (e) {
    toast.error('failed to send email');
  }
}

function* watchSendEmailLetter({ payload: { data: { payment_id, email } } }) {
  try {
    const url = `/send-email-letter?payment_id=${payment_id}&email=${email}`;
    yield call(post, url);
    toast.success('Send email successfully');
  } catch (e) {
    toast.error('failed to send email');
  }
}

function* watchDownloadReceipt({ payload: { payment_id } }) {
  try {
    const url = '/export-receipt-to-pdf';
    const params = {
      payment_id,
    };
    const config = {
      responseType: 'arraybuffer',
      headers: {
        'Content-Type': 'application/pdf',
      }
    };
    const res = yield call(get, url, params, config);
    Utils.downloadFile(res.data, `receipt_${(new Date()).toLocaleDateString("en-AU")}.pdf`);
  } catch (e) {
    toast.error('Failed to download receipt');
  }
}

function* watchGetLetters({ payload: { filter } }) {
  try {
    const url = '/tools/letters';
    const res = yield call(get, url, { filter });
    yield put({
      type: GET_LETTERS_SUCCESS,
      payload: {
        filter: filter,
        data: res.data.data,
      }
    });
  } catch (e) {
    yield put({
      type: GET_LETTERS_FAILURE,
      payload: {
        ...e
      },
    });
    toast.error(`Failed to get letters`);
  }
}

function* watchGenerateLetters({ payload: { filter, opts } }) {
  try {
    toast.info(`Downloading PDF ...`, { toastId: 'downloadPDFNotification', autoClose: false });
    const cleanObject = Utils.cleanObject(filter);
    const letterFilter = encodeURIComponent(JSON.stringify(cleanObject));
    const url = `/tools/generate-letters?filter=${letterFilter}`;
    const params = {
      ...opts,
    };
    const config = {
      responseType: 'blob',
    };
    const res = yield call(get, url, params, config);
    const { type, state } = filter;
    let renameType = ''
    if(type === 'membership_expiring') renameType = 'membership_renewal'
    Utils.downloadFile(res.data, `${state}_${renameType || type}_${(new Date()).toLocaleDateString("en-AU")}.pdf`);
    const letters = yield select(state => state.tools.letters);
    yield put(ToolsActionCreator.getLetters(letters.filter))
    toast.dismiss('downloadPDFNotification');
  } catch (e) {
    toast.dismiss('downloadPDFNotification');
    toast.error(`Failed to download PDF`);
  }
}

function* watchDownLoadLetter({ payload: { paymentId, type, opts } }) {
  try {
    toast.info(`Downloading PDF ...`, { toastId: 'downloadPDFNotification', autoClose: false });
    const url = `/tools/download-letter?payment_id=${paymentId}&type=${type}`;
    const params = {
      ...opts,
    }
    const config = {
      responseType: 'blob',
    }
    const res = yield call(get, url, params, config)
    Utils.downloadFile(res.data, `${type}_${(new Date()).toLocaleDateString("en-AU")}.pdf`);
    toast.dismiss('downloadPDFNotification');
  } catch (e) {
    toast.dismiss('downloadPDFNotification');
    toast.error(`Failed to download PDF`);
  }
}

function* watchGetPrintings({ payload: { filter, activePage } }) {
  try {
    const skip = (activePage - 1) * Enum.PerPage;
    const params = {
      filter,
      limit: Enum.PerPage,
      skip,
      sort: { "created_at": -1}
    };
    const url = '/printings';
    const res = yield call(get, url, params);
    const totalPage = Utils.getTotalPage(res.data.metadata);
    yield put({
      type: GET_PRINTINGS_SUCCESS,
      payload: {
        filter: filter,
        data: res.data.data,
        activePage,
        totalPage,
        totalRecord: res.data.metadata.total,
      }
    });
  } catch (e) {
    yield put({
      type: GET_PRINTINGS_FAILURE,
      payload: {
        ...e
      },
    });
    toast.error(`Failed to get printings`);
  }
}

function* watchDeletePrinting({ payload: { id, opts } }) {
  try {
    const url = `/printings/${id}`;
    yield call(deleteClient, url);
    toast.success(`Delete printing successful`);
    let printings = yield select(state => state.tools.printings);
    yield put(ToolsActionCreator.getPrintings(printings.activePage, printings.filter));
  } catch (e) {
    toast.error(`Failed to delete printing`);
  }
}

function* watchGetElectoralRolls({ payload: { filter, activePage } }) {
  try {
    const skip = (activePage - 1) * Enum.PerPage
    let newFilter = filter
    if(filter.searchText) {
      newFilter["$text"] = { "$search": filter.searchText };
      delete newFilter.searchText
    }
    const params = {
      filter: newFilter,
      limit: Enum.PerPage,
      skip
    };
    const url = '/electoral-rolls';
    const res = yield call(get, url, params);
    const totalPage = Utils.getTotalPage(res.data.metadata);
    yield put({
      type: GET_ELECTORAL_ROLLS_SUCCESS,
      payload: {
        filter: filter,
        data: res.data.data,
        activePage,
        totalPage,
        totalRecord: res.data.metadata.total,
      }
    });
  } catch (e) {
    yield put({
      type: GET_ELECTORAL_ROLLS_FAILURE,
      payload: {
        ...e
      },
    });
    toast.error(`Failed to get Electoral Rolls`);
  }
}

function* watchGetImportHistory({ payload: { filter, activePage } }) {
  try {
    const skip = (activePage - 1) * Enum.PerPage
    let newFilter = filter
    const params = {
      filter: newFilter,
      limit: Enum.PerPage,
      skip,
      sort: {"created_at": -1}
    };
    const url = '/import-histories';
    const res = yield call(get, url, params);
    const totalPage = Utils.getTotalPage(res.data.metadata);
    yield put({
      type: GET_IMPORT_HISTORY_SUCCESS,
      payload: {
        filter: filter,
        data: res.data.data,
        activePage,
        totalPage,
        totalRecord: res.data.metadata.total,
      }
    });
  } catch (e) {
    yield put({
      type: GET_IMPORT_HISTORY_FAILURE,
      payload: {
        ...e
      },
    });
    toast.error(`Failed to get Import History`);
  }
}

function* ToolsSaga() {
  yield takeEvery(SEND_EMAIL_RECEIPT, watchSendEmailReceipt)
  yield takeEvery(SEND_EMAIL_LETTER, watchSendEmailLetter)
  yield takeEvery(DOWNLOAD_RECEIPT, watchDownloadReceipt)
  yield takeLatest(GET_LETTERS, watchGetLetters)
  yield takeEvery(GENERATE_LETTERS, watchGenerateLetters)
  yield takeEvery(DOWNLOAD_LETTER, watchDownLoadLetter)
  yield takeLatest(GET_PRINTINGS, watchGetPrintings)
  yield takeEvery(DELETE_PRINTING, watchDeletePrinting)
  yield takeLatest(GET_ELECTORAL_ROLLS, watchGetElectoralRolls)
  yield takeLatest(GET_IMPORT_HISTORY, watchGetImportHistory)
}

export default ToolsSaga;
